#ifndef __ANIMATED_MODEL_CONTAINER_H
#define __ANIMATED_MODEL_CONTAINER_H

#include "QSqlQueryModelFetchAll.h"
#include "PostfixTableRowQuery.h"

class AnimatedModelContainer;

class AnimatedModel
{
public:
	AnimatedModel();
	virtual ~AnimatedModel();
	void SetParent(AnimatedModelContainer *amc);

	virtual void SetAnimationRowIndices(int from, int to, double valueBefore, double valueAfter);
	virtual void ClearAnimationRowIndices(void);

	bool UseAnimationRowIndices(void) const {return useAnimationRowIndices==true && animationRowIndexTo>=0;}

	// Call InitDB before using the class! InitDB can fail with an invalid query. Failure message is returned if that is the case.
	// This is in a separate function rather than the constructor so we can prompt the user if the query fails.
	QString InitDB(QString queryString, QString postfixTickDefinition, QString databaseName);

	QSqlQueryModelFetchAll* GetModel(void) {return &model;}
	static AnimatedModel *GetLinkedList(void);
	// This changes if the query changes.
	int GetCurrentRowIndex(void) const {return currentRowIndex;}
	int GetLastRowIndex(void) const {return lastRowIndex;}
	void SetNextRowIndex(int ri) {lastRowIndex=currentRowIndex; currentRowIndex=ri;}
	bool GetDoubleAtIndex(int idx, QString columnName, double *d);
	bool GetDoubleAtIndex(int idx, int columnIndex, double *d);
	bool GetIndexOfValue(int *idxBefore, int *idxAfter, double *valueBefore, double *valueAfter, double valueIn, QString columnName);
	
	int GetAnimationFromIndex(void) const {return animationRowIndexFrom;}
	int GetAnimationToIndex(void) const {return animationRowIndexTo;}
	void GetAnimationRowIndices(int *rowStart, int *rowEnd, int fullTableRowConstraint);
	bool CanRowStep(void) const {return hasValidQuery && tableRowQuery.CanRowStep();}
	int GetNextRowStep(int currentIndex, bool forward);
	QString GetQuery(void) const {return model.query().lastQuery();}
	QString GetTickDefinition(void) const {return tableRowQuery.GetTickDefinition();}
	void SetTickDefinition(QString postfixTickDefinition);
	QString GetDBName(void) const {return dbName;}

	void AMC_Save(XMLSaver &xmlSaver);
	void AMC_Load(XMLSaver &xmlSaver);

	PostfixTableRowQuery tableRowQuery;
	AnimatedModel *next;
protected:
	// Advance rows while each column has an equal value
	int AdvanceWhileEqual(int nextRow, int columnIndex, int rowCount, int step, double valueToCheck);
	QSqlQueryModelFetchAll model;
	bool hasValidQuery;
	int lastRowIndex, currentRowIndex;
	int animationRowIndexFrom, animationRowIndexTo;
	bool useAnimationRowIndices;
	QString dbName;
};

class AnimatedModelList
{
public:
	virtual AnimatedModel* GetAnimatedModelAtIndex(int index)=0;
	virtual int GetAMCCount(void) const=0;
};


#endif // __ANIMATED_MODEL_CONTAINER_H
