#include "ByteArrayAsImageItemDelegate.h"
#include <QPainter>
#include <QBuffer>
#include <QImageReader>
#include <QAbstractItemView>
#include <QPixmapCache>
#include "SuperFastHash.h"

void ByteArrayAsImageItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
				   const QModelIndex &index) const
{
	const QVariant &v = index.data();
	if (v.type()==QVariant::ByteArray)
	{
		QByteArray ba = v.toByteArray();
		unsigned int hash = SuperFastHash(ba.data(),ba.length());
		QString hashKey;
		// Safer collision code commented out, in case I need it
		//hashKey.sprintf("%i%i%i",index.row(),index.column(),hash);
		// Faster, in case images are duplicated
		hashKey.sprintf("%i",hash);
		QPixmap pixmap;
		bool foundPixmap = QPixmapCache::find(hashKey, pixmap);
		bool paintLater=false;
		if (foundPixmap==false)
		{
			if (deferredPainter)
				paintLater=deferredPainter->paintLater(index);
			else
				paintLater=false;
			if (paintLater==false)
			{
				// Load pixmap
				QBuffer buffer;
				buffer.setBuffer(&ba);
				buffer.open(QIODevice::ReadOnly);
				QImageReader qir;
				qir.setDevice(&buffer);
				QImage image;
				if (qir.read(&image))
				{
					pixmap = QPixmap::fromImage(image);

					// Save to cache
					QPixmapCache::insert(hashKey,pixmap);
				}
				else
				{
					// Not an image?
					QItemDelegate::paint(painter,option,index);
					return;
				}
			}
		}

		if (foundPixmap==true || paintLater==false)
		{
			// Paint
			QSize imageSize = pixmap.size();
			QRect drawRect;
			float scaleWidth, scaleHeight;
			float drawWidth, drawHeight;
			scaleWidth = (float) imageSize.width() / (float)option.rect.width();
			scaleHeight = (float)imageSize.height() / (float)option.rect.height();
			// Downscale by the larger of the two proportions
			if (scaleHeight > scaleWidth)
			{
				drawWidth=(float)imageSize.width()/scaleHeight;
				drawHeight=(float)imageSize.height()/scaleHeight;
			}
			else
			{
				drawWidth=(float)imageSize.width()/scaleWidth;
				drawHeight=(float)imageSize.height()/scaleWidth;
			}

			float centerX = (float)option.rect.left() + (float)option.rect.width()/2.0;
			float centerY = (float)option.rect.top() + (float)option.rect.height()/2.0;
			drawRect.setLeft(centerX-drawWidth/2.0);
			drawRect.setRight(centerX+drawWidth/2.0);
			drawRect.setTop(centerY-drawHeight/2.0);
			drawRect.setBottom(centerY+drawHeight/2.0);
			painter->drawPixmap(drawRect, pixmap);
		}

	}
	else
	{
		QItemDelegate::paint(painter,option,index);
	}
}
ModelIndicesDeferredPainter::ModelIndicesDeferredPainter(QAbstractItemView *view, QObject *parent)
: QObject(parent)
{
	storedItemView = view;

	timer.setSingleShot(true);
	timer.setInterval(100);
	connect(&timer, SIGNAL(timeout()),
		this, SLOT(paintPendingIndexes()));
	isPaintingLater=false;
}

bool ModelIndicesDeferredPainter::paintLater(const QModelIndex& index)
{
	if (isPaintingLater)
		return false;

	if (indicesToPaint.contains(index)==false)
	{
		indicesToPaint.append(index);
		timer.start();
	}

	return true;
}

void ModelIndicesDeferredPainter::paintPendingIndexes()
{
	// This flag is needed so ByteArrayAsImageItemDelegate::paint does not just call paintLater again()
	isPaintingLater=true;

	foreach(const QModelIndex& index, indicesToPaint)
		storedItemView->update(index);
	storedItemView->repaint();

	isPaintingLater=false;

	indicesToPaint.clear();
}
