#ifndef __BYTE_ARRAY_AS_IMAGE_ITEM_DELEGATE_H_
#define __BYTE_ARRAY_AS_IMAGE_ITEM_DELEGATE_H_

#include <QItemDelegate>
#include <QTimer>

class ModelIndicesDeferredPainter;

// If an item is a byte array, wait 100 milliseconds, then load it as an image if possible.
class ByteArrayAsImageItemDelegate : public QItemDelegate
{
	Q_OBJECT
public:
	ByteArrayAsImageItemDelegate(QWidget *parent = 0) : QItemDelegate(parent) {deferredPainter=0;}

	virtual void paint(QPainter *painter, const QStyleOptionViewItem &option,
		const QModelIndex &index) const;

	// This is needed because you can't emit events from paint() since paint() is const
	void SetDeferredPainter(ModelIndicesDeferredPainter *df) {deferredPainter=df;}

signals:
	void paintingDeferred(const QModelIndex& index);

protected:
	ModelIndicesDeferredPainter *deferredPainter;
};

class ModelIndicesDeferredPainter : public QObject
{
	Q_OBJECT

public:
	ModelIndicesDeferredPainter(QAbstractItemView *view, QObject *parent);
	bool paintLater(const QModelIndex& index);

	protected slots:
	void paintPendingIndexes();

protected:
	QModelIndexList indicesToPaint;
	QAbstractItemView *storedItemView;

	// Don't immediately load images. This way, if an image immediately goes out of viewable area, it is not drawn
	QTimer timer;

	// This is needed so we know not to just call paintLater() again from ByteArrayAsImageItemDelegate
	bool isPaintingLater;
};


#endif
