#include "ScatterPlotWindow.h"
#include "EchoChamber.h"
#include "SQLQueryTool.h"
#include "TableWindow.h"
//#include "AnimatedTable.h"
#include "PieWindow.h"
#include "QwtWindow.h"
#include "SettingsScreen.h"
#include <qtcolorcombobox.h>
#include <QTimer>
#include <QSqlDatabase>
#include <QInputDialog>
#include <QMessageBox>
#include <QSqlError>
#include <QFileDialog>
#include <assert.h>
#include <QWhatsThis>
#include <QDesktopServices>
#include <QImageReader>
#include <QtXml/QDomDocument>
//#include <QDesktopWidget>
#include "StartupTips.h"
#include "ImageViewer.h"
#include "XMLSaver.h"

extern EchoChamber *echoChamber;

#define SQL_QUERY_TOOL_WINDOW_TITLE "SQL Query Tool"
#define WINDOW_NUMBER_SEPARATOR_STR ":"
#define WINDOW_NUMBER_SEPARATOR_CHAR ':'
#define SELECT_TABLES_QUERY "SELECT name FROM sqlite_master WHERE type='table' ORDER BY name;"

//static const int ANIMATION_RATE_MS=30;
//static const double TIMELINE_SLIDER_INCREMENT_PER_ANIMATION_TICK=30.0;

QwtText ScaleDrawNoScientific::label(double value) const
{
	return QLocale::system().toString(value,'g',10);
}

EchoChamber::EchoChamber(QWidget *parent, Qt::WFlags flags)
	: QMainWindow(parent, flags)
{
	setupUi(this);
	workspace = new QWorkspace(this);
	setCentralWidget(workspace); 

	windowSuffixInteger=1;
	needsUpdateWindowList=false;
	shuttingDown=false;

	settings.SetDefault();

	actionTimeline->setChecked(false);
	//actionQuery_Clipboard->setChecked(false);
	actionQuery_Clipboard->setChecked(true);
	dockWidget->setVisible(false);
	//dockWidget_2->setVisible(false);
	showTipsOnStartup=true;

	QMainWindow::statusBar()->hide();

	setWindowTitle("Echo Chamber Beta 1");


	QString qs;
	qs.sprintf("%f",timelineSlider->value());
	sliderTextLabel->setText(qs);

	timelineSlider->setScaleDraw(new ScaleDrawNoScientific);

	connect(&animationTimer, SIGNAL(timeout()), this, SLOT(animationTimerUpdate()));
	//AddDatabase("gradient.sqlite");
	//AddDatabase("scatterPlot.sqlite");
	//AddDatabase("functionLog.sqlite");
	//AddDatabase("video.sqlite");
	timelineSlider->setRange(0,10000,10000/10);
	timelineSliderIncrementPerAnimationTick=timestampIntervalSpinBox->value();

	stateChangedSinceLastSave=false;

	LoadRecentProjectList();
	LoadAppSettings();

	if (showTipsOnStartup)
	{
		StartupTips *window = new StartupTips(this, 0);
		window->move(
			(int)this->x()+(this->width() - window->width())/2,
			(int)this->y()+(this->height() - window->height())/2);
		window->setAttribute(Qt::WA_DeleteOnClose, true);
		window->show();
	}
}

EchoChamber::~EchoChamber()
{
	SaveAppSettings();

	shuttingDown=true;
	delete workspace;
}
void EchoChamber::on_addLastQueryPushButton_released()
{
	QString query = lastQueryLineEdit->text();
	if (query.isEmpty()==true)
	{
		QMessageBox::warning(this, tr("Cannot add to query clipboard"), tr("Last Query editbox is empty. Either type in a query manually, or use the SQL query tool to generate a query."));
		return;
	}

	// inline QListWidgetItem *itemAt(int x, int y) const;
	// inline void addItem(const QString &label) { insertItem(count(), label); }
	// int count() const;
	// QListWidgetItem *takeItem(int row);
	// inline void addItem(QListWidgetItem *item);

	int listSize = queryClipboard->count();
	int itemIndex;
	QListWidgetItem *listWidgetItem=0;
	for (itemIndex=0; itemIndex < listSize; itemIndex++)
	{
		const QString &itemText=queryClipboard->item(itemIndex)->text();
		if (itemText==query)
		{
			listWidgetItem=queryClipboard->takeItem(itemIndex);
			break;
		}
	}

	if (listWidgetItem)
	{
		queryClipboard->insertItem(0,listWidgetItem);
	}
	else
	{
		queryClipboard->insertItem(0,query);
	}
	queryClipboard->setCurrentItem(queryClipboard->item(0));
}
void EchoChamber::on_removeSelectedQueryPushButton_released()
{
	QList<QListWidgetItem*> selectedItems = queryClipboard->selectedItems();
	for (int i=0; i < selectedItems.size(); i++)
		queryClipboard->removeItemWidget(selectedItems[i]);
}
void EchoChamber::AddToLastQueryLineEdit(const QString& query)
{
	if (query.isEmpty())
		return;

	lastQueryLineEdit->setText(query);
	lastQueryLineEdit->setCursorPosition(0);

}
void EchoChamber::on_sqlQueryTool_destroyed(QObject* o)
{
	int idx = sqlQueryTools.indexOf((SQLQueryTool*)o);
	if (idx!=-1)
		sqlQueryTools.remove(idx);
}
void EchoChamber::SetWindowName(QWidget *window, const QString & windowType, QString windowNameSuffix)
{
	QString windowTitle;
	QString windowSuffix;
	windowSuffix.sprintf(WINDOW_NUMBER_SEPARATOR_STR "%i", windowSuffixInteger++);
	windowTitle = windowType + windowSuffix + QString(" ") +  windowNameSuffix;
	window->setWindowTitle(windowTitle);

	on_visualizationWindow_created(window);
}
void EchoChamber::SetupWindow(QWidget *window)
{
	workspace->addWindow(window);
//	assert(window->isWindow() && window->x()!=4);
	window->setAttribute(Qt::WA_DeleteOnClose, true);
//	assert(window->isWindow() && window->x()!=4);
	connect(window, SIGNAL(destroyed(QObject *)), this, SLOT(onWindowClosed(QObject *)));
	connect(workspace, SIGNAL(windowActivated(QWidget *)), this, SLOT(onWorkspaceWindowGotFocus(QWidget *)));
//	QObject::connect(window, SIGNAL(destroyed(QObject*)), this, SLOT(on_window_destroyed(QObject*)));

	window->show();
	RebuildWindowListMenu();
}
void EchoChamber::on_visualizationWindow_created(QWidget* object)
{
	if (animationTimer.isActive())
		RegenerateTimelineConstraints();
}
void EchoChamber::on_visualizationWindow_destroyed(QObject* object)
{
	if (animationTimer.isActive())
		RegenerateTimelineConstraints();
}
// bool EchoChamber::InitAMCWithDB(AnimatedModelList *amc, QStringList queries, QStringList databaseNames)
// {
// 	bool success=true;
// 	assert(queries.size());
// 	for (int i=0; i < queries.size(); i++)
// 	{
// 		if (!InitAMCWithDB(amc->GetAnimatedModelAtIndex(i), queries[i], databaseNames[i]))
// 			success=false;
// 	}
// 	return success;
// }
bool EchoChamber::InitAMCWithDB(AnimatedModel *amc, QString query, QString databaseName)
{
	// Load the first database that does not return an error. Fallback to specified database if none available
	QString databaseError;
	bool dbLoaded=false;
	QStringList potentialDatabases;

	bool alreadyHadDatabaseLoaded = databases.contains(databaseName);
	if (alreadyHadDatabaseLoaded==true)
	{
		databaseError = amc->InitDB(query, settings.postfixTickDefinition, databaseName);
		if (databaseError.isEmpty()==true)
		{
			return true;
		}
	}

	for (int dbIndex=0; dbIndex < databases.size(); dbIndex++)
	{
		databaseError = amc->InitDB(query, settings.postfixTickDefinition, databases[dbIndex]);
		if (databaseError.isEmpty()==true)
		{
			dbLoaded=true;
			potentialDatabases.push_back(databases[dbIndex]);
		}
	}
	bool hasDatabase;
	if (alreadyHadDatabaseLoaded==false)
		hasDatabase=AddDatabase(databaseName);
	else
		hasDatabase=true;

	if (alreadyHadDatabaseLoaded==false && hasDatabase==true)
	{
		databaseError = amc->InitDB(query, settings.postfixTickDefinition, databaseName);
		if (databaseError.isEmpty()==true)
		{
			dbLoaded=true;
			potentialDatabases.push_back(databaseName);
		}
	}
	if (potentialDatabases.size()>1)
	{
		// Ask user which one they want to use
		QString text = QInputDialog::getItem(this, tr("Select database"),
			tr("Select database to use with layout."), potentialDatabases);
		databaseError = amc->InitDB(query, settings.postfixTickDefinition, text);
		// Loaded it for testing, but didn't use it
		if (alreadyHadDatabaseLoaded==false && hasDatabase==true && text!=databaseName)
			RemoveDatabase(databaseName);
	}
	return dbLoaded;
}

SQLQueryTool *EchoChamber::SpawnSQLQueryTool(QStringList queries, QStringList databaseNames)
{
	SQLQueryTool *sqlQueryTool = new SQLQueryTool(this, 0);
	workspace->addWindow(sqlQueryTool);

	for (int i=0; i < databaseNames.size(); i++)
	{
		if (databases.contains(databaseNames[i])==false)
			AddDatabase(databaseNames[i]);
	}


	sqlQueryTool->setAttribute(Qt::WA_DeleteOnClose, true);
	QString windowTitle;
	windowTitle.sprintf(SQL_QUERY_TOOL_WINDOW_TITLE WINDOW_NUMBER_SEPARATOR_STR "%i", windowSuffixInteger++);
	sqlQueryTool->setWindowTitle(windowTitle);
	connect(sqlQueryTool, SIGNAL(destroyed(QObject *)), this, SLOT(onWindowClosed(QObject *)));
	connect(workspace, SIGNAL(windowActivated(QWidget *)), this, SLOT(onWorkspaceWindowGotFocus(QWidget *)));
	if (queries.size()==0)
		sqlQueryTool->SetLoadedDatabases(databaseNames);
	else
		sqlQueryTool->ExecuteQueries(queries,databaseNames,true);
	sqlQueryTool->show();
	QObject::connect(sqlQueryTool, SIGNAL(destroyed(QObject*)), this, SLOT(on_sqlQueryTool_destroyed(QObject*)));
	sqlQueryTools.push_back(sqlQueryTool);

	sqlQueryTool->treeWidget->expandToDepth(0);

	RebuildWindowListMenu();


	return sqlQueryTool;
}
TableWindow *EchoChamber::SpawnTableWindow(QStringList queries, QStringList databaseNames)
{
	TableWindow *window = new TableWindow(this, Qt::WindowStaysOnTopHint);
	window->InitWithDb(queries, databaseNames);
	SetupWindow(window);
	return window;
}
PieWindow *EchoChamber::SpawnPieWindow(QStringList queries, QStringList databaseNames)
{
	/*
	# #RGB (each of R, G, and B is a single hex digit)
	# #RRGGBB
	# #RRRGGGBBB
	# #RRRRGGGGBBBB
	# A name from the list of colors defined in the list of SVG color keyword names provided by the World Wide Web Consortium; for example, "steelblue" or "gainsboro". These color names work on all platforms.
	*/
	PieWindow *window = new PieWindow(this, Qt::WindowStaysOnTopHint);
	window->InitWithDb(queries, databaseNames);
	SetupWindow(window);
	return window;
}
QwtWindow *EchoChamber::SpawnQwtWindow(QStringList queries, QStringList databaseNames ,int initialRenderMode)
{
	QwtWindow *window = new QwtWindow(this, Qt::WindowStaysOnTopHint);
	window->InitWithDb(queries, databaseNames, initialRenderMode);
	SetupWindow(window);
	return window;
}
ScatterPlotWindow *EchoChamber::SpawnScatterPlotWindow(QStringList queries, QStringList databaseNames, bool startInHeatMapMode)
{
	ScatterPlotWindow *window = new ScatterPlotWindow(this, Qt::WindowStaysOnTopHint);
	window->InitWithDb(queries, databaseNames, startInHeatMapMode);
	SetupWindow(window);
	return window;
}
ImageViewer *EchoChamber::SpawnImageViewer(void)
{
	ImageViewer *window = new ImageViewer(this);
	SetupWindow(window);
	SetWindowName(window, "Image Viewer", QString());
	return window;
}
void EchoChamber::on_actionSave_Layout_activated()
{
	QVector<ISaveableWindow*> saveableWindows;
	GetSaveableWindows(saveableWindows);
	if (saveableWindows.size()==0)
	{
		// Only application window!
		QMessageBox::information(echoChamber, "No Windows", "Nothing to save");
		return;
	}

	QString fileName
		= QFileDialog::getSaveFileName(this, tr("Save layout"), "Layouts", tr("Layouts (*.elt);;All Files (*.*)"));
	if (fileName.isEmpty())
		return;
	if (fileName.endsWith(".elt")==false)
		fileName.append(".elt");

	XMLSaver xmlSaver("Layout");
	SaveLayout(xmlSaver);
	xmlSaver.SaveFile(fileName);
}
void EchoChamber::on_actionLoad_Layout_activated()
{
	QString fileName
		= QFileDialog::getOpenFileName(this, tr("Load layout"), mostRecentLayoutPath.isEmpty() ? "Layouts" : mostRecentLayoutPath);
	if (fileName.isEmpty())
		return;
	XMLSaver xmlSaver("Layout");
	if (!xmlSaver.LoadFile(fileName)) {
		QString msg = tr("Failed to open %1")
			.arg(fileName);
		QMessageBox::warning(this, tr("Error"), msg);
		return;
	}
	
	LoadAllWindows(xmlSaver);

	QDir dir(fileName);
	dir.cdUp();			
	mostRecentLayoutPath=dir.path();

	QMessageBox::information(echoChamber, "Layout Loaded", "Loaded layout " + fileName);
}
void EchoChamber::on_actionSQL_Query_activated()
{
	if (databases.size()==0)
	{
		// QMessageBox::warning(this, tr("Failed to open SQL Query Tool"), "No sqlite databases are loaded. Open one with file / load database first.");

		LoadDatabase();
		if (databases.size()==0)
		{
			QMessageBox::warning(this, tr("Cannot open SQL Query Tool without a database"), "No sqlite databases are loaded. Open one with file / load database first. See the manual on how to generate databases from code, or use one created by a third party application.");
		}
		else
		{
			QStringList sl;
// 			for (int i=0; i < databases.size(); i++)
// 				sl.push_back(SELECT_TABLES_QUERY);
			SpawnSQLQueryTool(sl,databases);
		}
	}
	else
	{
		SQLQueryTool *sqlQueryTool=SpawnSQLQueryTool(QStringList(),databases);
	}
}
void EchoChamber::on_actionImage_Viewer_activated()
{
	QString fileName;
	QImage image;
	if (ImageViewer::OpenImageWithDialog(&image,fileName))
	{
		ImageViewer *imageViewer = echoChamber->SpawnImageViewer();
		echoChamber->SetWindowName(imageViewer, "Image Viewer", QString());
		imageViewer->SetToImage(image,fileName);
	}
}
void EchoChamber::on_queryClipboard_itemDoubleClicked(QListWidgetItem *item)
{
	QWidget *activeWindow=workspace->activeWindow();
	
	// Using window name to figure out what type of window it is
	QString qsTruncated;
	TruncateAtCharacter(activeWindow->windowTitle(), WINDOW_NUMBER_SEPARATOR_CHAR, qsTruncated);
	if (qsTruncated==SQL_QUERY_TOOL_WINDOW_TITLE)
	{
		SQLQueryTool *sqlQueryTool = (SQLQueryTool*) activeWindow;
	//	sqlQueryTool->SetQuery(item->text());
	//	sqlQueryTool->ExecuteQuery(true);
		sqlQueryTool->AppendQuery(item->text());
	}
}
void EchoChamber::on_actionQuit_activated()
{
	if (stateChangedSinceLastSave)
	{
		int result = QMessageBox::question(echoChamber, "Save?", "Save before quitting?", QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel );
		if (result==QMessageBox::Cancel)
			return;
		if (result==QMessageBox::Yes)
			on_actionSave_Project_activated();
	}
	close();
}
void EchoChamber::SetDefault()
{
	settings.SetDefault();
	actionTimeline->setChecked(false);
	actionQuery_Clipboard->setChecked(false);
	dockWidget->setVisible(false);
	dockWidget_2->setVisible(false);
	queryClipboard->clear();
	workspace->closeAllWindows();
	stateChangedSinceLastSave=false;
}
void EchoChamber::on_actionNew_Project_activated()
{
	// Prompt to save if changed
	if (stateChangedSinceLastSave)
	{
		if (QMessageBox::question(echoChamber, "Save?", "Save existing project?", QMessageBox::Yes, QMessageBox::No )==QMessageBox::Yes)
			on_actionSave_Project_activated();
	}

	SetDefault();

	projectFilename.clear();
	setWindowTitle("Echo Chamber");

}
void EchoChamber::OpenProject(QString fileName)
{
	SetDefault();
	XMLSaver xmlSaver("Project");
	if (!xmlSaver.LoadFile(fileName)) {
		QString msg = tr("Failed to open %1")
			.arg(fileName);
		QMessageBox::warning(this, tr("Error"), msg);
		int idx = recentProjects.indexOf(fileName);
		if (idx!=-1)
		{
			recentProjects.removeAt(idx);
			UpdateRecentProjectListWidget();
			SaveRecentProjectList();
		}
		return;
	}

	LoadProject(xmlSaver);
	projectFilename=fileName;
	setWindowTitle(projectFilename + " - Echo Chamber");

	AddToRecentProjectList(projectFilename);
}
void EchoChamber::on_actionOpen_Project_activated()
{
	// Prompt to save if changed
	if (stateChangedSinceLastSave)
	{
		if (QMessageBox::question(echoChamber, "Save?", "Save existing project?", QMessageBox::Yes, QMessageBox::No )==QMessageBox::Yes)
			on_actionSave_Project_activated();
	}

	QString fileName = QFileDialog::getOpenFileName(this,
		tr("Open Database"), mostRecentProjectPath.isEmpty() ? "Projects" : mostRecentProjectPath, tr("Project Files (*.ecproj);;All Files (*.*)"));
	if (fileName.isEmpty()==false)
	{
		OpenProject(fileName);
	}

}
void EchoChamber::on_actionSave_Project_activated()
{
	if (projectFilename.isEmpty())
		on_actionSave_Project_As_activated();
	else
	{
// 		QFile file(projectFilename);
// 		if (!file.open(QFile::WriteOnly)) {
// 			QString msg = tr("Failed to open %1\n%2")
// 				.arg(projectFilename)
// 				.arg(file.errorString());
// 			QMessageBox::warning(this, tr("Error"), msg);
// 			return;
// 		}
// 		QDataStream dataStream;
// 		dataStream.setDevice(&file);
		

		XMLSaver xmlSaver("EchoChamberProject");
		XML_SAVER_SAVE_ELEMENT(xmlSaver, showTipsOnStartup);
		SaveProject(xmlSaver);
		xmlSaver.SaveFile(projectFilename);
	}
}
void EchoChamber::on_actionSave_Project_As_activated()
{
	QString fileName
		= QFileDialog::getSaveFileName(this, tr("Save project"), "Projects", tr("Project Files (*.ecproj);;All Files (*.*)"));
	if (fileName.isEmpty())
		return;
	if (fileName.endsWith(".ecproj")==false)
		fileName.append(".ecproj");
	QFile file(fileName);
	if (!file.open(QFile::WriteOnly)) {
		QString msg = tr("Failed to open %1\n%2")
			.arg(fileName)
			.arg(file.errorString());
		QMessageBox::warning(this, tr("Error"), msg);
		return;
	}
	file.close();

	projectFilename=fileName;
	setWindowTitle(projectFilename + " - Echo Chamber");
// 	QDataStream dataStream;
// 	dataStream.setDevice(&file);
// 	SaveProject(dataStream);

	XMLSaver xmlSaver("EchoChamberProject");
	SaveProject(xmlSaver);
	xmlSaver.SaveFile(projectFilename);


	AddToRecentProjectList(projectFilename);
}
void EchoChamber::SaveProject(XMLSaver &xmlSaver)
{
	// project saving is query clipboard, preferences, layout resave, echo chamber window geometry, databases
	xmlSaver.SaveElement("ProductVersionName", GetProductVersionName());
	xmlSaver.SaveElement("LayoutVersionNumber", GetLayoutVersionNumber());;
	QByteArray geo_data = saveGeometry();
	QByteArray layout_data = saveState();
	XML_SAVER_SAVE_ELEMENT(xmlSaver, geo_data);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, layout_data);
	settings.Save(xmlSaver);
	xmlSaver.SaveElement("Databases", databases);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, mostRecentProjectPath);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, mostRecentDatabasePath);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, mostRecentLayoutPath);
	SaveLayout(xmlSaver);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, queryClipboard);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, lastQueryLineEdit);

	stateChangedSinceLastSave=false;
}
void EchoChamber::LoadProject(XMLSaver &xmlSaver)
{
	QString loadingProductVersionName;
	int loadingProductVersionNumber;
	xmlSaver.LoadElement("ProductVersionName", loadingProductVersionName, "Unknown");
	xmlSaver.LoadElement("LayoutVersionNumber", loadingProductVersionNumber, "Unknown");

	if (loadingProductVersionNumber > GetLayoutVersionNumber())
	{
		QString message("Project was saved with incompatible/newer version ");
		message+=loadingProductVersionName;
		message+=" our version is ";
		message+=GetProductVersionName();
		message+=".";
		QMessageBox::warning(this,"Cannot load project",
			message,
			"OK");
		return;
	}

	echoChamber->workspace->closeAllWindows();

	QByteArray geo_data;
	QByteArray layout_data;
	XML_SAVER_LOAD_ELEMENT(xmlSaver, geo_data, false);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, layout_data, false);

	restoreGeometry(geo_data);
	restoreState(layout_data);

	settings.Load(xmlSaver);
	QStringList potentialDatabases;
	xmlSaver.LoadElement("Databases", potentialDatabases, false);

	// If databases are currently loaded, and don't match what we are loading now, prompt the user if they want to unload the current ones
	if (databases.size()>0)
	{
		bool databasesMatch=true;
		if (potentialDatabases.size()==databases.size())
		{
			for (int j=0; j < potentialDatabases.size(); j++)
			{
				if (databases.contains(potentialDatabases[j], Qt::CaseSensitive)==false)
				{
					databasesMatch=false;
					break;
				}
			}
		}
		else
			databasesMatch=false;
		if (databasesMatch==false)
		{
			int result = QMessageBox::question(echoChamber, "Unload databases?", "Unload currently loaded databases?", QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel );
			if (result==QMessageBox::Cancel)
				return;
			if (result==QMessageBox::Yes)
			{
				for (int i=0; i < databases.size(); i++)
					QSqlDatabase::removeDatabase(databases[i]);
				databases.clear();
			}
		}
	}

	for (int i=0; i < potentialDatabases.count(); i++)
		AddDatabase(potentialDatabases[i]);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, mostRecentProjectPath, "");
	XML_SAVER_LOAD_ELEMENT(xmlSaver, mostRecentDatabasePath, "");
	XML_SAVER_LOAD_ELEMENT(xmlSaver, mostRecentLayoutPath, "");
	LoadAllWindows(xmlSaver);

	XML_SAVER_LOAD_ELEMENT(xmlSaver, queryClipboard,"");
	XML_SAVER_LOAD_ELEMENT(xmlSaver, lastQueryLineEdit,"");

	actionQuery_Clipboard->setChecked(dockWidget->isVisible());
	actionTimeline->setChecked(dockWidget_2->isVisible());
	
//	stateChangedSinceLastSave=false;

	// For now just flag always. I don't want to find every place in the code I change anything and call this.
	FlagNeedsSave();
}
void EchoChamber::on_actionCascade_activated()
{
	workspace->cascade();
}
void EchoChamber::on_actionTile_activated()
{
	workspace->tile();
}
void EchoChamber::on_actionQuery_Clipboard_activated()
{
	FlagNeedsSave();
}
void EchoChamber::on_actionTimeline_activated()
{
	FlagNeedsSave();
}
void EchoChamber::FlagNeedsSave(void)
{
	stateChangedSinceLastSave=true;
}
void EchoChamber::SpawnSQLQueryToolWithFirstPassingQuery(const QString &query)
{
	if (databases.size()>1)
	{
		int databaseIndex;
		for (databaseIndex=0; databaseIndex < databases.size(); databaseIndex++)
		{
			QSqlQueryModelFetchAll model;
			QSqlDatabase db = QSqlDatabase::addDatabase(DATABASE_TYPE, "TEMP");
			db.setDatabaseName(databases[databaseIndex]);
			db.open();
			model.setQuery(query, db);
			QSqlError error = model.lastError();
			QSqlDatabase::removeDatabase("TEMP");
			if (error.isValid()==false)
			{
				// No error
				QStringList queryList;
				queryList.push_back(query);
				QStringList queryList2;
				queryList2.push_back(databases[databaseIndex]);
				SQLQueryTool *queryTool = SpawnSQLQueryTool(queryList,queryList2);
				queryTool->SelectTree(databaseIndex);
				return;
			}
		}
	}
	if (databases.size()>0)
	{
		QStringList queryList;
		queryList.push_back(query);
		SpawnSQLQueryTool(queryList,databases);
	}
}
void EchoChamber::on_actionFunction_Calls_activated()
{
	SpawnSQLQueryToolWithFirstPassingQuery(settings.functionCallQuery);
}
void EchoChamber::on_actionPacket_Logs_activated()
{
	SpawnSQLQueryToolWithFirstPassingQuery(settings.packetLogQuery);
}
void EchoChamber::on_actionNetwork_Statistics_activated()
{
	SpawnSQLQueryToolWithFirstPassingQuery(settings.networkStatisticsQuery);
}
void EchoChamber::on_actionEditSettings_activated()
{
	SettingsScreen *w = new SettingsScreen(this, 0, settings);
	w->setModal(true);
	w->setAttribute(Qt::WA_DeleteOnClose, true);
	w->show();
	FlagNeedsSave();
}
void EchoChamber::onWorkspaceWindowGotFocus(QWidget *qw)
{
	RebuildWindowListMenu();
}
void EchoChamber::RemoveFromWindowListMenu(QWidget *qw)
{
	FlagNeedsSave();

	if (shuttingDown)
		return;

	QList<QAction*> acts = menuWindows->actions();
	unsigned int actionIndex;
	for (actionIndex=0; actionIndex < acts.size(); actionIndex++)
	{
		if (qw->windowTitle()==acts[actionIndex]->text())
		{
			menuWindows->removeAction(acts[actionIndex]);

			unsigned int remainingActionsCount=0;
			for (actionIndex=0; actionIndex < acts.size(); actionIndex++)
			{
				if (IsWidgetInWorkspace(acts[actionIndex]->text()))
					remainingActionsCount++;
			}
			if (remainingActionsCount==0)
			{
				// Remove the last separator if there is one
				acts = menuWindows->actions();
				if (acts.size()>0 && acts[acts.size()-1]->isSeparator())
				{
					menuWindows->removeAction(acts[acts.size()-1]);
				}
			}
		}
	}
}
void EchoChamber::RebuildWindowListMenu(void)
{
	if (shuttingDown)
		return;

	unsigned int widgetListIndex;
	QWidgetList widgetList=workspace->windowList();
	QWidget *activeWindow=workspace->activeWindow();
	
	// Remove all existing actions that reference windows
	QList<QAction*> acts = menuWindows->actions();
	QList<QAction*> actsToRemove;
	unsigned int actionIndex;
	for (actionIndex=0; actionIndex < acts.size(); actionIndex++)
	{
		if (IsWidgetInWorkspace(acts[actionIndex]->text()))
			actsToRemove.append(acts[actionIndex]);
	}
	for (actionIndex=0; actionIndex < actsToRemove.size(); actionIndex++)
	{
		menuWindows->removeAction(actsToRemove[actionIndex]);
	}

	bool lastActionIsSeparator = acts.size()>0 && acts[acts.size()-1]->isSeparator();
	bool hasMoreWindows = widgetList.size()>0;
	if (hasMoreWindows==true && lastActionIsSeparator==false)
		menuWindows->addSeparator();
	else if (hasMoreWindows==false && lastActionIsSeparator==true)
		menuWindows->removeAction(acts[acts.size()-1]);

	// Add all windows as actions
	for (widgetListIndex=0; widgetListIndex < widgetList.size(); widgetListIndex++)
	{
		QIcon icon;
		if (activeWindow==widgetList[widgetListIndex])
			icon.addFile(QString::fromUtf8("Resources/checkbox.png"), QSize(), QIcon::Normal, QIcon::Off);
		QAction *qa = menuWindows->addAction(icon, widgetList[widgetListIndex]->windowTitle(), widgetList[widgetListIndex], SLOT(setFocus()));
	}

	needsUpdateWindowList=false;
	FlagNeedsSave();
}
void EchoChamber::onWindowClosed(QObject *o)
{
	RemoveFromWindowListMenu((QWidget*)o);
}
void EchoChamber::onWindowClosed(QWidget *o)
{
	RemoveFromWindowListMenu((QWidget*)o);
}
bool EchoChamber::IsWidgetInWorkspace(QWidget *qw) const
{
	QWidgetList windowList = workspace->windowList();
	for (unsigned int i=0; i < windowList.size(); i++)
	{
		if (windowList[i]==qw)
			return true;
	}
	return false;
}
bool EchoChamber::IsWidgetInWorkspace(const QString &qs) const
{
	QString qsTruncated, truncatedWindowTitle;
	TruncateAtCharacter(qs, WINDOW_NUMBER_SEPARATOR_CHAR, qsTruncated);

	QWidgetList windowList = workspace->windowList();
	for (unsigned int i=0; i < windowList.size(); i++)
	{
		TruncateAtCharacter(windowList[i]->windowTitle(), WINDOW_NUMBER_SEPARATOR_CHAR, truncatedWindowTitle);
		if (qsTruncated==truncatedWindowTitle)
			return true;
	}
	return false;
}
void EchoChamber::TruncateAtCharacter(const QString &qs, QChar qc, QString &out)
{
	out = qs;
	int index = qs.indexOf(qc);
	if (index!=-1)
		out.chop(out.size()-index);
}
int EchoChamber::GetColumnIndex(const QSqlQueryModelFetchAll *model,
								const QString &columnName)
{
	int columnCount = model->columnCount();
	int columnIndex;
	for (columnIndex=0; columnIndex < columnCount; columnIndex++)
	{
		if (model->record().fieldName(columnIndex)==columnName)
			return columnIndex;
	}
	return -1;
}
void EchoChamber::GetColumnTypes(const QSqlQueryModelFetchAll *model,
								 QStringList &allColumnNames,
								 QStringList &numericalColumnNames, QList<int> &numericalColumnIndices,
								 QStringList &textColumnNames, QList<int> &textColumnIndices,
								 int &timestampColumnIndex)
{
// 	allColumnNames.clear();
// 	numericalColumnNames.clear();
// 	numericalColumnIndices.clear();
// 	textColumnNames.clear();
// 	textColumnIndices.clear();

	timestampColumnIndex=-1;

	if (model->columnCount()==0)
		return;

	int columnCount = model->columnCount();
	int columnIndex;
	for (columnIndex=0; columnIndex < columnCount; columnIndex++)
	{
		const QSqlRecord &record = model->record();
		// WRONG! The record type is always string. Need to read the data type
		//const QSqlField &field = record.field(columnIndex);
		QVariant v = model->data(model->index(0, columnIndex));
//		const QVariant &v = field.type();
		const QString &fieldName = record.fieldName(columnIndex);

		if (v.type()==QVariant::Int ||
			v.type()==QVariant::UInt ||
			v.type()==QVariant::Bool ||
			v.type()==QVariant::LongLong ||
			v.type()==QVariant::ULongLong ||
			v.type()==QVariant::Double)
		{
			// Numerical column
			if (numericalColumnNames.indexOf(fieldName)==-1)
			{
				numericalColumnNames.push_back(fieldName);
				numericalColumnIndices.push_back(columnIndex);
			}
		}
		else
		{
			// Text column
			if (textColumnNames.indexOf(fieldName)==-1)
			{
				textColumnNames.push_back(fieldName);
				textColumnIndices.push_back(columnIndex);
			}
		}

		if (allColumnNames.indexOf(fieldName)==-1)
		{
			allColumnNames.push_back(fieldName);
		}
	}

	timestampColumnIndex=settings.PickBestIndex(numericalColumnNames, EchoChamberSettings::SLS_TIME);
}
void EchoChamber::AddDefaultColors(QtColorComboBox *colorComboBox)
{
	colorComboBox->addColor(Qt::black, tr("Black"));
	colorComboBox->addColor(Qt::red, tr("Red"));
	colorComboBox->addColor(Qt::darkRed, tr("Dark red"));
	colorComboBox->addColor(Qt::green, tr("Green"));
	colorComboBox->addColor(Qt::darkGreen, tr("Dark green"));
	colorComboBox->addColor(Qt::blue, tr("Blue"));
	colorComboBox->addColor(Qt::darkBlue, tr("Dark blue"));
	colorComboBox->addColor(Qt::cyan, tr("Cyan"));
	colorComboBox->addColor(Qt::darkCyan, tr("Dark cyan"));
	colorComboBox->addColor(Qt::magenta, tr("Magenta"));
	colorComboBox->addColor(Qt::darkMagenta, tr("Dark magenta"));
	colorComboBox->addColor(Qt::yellow, tr("Yellow"));
	colorComboBox->addColor(Qt::darkYellow, tr("Dark yellow"));
	colorComboBox->addColor(Qt::gray, tr("Gray"));
	colorComboBox->addColor(Qt::darkGray, tr("Dark gray"));
	colorComboBox->addColor(Qt::lightGray, tr("Light gray"));
	colorComboBox->addColor(Qt::white, tr("White")); // White last so when I autopick colors when adding new tabs, it doesn't add a color you probably can't see
	colorComboBox->setColorDialogEnabled(true);
}

int EchoChamber::SetupXYZTimeComboBoxes(const QSqlQueryModelFetchAll *model, QComboBox *xComboBox, QComboBox *yComboBox, QComboBox *zComboBox, QComboBox *timeComboBox)
{
	QStringList numericalColumnNames;
	QList<int> numericalColumnIndices;
	QStringList textColumnNames;
	QList<int> textColumnIndices;
	QStringList allColumnNames;
	int timestampColumnIndex;
	int defaults=0;
	GetColumnTypes(model, allColumnNames, numericalColumnNames, numericalColumnIndices, textColumnNames, textColumnIndices, timestampColumnIndex);

	if (xComboBox)
	{
		xComboBox->clear();
		for (int i=0; i < numericalColumnNames.size(); i++)
			xComboBox->addItem(numericalColumnNames[i]);

		int idx = settings.PickBestIndex(numericalColumnNames, EchoChamberSettings::SLS_X);
		if (idx!=-1)
		{
			defaults|=1;
			xComboBox->setCurrentIndex(idx);
		}

	}
	if (yComboBox)
	{
		yComboBox->clear();
		for (int i=0; i < numericalColumnNames.size(); i++)
			yComboBox->addItem(numericalColumnNames[i]);
		int idx = settings.PickBestIndex(numericalColumnNames, EchoChamberSettings::SLS_Y);
		if (idx!=-1)
		{
			defaults|=2;
			yComboBox->setCurrentIndex(idx);
		}

	}
	if (zComboBox)
	{
		zComboBox->clear();
		for (int i=0; i < numericalColumnNames.size(); i++)
			zComboBox->addItem(numericalColumnNames[i]);
		int idx = settings.PickBestIndex(numericalColumnNames, EchoChamberSettings::SLS_Z);
		if (idx!=-1)
		{
			defaults|=4;
			zComboBox->setCurrentIndex(idx);
		}

	}
	if (timeComboBox)
	{
		timeComboBox->clear();
		for (int i=0; i < numericalColumnNames.size(); i++)
			timeComboBox->addItem(numericalColumnNames[i]);
		int idx = settings.PickBestIndex(numericalColumnNames, EchoChamberSettings::SLS_TIME);
		if (idx!=-1)
		{
			defaults|=8;
			timeComboBox->setCurrentIndex(idx);
		}
	}

	return defaults;
}
void EchoChamber::SetupSymbolMagnitudeComboBox(const QSqlQueryModelFetchAll *model, QComboBox *comboBox)
{
	QStringList numericalColumnNames;
	QList<int> numericalColumnIndices;
	QStringList textColumnNames;
	QList<int> textColumnIndices;
	QStringList allColumnNames;
	int timestampColumnIndex;
	GetColumnTypes(model, allColumnNames, numericalColumnNames, numericalColumnIndices, textColumnNames, textColumnIndices, timestampColumnIndex);
	comboBox->clear();
	for (int i=0; i < numericalColumnNames.size(); i++)
		comboBox->addItem(numericalColumnNames[i]);
	int idx = settings.PickBestIndex(numericalColumnNames, EchoChamberSettings::SLS_MAGNITUDE);
	if (idx!=-1)
		comboBox->setCurrentIndex(idx);
}
void EchoChamber::SetupIntensityComboBox(const QSqlQueryModelFetchAll *model, QComboBox *comboBox)
{
	QStringList numericalColumnNames;
	QList<int> numericalColumnIndices;
	QStringList textColumnNames;
	QList<int> textColumnIndices;
	QStringList allColumnNames;
	int timestampColumnIndex;
	GetColumnTypes(model, allColumnNames, numericalColumnNames, numericalColumnIndices, textColumnNames, textColumnIndices, timestampColumnIndex);
	comboBox->clear();
	for (int i=0; i < numericalColumnNames.size(); i++)
		comboBox->addItem(numericalColumnNames[i]);
	int idx = settings.PickBestIndex(numericalColumnNames, EchoChamberSettings::SLS_MAGNITUDE);
	if (idx!=-1)
		comboBox->setCurrentIndex(idx);
}
void EchoChamber::SetupRGBComboBox(const QSqlQueryModelFetchAll *model, QComboBox *comboBox, bool *defaultWasSet)
{
	QStringList numericalColumnNames;
	QList<int> numericalColumnIndices;
	QStringList textColumnNames;
	QList<int> textColumnIndices;
	QStringList allColumnNames;
	int timestampColumnIndex;
	GetColumnTypes(model, allColumnNames, numericalColumnNames, numericalColumnIndices, textColumnNames, textColumnIndices, timestampColumnIndex);
	comboBox->clear();
	for (int i=0; i < textColumnNames.size(); i++)
		comboBox->addItem(textColumnNames[i]);
	int idx = settings.PickBestIndex(textColumnNames, EchoChamberSettings::SLS_COLOR);
	if (idx!=-1)
		comboBox->setCurrentIndex(idx);
}
void EchoChamber::UpdateSettings(const EchoChamberSettings& newSettings)
{
	settings=newSettings;
}
void EchoChamber::on_timelineSlider_valueChanged(double timelineSliderValue)
{
}
void EchoChamber::on_timelineSlider_sliderMoved(double timelineSliderValue)
{
	UpdateLabelAndModels();
}
void EchoChamber::UpdateLabel(void)
{
	QString qs;
	qs.sprintf("%f",timelineSlider->value());
	sliderTextLabel->setText(qs);
}
void EchoChamber::UpdateLabelAndModels(void)
{
	UpdateLabel();
	SetAnimatedModelsToTimeline();
}
void EchoChamber::on_stepBackwardsToolButton_released()
{
	playAnimationToolButton->setIcon(QIcon("Resources/play.png"));
	animationTimer.stop();

	bool isForward=false;
	int increment=-1;
	// Advance to the lowest time past idxAfter
	double timelineSliderValue = timelineSlider->value();
	QVector<AnimatedModel*> animatedModels;
	GetAnimatedModels(animatedModels);
	int idxBefore;
	int idxAfter;
	double valueBefore;
	double valueAfter;
	double highestTimeBefore=DBL_MIN;
	double nextStepTime;
	for (int i=0; i < animatedModels.size(); i++)
	{
		if (animatedModels[i]->GetIndexOfValue(&idxBefore, &idxAfter, &valueBefore, &valueAfter, timelineSliderValue, timestampColumnComboBox->currentText()))
		{
			if (animatedModels[i]->CanRowStep() && animatedModels[i]->GetCurrentRowIndex()>=idxBefore-1 && animatedModels[i]->GetCurrentRowIndex()<=idxAfter+1)
				idxAfter=animatedModels[i]->GetNextRowStep(animatedModels[i]->GetCurrentRowIndex()+increment, isForward);
			else
				idxBefore--;

			if (animatedModels[i]->GetDoubleAtIndex(idxBefore, timestampColumnComboBox->currentText(), &nextStepTime))
			{
				if (nextStepTime>highestTimeBefore)
					highestTimeBefore=nextStepTime;
			}
		}
	}
	if (highestTimeBefore!=DBL_MIN)
	{
		timelineSlider->setValue(highestTimeBefore);
	}
	UpdateLabel();
	// Now that we know the maximum step time, advance everything one tick as long as it does not go past the timeline
	timelineSliderValue = timelineSlider->value();
	for (int i=0; i < animatedModels.size(); i++)
	{
		if (animatedModels[i]->GetIndexOfValue(&idxBefore, &idxAfter, &valueBefore, &valueAfter, timelineSliderValue, timestampColumnComboBox->currentText()))
		{
			bool usePostfix = animatedModels[i]->CanRowStep() && animatedModels[i]->GetCurrentRowIndex()>=idxBefore-1 && animatedModels[i]->GetCurrentRowIndex()<=idxAfter+1;
			if (usePostfix)
			{
				idxBefore=animatedModels[i]->GetCurrentRowIndex()+increment;
				idxAfter=animatedModels[i]->GetNextRowStep(animatedModels[i]->GetCurrentRowIndex()+increment, isForward);
			}

			if (animatedModels[i]->GetDoubleAtIndex(idxAfter, timestampColumnComboBox->currentText(), &nextStepTime))
			{
				if (nextStepTime<=timelineSliderValue)
				{
					if (usePostfix)
					{
						animatedModels[i]->SetNextRowIndex(idxAfter);
					}
					animatedModels[i]->SetAnimationRowIndices(idxBefore, idxAfter, valueBefore, valueAfter);
				}
			}
		}
	}
}
void EchoChamber::on_stepForwardsToolButton_released()
{
	playAnimationToolButton->setIcon(QIcon("Resources/play.png"));
	animationTimer.stop();

	bool isForward=true;
	int increment=1;
	// Advance to the lowest time past idxAfter
	double timelineSliderValue = timelineSlider->value();
	QVector<AnimatedModel*> animatedModels;
	GetAnimatedModels(animatedModels);
	int idxBefore;
	int idxAfter;
	double valueBefore;
	double valueAfter;
	double lowestTimePast=DBL_MAX;
	double nextStepTime;
	for (int i=0; i < animatedModels.size(); i++)
	{
		if (animatedModels[i]->GetIndexOfValue(&idxBefore, &idxAfter, &valueBefore, &valueAfter, timelineSliderValue, timestampColumnComboBox->currentText()))
		{
			if (animatedModels[i]->CanRowStep() && animatedModels[i]->GetCurrentRowIndex()>=idxBefore-1 && animatedModels[i]->GetCurrentRowIndex()<=idxAfter+1)
				idxAfter=animatedModels[i]->GetNextRowStep(animatedModels[i]->GetCurrentRowIndex()+increment, isForward);
			else
				idxAfter++;

			if (animatedModels[i]->GetDoubleAtIndex(idxAfter, timestampColumnComboBox->currentText(), &nextStepTime))
			{
				if (nextStepTime<lowestTimePast)
					lowestTimePast=nextStepTime;
			}
		}
	}
	if (lowestTimePast!=DBL_MAX)
	{
		timelineSlider->setValue(lowestTimePast);
	}
	UpdateLabel();

	// Now that we know the maximum step time, advance everything one tick as long as it does not go past the timeline
	timelineSliderValue = timelineSlider->value();
	for (int i=0; i < animatedModels.size(); i++)
	{
		if (animatedModels[i]->GetIndexOfValue(&idxBefore, &idxAfter, &valueBefore, &valueAfter, timelineSliderValue, timestampColumnComboBox->currentText()))
		{
			bool usePostfix = animatedModels[i]->CanRowStep() && animatedModels[i]->GetCurrentRowIndex()>=idxBefore-1 && animatedModels[i]->GetCurrentRowIndex()<=idxAfter+1;
			if (usePostfix)
			{
				idxBefore=animatedModels[i]->GetCurrentRowIndex()+increment;
				idxAfter=animatedModels[i]->GetNextRowStep(animatedModels[i]->GetCurrentRowIndex()+increment, isForward);
			}

			if (animatedModels[i]->GetDoubleAtIndex(idxAfter, timestampColumnComboBox->currentText(), &nextStepTime))
			{
				if (nextStepTime<=timelineSliderValue)
				{
					if (usePostfix)
					{
						animatedModels[i]->SetNextRowIndex(idxAfter);
					}
					animatedModels[i]->SetAnimationRowIndices(idxBefore, idxAfter, valueBefore, valueAfter);
				}
			}
		}
	}
}
void EchoChamber::SetAnimatedModelsToTimeline(void)
{
	double timelineSliderValue = timelineSlider->value();
	QVector<AnimatedModel*> animatedModels;
	GetAnimatedModels(animatedModels);
	int idxBefore;
	int idxAfter;
	double valueBefore;
	double valueAfter;
	for (int i=0; i < animatedModels.size(); i++)
	{
		if (animatedModels[i]->GetIndexOfValue(&idxBefore, &idxAfter, &valueBefore, &valueAfter, timelineSliderValue, timestampColumnComboBox->currentText()))
		{
			animatedModels[i]->SetAnimationRowIndices(idxBefore, idxAfter, valueBefore, valueAfter);
		}
	}
}
void EchoChamber::on_loopAnimationToolButton_released()
{
}
void EchoChamber::on_playAnimationToolButton_released()
{
	if (animationTimer.isActive())
	{
		playAnimationToolButton->setIcon(QIcon("Resources/play.png"));
		animationTimer.stop();
	}
	else
	{
		const float ANIMATION_RATE_MS = 1000.0/playbackFPSSpinBox->value();
		playAnimationToolButton->setIcon(QIcon("Resources/paused.png"));
		animationTimer.setInterval(ANIMATION_RATE_MS);
		animationTimer.setSingleShot(false);
		RegenerateTimelineConstraints();
		animationTimer.start(ANIMATION_RATE_MS);

		whenAnimationStarted = QTime::currentTime();
	}
}
void EchoChamber::on_playbackFPSSpinBox_valueChanged(double d)
{
	if (animationTimer.isActive())
	{
		animationTimer.stop();
		const float ANIMATION_RATE_MS = 1000.0/playbackFPSSpinBox->value();
		animationTimer.setInterval(ANIMATION_RATE_MS);
		animationTimer.setSingleShot(false);
		RegenerateTimelineConstraints();
		animationTimer.start(ANIMATION_RATE_MS);

		whenAnimationStarted = QTime::currentTime();
	}

	FlagNeedsSave();
}
void EchoChamber::on_stopAnimationToolButton_released()
{
	animationTimer.stop();

	QVector<AnimatedModel*> animatedModels;
	GetAnimatedModels(animatedModels);
	for (int i=0; i < animatedModels.size(); i++)
	{
		animatedModels[i]->ClearAnimationRowIndices();
	}
	timelineSlider->setValue(timelineSlider->minValue());


	playAnimationToolButton->setIcon(QIcon("Resources/play.png"));
	QString qs;
	qs.sprintf("%f",timelineSlider->value());
	sliderTextLabel->setText(qs);
}
void EchoChamber::on_timestampColumnComboBox_currentIndexChanged(int idx)
{
	RegenerateTimelineConstraints();
	on_stopAnimationToolButton_released();
	UpdateLabelAndModels();
	FlagNeedsSave();
}
void EchoChamber::on_timestampIntervalSpinBox_valueChanged(double d)
{
	timelineSliderIncrementPerAnimationTick=d;
	FlagNeedsSave();
}
void EchoChamber::animationTimerUpdate()
{
	float timestampMillisecondsPerRealMillisecond = timestampIntervalSpinBox->value() / playbackFPSSpinBox->value();
	QTime now = QTime::currentTime();
	float elapsedTimeline = timestampMillisecondsPerRealMillisecond * (whenAnimationStarted.msecsTo(now));
	whenAnimationStarted=now;

	double oldTimelineValue = timelineSlider->value();
	if (timelineSlider->value()+elapsedTimeline>timelineSlider->maxValue())
	{
		if (loopAnimationToolButton->isChecked())
		{
			float extraTime = elapsedTimeline - (timelineSlider->maxValue()-timelineSlider->value());
			if (extraTime >= timelineSlider->maxValue()-timelineSlider->minValue())
				timelineSlider->setValue(timelineSlider->minValue());
			else
				timelineSlider->setValue(timelineSlider->minValue()+extraTime);
		}
		else
		{
			timelineSlider->setValue(timelineSlider->maxValue());
			animationTimer.stop();
		}
	}
	else
	{
		timelineSlider->setValue(timelineSlider->value()+elapsedTimeline);
	}

	double newTimelineValue = timelineSlider->value();
	UpdateLabel();
	QVector<AnimatedModel*> animatedModels;
	GetAnimatedModels(animatedModels);
	int idxBefore_old;
	int idxAfter_old;
	double valueBefore_old;
	double valueAfter_old;
	int idxBefore_new;
	int idxAfter_new;
	double valueBefore_new;
	double valueAfter_new;
	for (int i=0; i < animatedModels.size(); i++)
	{
		if (animatedModels[i]->GetIndexOfValue(&idxBefore_old, &idxAfter_old, &valueBefore_old, &valueAfter_old, oldTimelineValue, timestampColumnComboBox->currentText()))
		{
			animatedModels[i]->GetIndexOfValue(&idxBefore_new, &idxAfter_new, &valueBefore_new, &valueAfter_new, newTimelineValue, timestampColumnComboBox->currentText());
			animatedModels[i]->SetAnimationRowIndices(idxBefore_old, idxAfter_new, valueBefore_old, valueAfter_new);
		}
	}
}
void EchoChamber::RegenerateTimestampColumn(int databaseIndex)
{
	QSqlQueryModelFetchAll model;
	QSqlDatabase treeViewDb = QSqlDatabase::addDatabase(DATABASE_TYPE, "TEMP");
	treeViewDb.setDatabaseName(databases[databaseIndex]);
	treeViewDb.open();
	bool anyAdded=false;

	QSqlQuery query(treeViewDb);
	query.exec(SELECT_TABLES_QUERY);
	while (query.next())
	{
		QString name = query.value(0).toString();
		QString queryStr = "PRAGMA table_info(";
		queryStr+=name;
		queryStr+=");";
		QSqlQuery query2(treeViewDb);
		query2.exec(queryStr);
		while (query2.next())
		{
			QString columnName = query2.value(1).toString();
			QString columnType = query2.value(2).toString();
			if (columnType.compare("smallint",Qt::CaseInsensitive)==0 ||
				columnType.compare("integer",Qt::CaseInsensitive)==0 ||
				columnType.compare("bigint",Qt::CaseInsensitive)==0 ||
				columnType.compare("decimal",Qt::CaseInsensitive)==0 ||
				columnType.compare("numeric",Qt::CaseInsensitive)==0 ||
				columnType.compare("real",Qt::CaseInsensitive)==0 ||
				columnType.compare("double precision",Qt::CaseInsensitive)==0 ||
				columnType.compare("serial",Qt::CaseInsensitive)==0 ||
				columnType.compare("bigserial",Qt::CaseInsensitive)==0)
			{
				if (timestampColumnComboBox->findText(columnName,Qt::MatchExactly)==-1)
				{
					timestampColumnComboBox->addItem(columnName);
					anyAdded=true;
				}
			}
		}
	}
	
	if (anyAdded)
	{
		int i;
		for (i=0; i < settings.stringListSettings[EchoChamberSettings::SLS_TIME].size(); i++ )
		{
			QString setting = settings.stringListSettings[EchoChamberSettings::SLS_TIME][i];
			int idx = timestampColumnComboBox->findText(setting,Qt::MatchExactly);
			if (idx!=-1)
			{
				timestampColumnComboBox->setCurrentIndex(idx);
				return;
			}
		}
	}

	QSqlDatabase::removeDatabase("TEMP");
}
void EchoChamber::RegenerateTimestampColumns(void)
{
	timestampColumnComboBox->clear();
	if (databases.size()>1)
	{
		int databaseIndex;
		for (databaseIndex=0; databaseIndex < databases.size(); databaseIndex++)
		{
			RegenerateTimestampColumn(databaseIndex);
		}
	}
}
void EchoChamber::RegenerateTimelineConstraints(void)
{
	double min=DBL_MAX,max=DBL_MIN;
	QVector<AnimatedModel*> animatedModels;
	GetAnimatedModels(animatedModels);
	for (int i=0; i < animatedModels.size(); i++)
	{
		double minTemp, maxTemp;
		if (animatedModels[i]->GetModel()->GetNumericalColumnConstraints(timestampColumnComboBox->currentText(), &minTemp, &maxTemp))
		{
			if (minTemp<min)
				min=minTemp;
			if (maxTemp>max)
				max=maxTemp;
		}
	}

	if (max>=min)
	{
		timelineSlider->setRange(min,max);
		timelineSlider->setScale(min,max);

		if (timelineSlider->value()<min)
			timelineSlider->setValue(min);
		else if (timelineSlider->value()>max)
			timelineSlider->setValue(max);
	}
}
void EchoChamber::GetAnimatedModels(QVector<AnimatedModel*> &animatedModels)
{
	animatedModels.clear();
	AnimatedModel * animatedModelList = AnimatedModel::GetLinkedList();
	while (animatedModelList)
	{
		animatedModels.push_back(animatedModelList);
		animatedModelList=animatedModelList->next;
	}
}
void EchoChamber::GetSaveableWindows(QVector<ISaveableWindow*> &saveableWindows)
{
	saveableWindows.clear();
	ISaveableWindow * saveableList = ISaveableWindow::GetLinkedList();
	while (saveableList)
	{
		saveableWindows.push_back(saveableList);
		saveableList=saveableList->next;
	}
}
QString EchoChamber::GetTimelineColumn(void) const
{
	return timestampColumnComboBox->currentText();
}
// QString EchoChamber::QueryForSQLEdit(QSqlQueryModelFetchAll* model, QString databaseName)
// {
// 	bool pressedOk;
// 	while (1)
// 	{
// 		QString text = QInputDialog::getText(this, tr("Edit Query"),
// 			tr("SQL Query"), QLineEdit::Normal,
// 			model->query().lastQuery(), &pressedOk);
// 		if (pressedOk && !text.isEmpty() && text!=model->query().lastQuery())
// 		{
// 			QSqlQueryModel testModel;
// 			testModel.setQuery(text);
// 			QSqlError error = testModel.lastError();
// 			if (error.isValid()==false)
// 			{
// 				AddToLastQueryLineEdit(text);
// 				model->setQuery(text);
// 				return text;
// 			}
// 			else
// 			{
// 				QMessageBox::warning(this,text,
// 					error.databaseText(),
// 					"OK");
// 			}
// 		}
// 		else
// 			break;
// 	}
// 	return QString();
// }
QString EchoChamber::QueryForTickEdit(AnimatedModel* model)
{
	bool pressedOk;
	while (1)
	{
		QString text = QInputDialog::getText(this, tr("Edit Postfix Tick Definition"),
			tr("Tick Definition"), QLineEdit::Normal,
			model->GetTickDefinition(), &pressedOk);
		if (pressedOk && !text.isEmpty() && text!=model->GetTickDefinition())
		{
			QList<PostfixTableRowQuery::QueryToken> tokenList;
			if (PostfixTableRowQuery::EvaluateQuery(text,tokenList))
			{
				model->SetTickDefinition(text);
				return text;
			}
			else
			{
				QMessageBox::warning(this,text,
					"Invalid Syntax",
					"OK");
			}
		}
		else
			break;
	}
	return QString();
}
QString EchoChamber::LoadDatabase()
{
	QString fileName = QFileDialog::getOpenFileName(this,
		tr("Open Database"), mostRecentDatabasePath.isEmpty() ? QString::null : mostRecentDatabasePath, tr("SQLite Files (*.sqlite);;All Files (*.*)"));
	if (fileName.isEmpty()==false)
	{
		if (AddDatabase(fileName))
			FlagNeedsSave();
		else
			return QString();
	}
	return fileName;

}
void EchoChamber::on_actionLoad_Database_triggered()
{
	QString fileName = LoadDatabase();

	// Annoying
	/*
	if (fileName.isEmpty()==false && sqlQueryTools.size()==0)
	{
		QString msg = "Loaded ";
		msg+=fileName;
		msg+=". Execute Queries / SQL Query from the file menu to run SQL queries on this database.";
		QMessageBox::information(this,"Loaded database",
			msg,
			"OK");
	}
	*/
}
void EchoChamber::on_actionUnload_Database_triggered()
{
	if (databases.size()==0)
	{
		QMessageBox::warning(this,tr("Can't unload"),
			tr("No databases are loaded."),
			"OK");
		return;
	}

	bool pressedOk;
	QString text = QInputDialog::getItem(this, tr("Unload Database"),
		tr("Select database to unload"), databases,
		0, false, &pressedOk);
	if (pressedOk && !text.isEmpty())
	{
		RemoveDatabase(text);
	}
}

bool EchoChamber::AddDatabase(QString dbName)
{
	if (databases.contains(dbName)==false)
	{
		// Validate database
		QSqlQueryModelFetchAll model;
		if (QSqlDatabase::isDriverAvailable(DATABASE_TYPE)==false)
		{
			QMessageBox::critical(this,"SQL Driver not available",
				"Can't open driver QSQLITE. Unable to load any database!",
				"OK");
			return false;
		}
		QSqlDatabase db = QSqlDatabase::addDatabase(DATABASE_TYPE, dbName);
		db.setDatabaseName(dbName);
		if (db.open()==false || db.isOpenError()==true)
		{
			QString errorMsg = "Failed to open \"";
			errorMsg+=dbName;
			errorMsg+="\". ";
			errorMsg+=db.lastError().text();
			QMessageBox::warning(this,"Can't open file",
				errorMsg,
				"OK");
			return false;
		}
		if (db.isValid()==false)
		{
			QString errorMsg = "Failed to open \"";
			errorMsg+=dbName;
			errorMsg+="\". ";
			errorMsg+=db.lastError().text();
			QMessageBox::warning(this,"Invalid database",
				errorMsg,
				"OK");
			return false;
		}

		QSqlQuery query;
		model.setQuery(SELECT_TABLES_QUERY,db);
		QSqlError error = model.lastError();
		if (error.isValid()==true)
		{
			db.close();
			QSqlDatabase::removeDatabase(dbName);

			QString errorMsg = "\"";
			errorMsg+=dbName;
			errorMsg+="\" can't be queried. ";
			errorMsg+=error.databaseText();
			QMessageBox::critical(this,"Can't query database tables",
				errorMsg,
				"OK");
			return false;
		}
		else
		{
			db.close();
			QDir dir(dbName);
			dir.cdUp();			
			mostRecentDatabasePath=dir.path();
			databases.append(dbName);
			RegenerateTimestampColumn(databases.size()-1);

			for (int i=0; i < sqlQueryTools.size(); i++)
			{
				sqlQueryTools[i]->AddDatabase(dbName);
// 				sqlQueryTools[i]->UpdateTreeView();
// 				sqlQueryTools[i]->treeWidget->expandToDepth(0);
			}
			return true;
		}
	}
	return false;
}
void EchoChamber::RemoveDatabase(QString dbName)
{
	int index = databases.indexOf(dbName);
	if (index!=-1)
	{
		QSqlDatabase::removeDatabase(dbName);
		databases.removeAt(index);
		RegenerateTimestampColumns();

		for (int i=0; i < sqlQueryTools.size(); i++)
		{
			sqlQueryTools[i]->UnloadDB(dbName);
// 			if (sqlQueryTools[i]->UnloadDB(dbName) && databases.size())
// 				sqlQueryTools[i]->SetLoadedDatabases(databases);
// 			sqlQueryTools[i]->UpdateTreeView();
// 			sqlQueryTools[i]->treeWidget->expandToDepth(0);
		}
	}

}
void EchoChamber::SaveLayout(XMLSaver &xmlSaver)
{
	ISaveableWindow* saveableWindow;
	QVector<ISaveableWindow*> saveableWindows;
	GetSaveableWindows(saveableWindows);
	xmlSaver.SaveElement("ProductVersionName", GetProductVersionName());
	xmlSaver.SaveElement("LayoutVersionNumber", GetLayoutVersionNumber());
	for (int i=0; i < saveableWindows.size(); i++)
	{
		xmlSaver.CDDown(xmlSaver.MakeDirectory("SaveableWindow"));
		saveableWindow=saveableWindows[i];
		saveableWindow->Save(xmlSaver);
		xmlSaver.CDUp();
	}
}
void EchoChamber::LoadAllWindows(XMLSaver &xmlSaver)
{
	QString loadingProductVersionName;
	int loadingProductVersionNumber;
	xmlSaver.LoadElement("ProductVersionName", loadingProductVersionName, "Unknown");
	xmlSaver.LoadElement("LayoutVersionNumber", loadingProductVersionNumber, "-1");

	if (loadingProductVersionNumber > GetLayoutVersionNumber())
	{
		QString message("Layout was saved with incompatible/newer version ");
		message+=loadingProductVersionName;
		message+=" our version is ";
		message+=GetProductVersionName();
		message+=".";
		QMessageBox::warning(echoChamber,"Cannot load layout",
			message,
			"OK");
		return;
	}

	if (echoChamber->workspace->activeWindow()!=0)
	{
		if (QMessageBox::question(echoChamber, "Close existing windows?", "Close existing windows before loading new layout?", QMessageBox::Yes, QMessageBox::No )==QMessageBox::Yes)
		{
			echoChamber->workspace->closeAllWindows();
		}
	}

	QStringList databaseNames;
	QStringList queries;
	QString windowTitle;
	if (xmlSaver.CDDown("SaveableWindow"))
	{
		do
		{
			WindowTypeId windowTypeId;
			ISaveableWindow::LoadType(xmlSaver, windowTypeId);
			windowTitle = ISaveableWindow::LoadWindowTitle(xmlSaver);
			QString qsTruncated;
			TruncateAtCharacter(windowTitle, WINDOW_NUMBER_SEPARATOR_CHAR, qsTruncated);
			switch (windowTypeId)
			{
			case WTI_SQL_QUERY_TOOL:
				{
					SQLQueryTool *newWindow = echoChamber->SpawnSQLQueryTool(queries, databaseNames);
					newWindow->Load(xmlSaver);
					echoChamber->SetWindowName(newWindow, qsTruncated, QString());
				}
				break;
			case WTI_PIE_WINDOW:
				{
					PieWindow *newWindow = echoChamber->SpawnPieWindow(queries, databaseNames);
					newWindow->Load(xmlSaver);
					echoChamber->SetWindowName(newWindow, qsTruncated, newWindow->GetDBName());
				}
				break;
			case WTI_QWT_WINDOW:
				{
					QwtWindow *newWindow = echoChamber->SpawnQwtWindow(queries, databaseNames, 0);
					newWindow->Load(xmlSaver);
					echoChamber->SetWindowName(newWindow, qsTruncated, newWindow->GetDBName());
				}
				break;
			case WTI_SCATTER_PLOT_WINDOW:
				{
					ScatterPlotWindow *newWindow = echoChamber->SpawnScatterPlotWindow(queries, databaseNames, false);
					newWindow->Load(xmlSaver);
					echoChamber->SetWindowName(newWindow, qsTruncated, newWindow->GetDBName());
				}
				break;
			case WTI_TABLE_WINDOW:
				{
					TableWindow *newWindow = echoChamber->SpawnTableWindow(queries, databaseNames);
					newWindow->Load(xmlSaver);
					echoChamber->SetWindowName(newWindow, qsTruncated, newWindow->GetDBName());
				}
				break;
			case WTI_IMAGE_VIEWER:
				{
					ImageViewer *newWindow = echoChamber->SpawnImageViewer();
					newWindow->Load(xmlSaver);
					echoChamber->SetWindowName(newWindow, qsTruncated, QString());
				}
				break;
			}
		} while (xmlSaver.CDNext("SaveableWindow"));
		xmlSaver.CDUp();
	}
}
void EchoChamber::SetCurrentText(QComboBox *comboBox, QString string)
{
	int index = comboBox->findText(string);
	if (index!=-1)
		comboBox->setCurrentIndex(index);
}
void EchoChamber::AddToRecentProjectList(QString name)
{
	int idx = recentProjects.indexOf(name);
	if (idx!=-1)
		recentProjects.removeAt(idx);
	QDir dir(name);
	dir.cdUp();
	mostRecentProjectPath=dir.path();
	recentProjects.push_front(name);
	if (recentProjects.size()>5)
		recentProjects.pop_back();
	SaveRecentProjectList();

	UpdateRecentProjectListWidget();
}
void EchoChamber::UpdateRecentProjectListWidget(void)
{
	menu_Recent_Projects->clear();
	for (int i=0; i < recentProjects.size(); i++)
	{
	//	QAction *recentProjectAction;
	//	recentProjectAction = new QAction(this);
	//	recentProjectAction->setObjectName(recentProjects[i]);
	//	recentProjectAction->setText(recentProjects[i]);
		QAction *recentProjectAction = menu_Recent_Projects->addAction(recentProjects[i]);
	}
}
void EchoChamber::on_menu_Recent_Projects_triggered(QAction *action)
{
	OpenProject(action->text());
}
void EchoChamber::SaveRecentProjectList(void)
{
	XMLSaver xmlSaver("RecentProjects");
	XML_SAVER_SAVE_ELEMENT(xmlSaver, recentProjects);
	xmlSaver.SaveFile("RecentProjects.xml");
}
void EchoChamber::LoadRecentProjectList(void)
{

	XMLSaver xmlSaver("RecentProjects");
	if (xmlSaver.LoadFile("RecentProjects.xml"))
	{
		XML_SAVER_LOAD_ELEMENT(xmlSaver, recentProjects, false);
		UpdateRecentProjectListWidget();
	}
}
void EchoChamber::on_actionWhat_s_This_activated()
{
	if (QWhatsThis::inWhatsThisMode())
		QWhatsThis::leaveWhatsThisMode();
	else
		QWhatsThis::enterWhatsThisMode();
}
void EchoChamber::on_actionManual_activated()
{
	QDesktopServices::openUrl ( QUrl("http://www.jenkinssoftware.com/echochamber/manual") );
}
void EchoChamber::on_actionWebsite_activated()
{
	QDesktopServices::openUrl ( QUrl("http://www.jenkinssoftware.com/echochamber") );
}
void EchoChamber::on_actionAbout_activated()
{
	QMessageBox::information(echoChamber, "Echo Chamber Beta 1", "Echo Chamber version Beta1\nBy Jenkins Software LLC\nBuilt on " + QString(__DATE__) + QString(" at ") + QString(__TIME__), QMessageBox::Yes );
}
void EchoChamber::LoadAppSettings()
{
	XMLSaver xmlSaver("AppSettings");
	if (xmlSaver.LoadFile("AppSettings.xml"))
	{
		XML_SAVER_LOAD_ELEMENT(xmlSaver, showTipsOnStartup, true);
	}
}
void EchoChamber::SaveAppSettings()
{
	XMLSaver xmlSaver("AppSettings");
	XML_SAVER_SAVE_ELEMENT(xmlSaver, showTipsOnStartup);
	xmlSaver.SaveFile("AppSettings.xml");


// 	QDomDocument doc("AppSettings");
// 	QDomElement root = doc.createElement("AppSettings");
// 	doc.appendChild(root);
//  	QDomElement tag = doc.createElement("showTipsOnStartup");
//  	root.appendChild(tag);
//  	tag.setAttribute("value", (int) showTipsOnStartup);
// 
// 	//////////
// 
// 
// 	QFile file("AppSettings.xml");
// 	if (!file.open(QFile::WriteOnly)) {
// 		return;
// 	}
// 	file.write(doc.toByteArray());
// 	file.close();
}
bool EchoChamber::GetImageFromModel(QModelIndex modelIndex, QImage *image)
{
	QBuffer buffer;
	const QVariant &v = modelIndex.data();
	if (v.type()==QVariant::ByteArray)
	{
		QBuffer buffer;
		QByteArray ba;
		ba = v.toByteArray();;
		buffer.setBuffer(&ba);
		buffer.open(QIODevice::ReadOnly);
		QImageReader qir;
		qir.setDevice(&buffer);
		return qir.read(image);
	}
	return false;
}
