#ifndef __ECHO_CHAMBER_SETTINGS_H
#define __ECHO_CHAMBER_SETTINGS_H

#include <QString>
#include <QStringList>
#include <QDataStream>
#include "XMLSaver.h"

struct EchoChamberSettings
{
	enum StringListSettings
	{
		SLS_MAGNITUDE,
		SLS_X,
		SLS_Y,
		SLS_Z,
		SLS_TIME,
		SLS_COLOR,
		SLS_COUNT,
	};

	QStringList stringListSettings[SLS_COUNT];
	QString functionCallQuery;
	QString packetLogQuery;
	QString networkStatisticsQuery;
	QString postfixTickDefinition;
	bool showTipsOnStartup;

	void Save(XMLSaver &xmlSaver);
	void Load(XMLSaver &xmlSaver);
	void SetDefault(void)
	{
		stringListSettings[SLS_MAGNITUDE].clear(); stringListSettings[SLS_MAGNITUDE].push_back("Magnitude");
		stringListSettings[SLS_MAGNITUDE].push_back("Intensity");
		stringListSettings[SLS_Y].clear();
		stringListSettings[SLS_Y].push_back("PositionY");
		stringListSettings[SLS_Y].push_back("Y");
		stringListSettings[SLS_Y].push_back("FPS");
		stringListSettings[SLS_Y].push_back("lastFPS");
		stringListSettings[SLS_Y].push_back("averageFPS");
		stringListSettings[SLS_Y].push_back("minFPS");
		stringListSettings[SLS_Y].push_back(stringListSettings[SLS_MAGNITUDE][0]);
		stringListSettings[SLS_Y].push_back(stringListSettings[SLS_MAGNITUDE][1]);
		stringListSettings[SLS_Z].clear();
		stringListSettings[SLS_Z].push_back("PositionZ");
		stringListSettings[SLS_Z].push_back("Z");
		stringListSettings[SLS_TIME].clear(); stringListSettings[SLS_TIME].push_back("TimestampNumeric");
		stringListSettings[SLS_TIME].push_back("Timestamp");
		stringListSettings[SLS_X].clear();
		stringListSettings[SLS_X].push_back("PositionX");
		stringListSettings[SLS_X].push_back("X");
		stringListSettings[SLS_X].push_back(stringListSettings[SLS_TIME][0]);
		stringListSettings[SLS_X].push_back(stringListSettings[SLS_TIME][1]);
		stringListSettings[SLS_COLOR].push_back("Color");
		functionCallQuery="SELECT FunctionCallFriendlyText, SourceFile, SourceLine, AutoTickCount, TimestampNumeric FROM functionCalls";
		packetLogQuery="SELECT * FROM defaultPacketLogTable";
		networkStatisticsQuery="SELECT * FROM defaultNetworkStatisticsTable";
		postfixTickDefinition="AutoIPAddress,SAME,AutoTickCount,CHANGED,AND";
		showTipsOnStartup=true;
	}

	bool IsInSettingList(const QString& string, StringListSettings slsType)
	{
		int i;
		for (i=0; i < stringListSettings[slsType].size(); i++ )
		{
			if (stringListSettings[slsType][i].toLower()==string.toLower())
				return true;
		}
		return false;
	}
	int PickBestIndex(QStringList stringList, StringListSettings slsType)
	{
		int i;
		for (i=0; i < stringListSettings[slsType].size(); i++ )
		{
			int j;
			for (j=0; j < stringList.size(); j++)
			{
				QString q1 = stringListSettings[slsType][i].toLower();
				QString q2 = stringList[j].toLower();
				if (q1==q2)
					return j;
			}
		}
		return -1;
	}
};

#endif
