#ifndef __IMAGEVIEWER_H
#define __IMAGEVIEWER_H

#include <QMainWindow>
#include <QPrinter>
#include "ISaveableWindow.h"

class QAction;
class QLabel;
class QMenu;
class QScrollArea;
class QScrollBar;

class ImageViewer : public QMainWindow, public ISaveableWindow
{
	Q_OBJECT

public:
	ImageViewer(QWidget *parent);
	virtual void Save(XMLSaver &xmlSaver);
	virtual void Load(XMLSaver &xmlSaver);
	virtual WindowTypeId GetWindowTypeId(void) const {return WTI_IMAGE_VIEWER;}
	virtual int GetVersionNumber(void) const {return 0;}
	bool OpenFile(QString _fileName);
	void OpenImageWithDialog(void);
	static bool OpenImageWithDialog(QImage *image, QString &_fileName);
	void SetToImage(QImage image, QString _fileName=QString());

	private slots:
		void open();
		void saveAs();
		void print();
		void zoomIn();
		void zoomOut();
		void normalSize();
		void fitToWindow();

private:
	void CreateActions();
	void CreateMenus();
	void UpdateActions();
	void ScaleImage(double amount);
	void AdjustScrollBar(QScrollBar *scrollBar, double factor);

	QLabel *imageLabel;
	QScrollArea *scrollArea;
	double scaleFactor;

	QPrinter printer;

	QAction *openAct;
	QAction *saveAsAct;
	QAction *printAct;
	QAction *exitAct;
	QAction *zoomInAct;
	QAction *zoomOutAct;
	QAction *normalSizeAct;
	QAction *fitToWindowAct;

	QMenu *fileMenu;
	QMenu *viewMenu;

	QString fileName;
};

#endif
