#include "PieWidget.h"
#include "PieWindow.h"
#include "EchoChamber.h"
#include <QPen>
#include <QBrush>
#include <QPainter>
#include <QPaintEvent>
#include <QColor>
#include <math.h>
#include <qwt_math.h>

extern EchoChamber *echoChamber;


PieWidget::PieWidget(QWidget *parent) : QWidget(parent), pieWindow((PieWindow*)parent)
{
}

PieWidget::~PieWidget()
{
}
void PieWidget::paintEvent(QPaintEvent *event)
{
	QBrush background(Qt::white);
	QPen foregroundLines(Qt::black);
	QPen textPen(Qt::black);
	QPainter painter(this);
	painter.setRenderHint(QPainter::Antialiasing);

	const int maxPossibleRows=50;
	const float pieBorderWidthLeft=125;
	const float labelExteriorBorder=10;
	const float pieBorderHeight=125;
	const float labelsBorderHeight=50;
	float labelsBorderToPieMargin=125;
	const float labelsMinWidth=150;
	int pixelsHeightBetweenText=2;
	int pixelsBetweenRightOfBoxAndText=4;
	const int boxMargin=2;
	int fontHeightUpFix=3; // Centering font doesn't really center it for unknown reasons
	int fontHeightWidthFix=5; // Same for width
	QRectF labelsBox;
	const float labelsBoxLeftInteriorBorder=5;
	const float labelsBoxRightInteriorBorder=5;
	const float labelsBoxTopInteriorBorder=3;
	const float minDistanceForTextSurroundingPie=10;
	int ellipseLeft, ellipseTop;
	ellipseLeft=pieBorderWidthLeft;
	int availableHeight=height()-pieBorderHeight*2;
	float labelsWidth=width()-(pieBorderWidthLeft+availableHeight+labelExteriorBorder*2+labelsBorderToPieMargin);
	if (labelsWidth<labelsMinWidth)
	{
		labelsWidth=0;
		labelsBorderToPieMargin=0;
	}
	int availableWidth=width()-(pieBorderWidthLeft+labelExteriorBorder)-labelsBorderToPieMargin*2;
	if (availableHeight<=0 || availableWidth<=0)
		return;
	int diameter;
	if (availableHeight < availableWidth)
		diameter=availableHeight;
	else
		diameter=availableWidth;
	ellipseTop=(height()-diameter)/2;

	painter.fillRect(event->rect(), background);
	painter.setPen(foregroundLines);

	if (labelsWidth>0)
	{
		labelsBox.setRight(width()-labelExteriorBorder);
		labelsBox.setLeft(labelsBox.right()-labelsWidth);
		labelsBox.setTop(labelsBorderHeight);
		labelsBox.setBottom(height()-labelsBorderHeight);

		painter.drawRect(labelsBox);

	}
	QList<PieWindow::Row>::iterator iter;
	QFont font;
	font.setPointSize(10);
	painter.setFont(font);
	int realFontHeight = QFontMetrics(font).height();
	int rowHeight = realFontHeight+pixelsHeightBetweenText;
	float availableLabelsBoxSpace=labelsBox.height()-labelsBoxTopInteriorBorder*2;
	int numItemsCanShow=availableLabelsBoxSpace/rowHeight;
	int wastedVerticalSpace;
	if (pieWindow->rowEnd-pieWindow->rowStart > numItemsCanShow)
		wastedVerticalSpace = availableLabelsBoxSpace-numItemsCanShow*rowHeight;
	else
		wastedVerticalSpace = 0;
	int labelsRenderX=labelsBox.left()+labelsBoxLeftInteriorBorder;
	int labelsRenderY=labelsBox.top()+labelsBoxTopInteriorBorder+wastedVerticalSpace/2;
	QRectF renderBox;
	renderBox.setLeft(labelsRenderX);
	renderBox.setRight(labelsRenderX+rowHeight-boxMargin*2);
	QColor rowColor;
	QFontMetrics fm = painter.fontMetrics();
	int textLeft = renderBox.right() + pixelsBetweenRightOfBoxAndText;
	int textRight=labelsBox.right()-labelsBoxRightInteriorBorder;
	int availableTextWidth=textRight-textLeft;
	QString ellipses="...";
	int ellipsesWidth=fm.width(ellipses);
	int numRows;
	if (labelsWidth<=0 || pieWindow->rowEnd-pieWindow->rowStart<numItemsCanShow)
		numRows=pieWindow->rowEnd-pieWindow->rowStart;
	else
		numRows=numItemsCanShow;
	if (numRows>maxPossibleRows)
		numRows=maxPossibleRows;
	int rowIndex=0;
	double sumOfValues=0;
	iter=pieWindow->sortedRows.begin();
	while (iter!=pieWindow->sortedRows.end() && rowIndex++<numRows)
	{
		sumOfValues+=iter->value;
		iter++;
	}
	rowIndex=0;

	// Draw labels
	if (labelsWidth>0)
	{
		iter=pieWindow->sortedRows.begin();
		while (iter!=pieWindow->sortedRows.end() && rowIndex++<numRows)
		{
			// Draw a little box
			renderBox.setTop(labelsRenderY+boxMargin);
			renderBox.setBottom(labelsRenderY+rowHeight-boxMargin);
			rowColor=iter->color;
			QBrush boxInteriorBrush(rowColor);
			painter.fillRect(renderBox, boxInteriorBrush);
			painter.drawRect(renderBox);			

			// inline void QPainter::drawText(int x, int y, int w, int h, int flags, const QString &str, QRect *br)

			QString baseTextToDraw=iter->label;
			QString valAndPercent;
			valAndPercent.sprintf(" (%.2f) (%.0f%%)", iter->value, 100.0*iter->value/sumOfValues);
			baseTextToDraw+=valAndPercent;
			int baseTextLength=baseTextToDraw.length();
			QString croppedText = baseTextToDraw;
			int textWidth = fm.width(croppedText);
			if (textWidth+ellipsesWidth>availableTextWidth)
			{
				float printableProportion=(float)(textWidth)/(float)availableTextWidth;
				int numChars=(int)((float)(croppedText.length()-ellipses.length())*printableProportion);
				croppedText.truncate(numChars);
				croppedText.append(ellipses);
				int adjustedWidth=fm.width(croppedText);
				if (adjustedWidth>availableTextWidth)
				{
					// Reduce string length
					do 
					{
						croppedText.truncate(croppedText.length()-ellipses.length()-1);
						croppedText.append(ellipses);
						adjustedWidth=fm.width(croppedText);
					} while (adjustedWidth>availableTextWidth && croppedText.isNull()==false);
				}
				else
				{
					// Increase string length
					while (adjustedWidth<availableTextWidth && numChars<baseTextLength) 
					{
						numChars++;
						croppedText = baseTextToDraw;
						croppedText.truncate(numChars);
						croppedText.append(ellipses);
						adjustedWidth=fm.width(croppedText);
					}

					croppedText = baseTextToDraw;
					croppedText.truncate(numChars-1);
					croppedText.append(ellipses);
				}
			}

			painter.drawText(textLeft, (renderBox.bottom()-renderBox.top())/2+renderBox.top()+realFontHeight/2-fontHeightUpFix, croppedText );

			labelsRenderY+=rowHeight;
			iter++;
		}
	}

	// Draw pie segments
	float ellipseCenterX=ellipseLeft+diameter/2;
	float ellipseCenterY=ellipseTop+diameter/2;
	float textCenterX,textCenterY,renderTextX1,renderTextY1,renderTextX2,renderTextY2;
	double startAngle=0;
	double startProportion=0;
	QFont f = painter.font();
	iter=pieWindow->sortedRows.begin();
	QRect textRect;
	painter.setPen(Qt::black);
	float sliceProportionLineThreshhold=.03;
	int index=0;
	rowIndex=0;
	while (iter!=pieWindow->sortedRows.end() && rowIndex++<numRows)
	{
		double sliceProportion = iter->value/sumOfValues;
		double sliceRadiansCenter=(startProportion-sliceProportion/2)*2.0*M_PI;
		rowColor=iter->color;
		QBrush pieInteriorBrush(rowColor);
		painter.setBrush(pieInteriorBrush);
		painter.setPen(Qt::black);
		painter.drawPie(ellipseLeft, ellipseTop, diameter, diameter, (int) startAngle, (int) 360.0*sliceProportion*16.0);

		float textWidth=fm.width(iter->label);
		float extraDist;
		extraDist=textWidth/2;
		if (extraDist<rowHeight*2)
			extraDist=rowHeight*2;
		if (extraDist<minDistanceForTextSurroundingPie)
			extraDist=minDistanceForTextSurroundingPie;
		if (extraDist > labelsBorderToPieMargin/2)
			extraDist=labelsBorderToPieMargin/2;

		if (sliceProportion<sliceProportionLineThreshhold)
		{
			textCenterX=ellipseCenterX+cos(sliceRadiansCenter)*((1+(index&1))*(extraDist+fontHeightWidthFix)+diameter/2);
			textCenterY=ellipseCenterY+sin(sliceRadiansCenter)*((1+(index&1))*(extraDist+fontHeightWidthFix)+diameter/2);
		}
		else
		{
			textCenterX=ellipseCenterX+cos(sliceRadiansCenter)*(extraDist+fontHeightWidthFix+diameter/2);
			textCenterY=ellipseCenterY+sin(sliceRadiansCenter)*(extraDist+fontHeightWidthFix+diameter/2);
		}

		

		renderTextX1=textCenterX-textWidth/2;
		renderTextY1=textCenterY;
		textRect.setLeft(renderTextX1);
		textRect.setTop(renderTextY1-realFontHeight);
		textRect.setRight(renderTextX1+textWidth);
		textRect.setBottom(renderTextY1+realFontHeight);
	//	f.setBold(true);
		f.setPointSize(10);
		painter.setFont(f);
		QString percentage;
		percentage.sprintf("%.0f%%", sliceProportion*100);
		textWidth=fm.width(percentage);
		renderTextX2=textCenterX-textWidth/2;
		renderTextY2=renderTextY1+realFontHeight;

		if (textWidth>textRect.right()-textRect.left())
		{
			textRect.setLeft(renderTextX2);
			textRect.setRight(renderTextX2+textWidth);
		}

	//	f.setBold(false);
	//	f.setPointSize(8);
	//	painter.setFont(f);


//		if (sliceProportion<sliceProportionLineThreshhold)
		{
			painter.setPen(iter->color);
			QPoint p1(ellipseCenterX+cos(sliceRadiansCenter)*(fontHeightWidthFix+diameter/2),ellipseCenterY+sin(sliceRadiansCenter)*(fontHeightWidthFix+diameter/2));
			QPoint p2(textCenterX,textCenterY);
			p2 = (p2 - p1) *.75 + p1;
			QPoint p3 = p2-p1;
			if (p3.x()*p3.x()+p3.y()*p3.y()>10*10)
				painter.drawLine(p1,p2);
		//	painter.fillRect(textRect,Qt::white);
			//	painter.eraseRect(textRect);
		}
		painter.setPen(Qt::black);
		//painter.setClipping(false);
		painter.drawText(renderTextX1, renderTextY1, iter->label);
		painter.drawText(renderTextX2, renderTextY2, percentage);


		startAngle+=360.0*sliceProportion*16.0;
		startProportion-=sliceProportion;
		iter++;
		index++;
	}

	// painter.drawEllipse(ellipseLeft, ellipseTop, diameter, diameter);

}
