#include "PieWindow.h"
#include <qdrawutil.h>
#include "EchoChamber.h"
#include "PieWidget.h"

extern EchoChamber *echoChamber;	

/*
void ColorDecorationFromModel::paint(QPainter *painter, const QStyleOptionViewItem &option,
									  const QModelIndex &index) const
{
	QColor color;
	if (dataColorColumn>=0)
	{
		QModelIndex colorIndex = model->index(index.row(), dataColorColumn);
		color = QColor(model->data(colorIndex,
			Qt::DecorationRole).toString());
	}
	else
	{
		color=QColor::colorNames()[index.row()];
	}

	int x,y,w,h;
	x=option.rect.x();
	y=option.rect.y();
	w=option.rect.right()-option.rect.x();
	h=option.rect.bottom()-option.rect.y();
	if (drawAsPrefix)
	{
		qDrawPlainRect ( painter, x,y,h,h, QColor(0,0,0), 1, &QBrush(color) );
		QStyleOptionViewItem optionModified=option;
		optionModified.rect.setX(x+h);
		QItemDelegate::paint(painter, optionModified, index);	
	}
	else
	{
		qDrawPlainRect ( painter, option.rect.x(), option.rect.y(), option.rect.right()-option.rect.x(), option.rect.bottom()-option.rect.y(), color, 1, &QBrush(color) );
		QItemDelegate::paint(painter, option, index);	
	}


	
}
*/

PieWindow::PieWindow(QWidget* parent, Qt::WindowFlags f)
: QMainWindow(parent,f)
{
	setupUi(this);

}

void PieWindow::InitWithDb(QStringList queries, QStringList databaseNames)
{
	for (int i=0; i < databaseNames.size(); i++)
		echoChamber->InitAMCWithDB(GetAnimatedModelAtIndex(i), queries[i], databaseNames[i]);

	int labelColumn=1;
	int valueColumn=0;
	int colorColumn=-1;
	disableRecalcuate=false;

	//	pvwd1 = new ColorDecorationFromModel(this,GetModel(),colorColumn);
	//	pvwd1->SetDrawAsPrefix(true);
	//pieView->setItemDelegate(pvwd1);

	pieChartLayout->setSpacing(0);
	pieChartLayout->setMargin(0);
	pieWidget = new PieWidget(this);
	pieChartLayout->addWidget(pieWidget);

	QObject::connect(labelColumnCombobox, SIGNAL(currentIndexChanged(QString)), pieWidget, SLOT(repaint()));
	QObject::connect(valueColumnCombobox, SIGNAL(currentIndexChanged(QString)), pieWidget, SLOT(repaint()));
	QObject::connect(colorColumnCombobox, SIGNAL(currentIndexChanged(QString)), pieWidget, SLOT(repaint()));

	SetupComboBoxes();
	Recalcuate();
	show();
}
PieWindow::~PieWindow()
{
//	delete pvwd1;
	delete pieWidget;
}
void PieWindow::on_labelColumnCombobox_currentIndexChanged(QString string)
{
	Recalcuate();
	pieWidget->repaint();
}
void PieWindow::on_valueColumnCombobox_currentIndexChanged(QString string)
{
	Recalcuate();
	pieWidget->repaint();
}
void PieWindow::on_colorColumnCombobox_currentIndexChanged(QString string)
{
	Recalcuate();
	pieWidget->repaint();
}
void PieWindow::SetupComboBoxes(void)
{
	QStringList numericalColumnNames;
	QList<int> numericalColumnIndices;
	QStringList textColumnNames;
	QList<int> textColumnIndices;
	QStringList allColumnNames;
	int timestampColumnIndex;
	disableRecalcuate=true;
	echoChamber->GetColumnTypes(GetModel(), allColumnNames, numericalColumnNames, numericalColumnIndices, textColumnNames, textColumnIndices, timestampColumnIndex);

	for (int i=0; i < allColumnNames.size(); i++)
		labelColumnCombobox->addItem(allColumnNames[i]);
	bool defaultWasSet;
	echoChamber->SetupRGBComboBox(GetModel(), colorColumnCombobox, &defaultWasSet);
	echoChamber->SetupIntensityComboBox(GetModel(), valueColumnCombobox);
	colorColumnCombobox->addItem("<AutoSelect>");
	if (defaultWasSet==false)
		colorColumnCombobox->setCurrentIndex(colorColumnCombobox->count()-1);
	disableRecalcuate=false;
}
void PieWindow::on_actionEdit_SQL_triggered(bool b)
{
// 	QString str = echoChamber->QueryForSQLEdit(GetModel(), dbName);
// 	if (str.isEmpty()==false)
// 	{
// 		tableRowQuery.Preparse(GetModel());
// 		Recalcuate();
// 		pieWidget->repaint();
// 	}

	QStringList stringListQueries, stringListDatabases;
	stringListQueries.push_back(GetModel()->query().lastQuery());
	stringListDatabases.push_back(GetDBName());
	echoChamber->SpawnSQLQueryTool(stringListQueries, stringListDatabases);
}
void PieWindow::on_actionEdit_Tick_triggered(bool b)
{
	QString str = echoChamber->QueryForTickEdit(this);
}
void PieWindow::on_actionShow_Columns_toggled(bool b)
{
	dockWidget->setVisible(b);
}
void PieWindow::SetAnimationRowIndices(int from, int to, double valueBefore, double valueAfter)
{
	AnimatedModel::SetAnimationRowIndices(from,to,valueBefore,valueAfter);
	pieWidget->repaint();

}
void PieWindow::ClearAnimationRowIndices(void)
{
	ClearAnimationRowIndices();
	pieWidget->repaint();
}
void PieWindow::Save(XMLSaver &xmlSaver)
{
	// this is always first
	SaveType(xmlSaver);
	SaveWindowTitle(xmlSaver,this);
	SaveVersion(xmlSaver);


	QStringList databaseNames;
	QStringList queries;
	databaseNames.push_back(GetDBName());
	queries.push_back(GetModel()->query().lastQuery());
	XML_SAVER_SAVE_ELEMENT(xmlSaver, databaseNames);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, queries);

	QByteArray geo_data = saveGeometry();
	QByteArray layout_data = saveState();
	XML_SAVER_SAVE_ELEMENT(xmlSaver, geo_data);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, layout_data);

	GetAnimatedModelAtIndex(0)->AMC_Save(xmlSaver);

//	EchoChamber::SaveQtColorComboBox(colorColumnCombobox, dataStream);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, colorColumnCombobox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, labelColumnCombobox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, valueColumnCombobox);
}
void PieWindow::Load(XMLSaver &xmlSaver)
{
	int version;
	LoadVersion(xmlSaver, version);
	QStringList databaseNames;
	QStringList queries;
	XML_SAVER_LOAD_ELEMENT(xmlSaver, databaseNames, false);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, queries, false);
	for (int i=0; i < databaseNames.size(); i++)
		echoChamber->InitAMCWithDB(GetAnimatedModelAtIndex(i), queries[i], databaseNames[i]);


	QByteArray geo_data;
	QByteArray layout_data;
	XML_SAVER_LOAD_ELEMENT(xmlSaver, geo_data, false);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, layout_data, false);
	restoreGeometry(geo_data);
	restoreState(layout_data);

	GetAnimatedModelAtIndex(0)->AMC_Load(xmlSaver);
	SetupComboBoxes();

	XML_SAVER_LOAD_ELEMENT(xmlSaver, colorColumnCombobox, false);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, labelColumnCombobox, false);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, valueColumnCombobox, false);

	actionShow_Columns->setChecked(dockWidget->isVisible());

	Recalcuate();
	pieWidget->repaint();
}
void PieWindow::Recalcuate(void)
{
	if (disableRecalcuate)
		return;

	// Cull out to positive values
	QList<Row>::iterator iter;
	QList<Row>::iterator largestIter;
	int dataIndex;
	GetAnimationRowIndices(&rowStart, &rowEnd, 99999999);
	const int maxPossibleRows=50;
	int colorColumn = EchoChamber::GetColumnIndex(GetModel(), colorColumnCombobox->currentText());
	int labelColumn = EchoChamber::GetColumnIndex(GetModel(), labelColumnCombobox->currentText());
	int valueColumn = EchoChamber::GetColumnIndex(GetModel(), valueColumnCombobox->currentText());
	unsortedRows.clear();
	sortedRows.clear();

	Row row;
	srand(0);
	sumOfValues=0.0;
	int rowIndex;
	for (rowIndex=0, dataIndex=rowStart; dataIndex<rowEnd && rowIndex < maxPossibleRows; dataIndex++)
	{
		row.value=GetModel()->data(GetModel()->index(dataIndex, valueColumn)).toDouble();
		if (row.value>0.0)
		{
			row.color=QColor();
			row.label=GetModel()->data(GetModel()->index(dataIndex, labelColumn)).toString();
			if (colorColumn!=-1)
			{
				QVariant variant = GetModel()->data(GetModel()->index(dataIndex, colorColumn));
				if (variant.isNull()==false)
					row.color.setNamedColor(variant.toString());
			}

			if (row.color.isValid()==false)
			{
				row.color.setRed(rand()%256);
				row.color.setBlue(rand()%256);
				row.color.setGreen(rand()%256);
			}

			bool wasInserted=false;
			// If label is duplicate, add to it
			iter=unsortedRows.begin();
			while (iter!=unsortedRows.end())
			{
				if (iter->label==row.label)
				{
					iter->value+=row.value;
					sumOfValues+=row.value;
					wasInserted=true;
					break;
				}
				iter++;
			}

			if (wasInserted==false)
			{
				rowIndex++;
				unsortedRows.insert(iter,row);
			}
		}
	}

	// Sort the rows. Use a simple sort
	while (unsortedRows.length())
	{
		iter=unsortedRows.begin();
		largestIter=iter;
		double iterValue=iter->value;
		double largestValue=iterValue;
		iter++;
		while (iter!=unsortedRows.end())
		{
			iterValue=iter->value;
			if (iterValue > largestValue)
			{
				largestValue=iter->value;
				largestIter=iter;
				break;
			}
			iter++;
		}

		sortedRows.push_back(*largestIter);
		unsortedRows.erase(largestIter);
	}
}
