#include "PointSorter.h"
#include <assert.h>
#include "DS_Multilist.h"

struct RenderablePointHolder
{
	RenderablePointHolder() {}
	RenderablePointHolder(RenderablePoint *p) : renderablePoint(p) {}
	RenderablePoint *renderablePoint;
};
bool operator<=( const RenderablePointHolder &p1, const RenderablePointHolder &p2 )
{
	return p1.renderablePoint->distance <= p2.renderablePoint->distance;
}
bool operator<( const RenderablePointHolder &p1, const RenderablePointHolder &p2 )
{
	return p1.renderablePoint->distance < p2.renderablePoint->distance;
}
bool operator>( const RenderablePointHolder &p1, const RenderablePointHolder &p2 )
{
    	return p1.renderablePoint->distance > p2.renderablePoint->distance;
}
bool operator>=( const RenderablePointHolder &p1, const RenderablePointHolder &p2 )
{
	return p1.renderablePoint->distance >= p2.renderablePoint->distance;
}
bool operator==( const RenderablePointHolder &p1, const RenderablePointHolder &p2 )
{
	return p1.renderablePoint->distance == p2.renderablePoint->distance;
}
void PointSorter::AddPoint(const RenderablePoint &renderablePoint)
{
	pointList[addPointIndex]=renderablePoint;
	pointList[addPointIndex].distance=cameraBackplane.PointToPlaneDistSq(pointList[addPointIndex].point);
	++addPointIndex;
}
void PointSorter::SortPoints(void)
{
	batches.clear();
	if (addPointIndex==0)
		return;

	// Add all pointers, to prevent needless copying around. Plus the distance value isn't copied anyway
	DataStructures::Multilist<ML_STACK, RenderablePointHolder> multilist;
	multilist.Reallocate(addPointIndex);
	int i;
	for (i=0; i < addPointIndex; ++i)
		multilist.Push(RenderablePointHolder(pointList+i));
	multilist.SetSortOrder(false);
	multilist.Sort(true);

	// Allocate a new list, copy data over
	RenderablePoint *sortedPointList = new RenderablePoint[addPointIndex];
	for (i=0; i < addPointIndex; ++i)
		sortedPointList[i]=* (multilist[i].renderablePoint);
	delete [] pointList;
	pointList=sortedPointList;

	// Now figure out the batches
	batches.push_back(0);
	for (i=1; i < addPointIndex; ++i)
	{
		if (IsBatchable(pointList[i], pointList[i-1])==false)
			batches.push_back(i);
	}
	batches.push_back(addPointIndex);
}
int PointSorter::GetBatchCount(void) const
{
	if (batches.isEmpty())
		return 0;
	return batches.size()-1;
}
int PointSorter::GetBatchSize(int batchCountIndex)
{
	return batches[batchCountIndex+1]-batches[batchCountIndex];
}
RenderablePoint* PointSorter::GetBatch(int batchCountIndex)
{
	return pointList+batches[batchCountIndex];
}
