#ifndef __POINT_SORTER_H
#define __POINT_SORTER_H

#include "QDataTypes3D.h"
#include <QVector>

struct ScatterPlotFrameInput;

struct RenderablePoint
{
public:
/**
	RenderablePoint() {}
	RenderablePoint(const QPoint3F &_point, unsigned char _pointColor[4], int _pointSize, GLuint _textureId) :
	point(_point), pointSize(_pointSize), textureId(_textureId) {
		pointColor[0]=_pointColor[0];
		pointColor[1]=_pointColor[1];
		pointColor[2]=_pointColor[2];
		pointColor[3]=_pointColor[3];
	}

	explicit RenderablePoint(const RenderablePoint & input) :
		point( input.point ),
		pointSize( input.pointSize ),
		textureId( input.textureId ) {
		pointColor[0]=input.pointColor[0];
		pointColor[1]=input.pointColor[1];
		pointColor[2]=input.pointColor[2];
		pointColor[3]=input.pointColor[3];
	}
	RenderablePoint& operator = ( const RenderablePoint& input )
	{
		point=input.point;
		pointSize=input.pointSize;
		textureId=input.textureId;
		pointColor[0]=input.pointColor[0];
		pointColor[1]=input.pointColor[1];
		pointColor[2]=input.pointColor[2];
		pointColor[3]=input.pointColor[3];
		return *this;
	}
	// public for easy access
	QPoint3F point;
	unsigned char pointColor[4];
	int pointSize;
	GLuint textureId;
/*/

//*/
	RenderablePoint() : frameInput( NULL ) {}
	explicit RenderablePoint(const QPoint3F &_point, unsigned char _pointColor[4], ScatterPlotFrameInput * const _frameInput) :
	point(_point), frameInput(_frameInput) {
		pointColor[0]=_pointColor[0];
		pointColor[1]=_pointColor[1];
		pointColor[2]=_pointColor[2];
		pointColor[3]=_pointColor[3];
	}

	explicit RenderablePoint(const RenderablePoint & input) :
        point( input.point ),
        frameInput( input.frameInput )
        {
		pointColor[0]=input.pointColor[0];
		pointColor[1]=input.pointColor[1];
		pointColor[2]=input.pointColor[2];
		pointColor[3]=input.pointColor[3];
	}
	RenderablePoint & operator = ( const RenderablePoint& input )
	{
		point=input.point;
                frameInput=input.frameInput;
		pointColor[0]=input.pointColor[0];
		pointColor[1]=input.pointColor[1];
		pointColor[2]=input.pointColor[2];
		pointColor[3]=input.pointColor[3];
		return *this;
	}
        
	// public for easy access
	QPoint3F point;
	unsigned char pointColor[4];
        ScatterPlotFrameInput * frameInput;
//*/

	// Calculated when PointSorter::AddPoint is called.
	// Only PointSorter needs to know this, so not copied in copy constructor
	float distance;
};

class PointSorter
{
public:
	// Initialize with the maximum number of points to be added, and the camera backplane
	PointSorter(int maxPoints, QPlane _cameraBackplane ) : cameraBackplane(_cameraBackplane) {
		pointList = new RenderablePoint[maxPoints];
		addPointIndex=0;
	}

	~PointSorter() {delete pointList;}

	// Add a point. Distance is calculated at this time
	void AddPoint(const RenderablePoint &renderablePoint);

	// Sort points. Call once all calls to AddPoint is done. Sorted from greatest distance to least
	void SortPoints(void);

	// Return number of batches, where similar items are defined by IsBatchable()
	// Must call SortPoints before calling GetBatchCount, GetBatchSize, GetBatch
	int GetBatchCount(void) const;

	// Returns number of similar items for a given batch, where similar items are defined by IsBatchable()
	int GetBatchSize(int batchCountIndex);

	// Returns items for a given batch, where similar items are defined by IsBatchable()
	RenderablePoint* GetBatch(int batchCountIndex);

	// If passes, two points can be drawn in a point list back to back
	inline bool IsBatchable(const RenderablePoint &p1, const RenderablePoint &p2)
	{
		//return p1.pointSize==p2.pointSize && p1.textureId==p2.textureId;
            return p1.frameInput == p2.frameInput;
	}

protected:
	QPlane cameraBackplane;
	RenderablePoint *pointList;
	// Where to write to in pointList, the next time AddPoint is called
	int addPointIndex;

	/// int is index into pointList for this batch
	/// Size of batches array is 1 greater than the actual number of batches, so the last length can be returned consistently
	QVector<int> batches;

};

#endif
