#ifndef __POSTFIX_TABLE_ROW_QUERY_H
#define __POSTFIX_TABLE_ROW_QUERY_H

#include <QString>
#include <QList>
#include "PostfixTableRowQuery.h"
#include "QSqlQueryModelFetchAll.h"
#include "XMLSaver.h"

#define POSTFIX_TABLE_ROW_QUERY_AND "AND"
#define POSTFIX_TABLE_ROW_QUERY_OR "OR"
#define POSTFIX_TABLE_ROW_QUERY_CHANGED "CHANGED"
#define POSTFIX_TABLE_ROW_QUERY_SAME "SAME"
#define POSTFIX_TABLE_ROW_TOKEN_SEPARATOR ','

class PostfixTableRowQuery
{
public:
	PostfixTableRowQuery();
	~PostfixTableRowQuery();

	// Returns true on a valid query, false on invalid
	bool SetQuery(const QString &query);

	// Call before GetNextRowStepIndexInclusive so all row checks are valid
	bool Preparse(const QSqlQueryModelFetchAll *model);

	bool CanRowStep(void) const {return currentQuery.isEmpty()==false;}

	// For single step
	// Call Preparse before using
	int GetNextRowStepIndexInclusive(QSqlQueryModelFetchAll *model, int lastUsedRow, bool forward);

	QString GetTickDefinition(void) const {return queryString;}

	struct QueryToken
	{
		enum QueryTokenOperation
		{
			COLUMN_CHANGED,
			COLUMN_SAME,
			AND,
			OR,
			QTRUE,
			QFALSE, // Can't use TRUE and FALSE because of stupid global defines from Windows
			NONE,
		} operation;

		QueryToken() {}
		QueryToken(QString cn, QueryTokenOperation op) : operation(op), columnName(cn) {}
		QString columnName;
		int columnIndex;
	};

	static bool EvaluateQuery(const QString &query, QList<QueryToken> &tokenList);

	void Save(XMLSaver &xmlSaver);
	void Load(XMLSaver &xmlSaver);

protected:
	bool ExpressionPassed(const QSqlQueryModelFetchAll *model, int curRow, int otherRow);
	bool ColumnChanged(const QSqlQueryModelFetchAll *model, int curRow, int otherRow, int column) const;
	void EvalColumn(const QSqlQueryModelFetchAll *model, int curRow, int otherRow, int curIndex, QList<PostfixTableRowQuery::QueryToken> &workingQuery);
	QString queryString;
	QList<QueryToken> currentQuery;
};

#endif
