#include "QwtWindow.h"
#include <QFileDialog>
#include "QwtWindowTab.h"
#include <qwt_painter.h>
#include <QTimer>
#include <assert.h>
#include "EchoChamber.h"
#include "qwt_scale_engine.h"
#include "qwt_scale_widget.h"
#include "qwt_legend.h"
#include "qwt_legend_item.h"
#include "qwt_plot_zoomer.h"
#include "qwt_plot_panner.h"

extern EchoChamber *echoChamber;

const unsigned int c_rangeMax = 1000;

Zoomer::Zoomer(QwtPlotCanvas *canvas):
ScrollZoomer(canvas)
{
}

void Zoomer::rescale()
{
  QwtScaleWidget *scaleWidget = plot()->axisWidget(yAxis());
  QwtScaleDraw *sd = scaleWidget->scaleDraw();

  int minExtent = 0;
  if ( zoomRectIndex() > 0 )
  {
	  // When scrolling in vertical direction
	  // the plot is jumping in horizontal direction
	  // because of the different widths of the labels
	  // So we better use a fixed extent.

	  minExtent = sd->spacing() + sd->majTickLength() + 1;
	  minExtent += sd->labelSize(
		  scaleWidget->font(), c_rangeMax).width();
  }

  sd->setMinimumExtent(minExtent);

  ScrollZoomer::rescale();
}
// Spectogram
// qwt_plot_spectrogram.h
// http://qwt.sourceforge.net/spectrogramscreenshots.html

// Tip on blinking
// http://lists.trolltech.com/qt-interest/2004-09/thread00520-0.html

QwtText QwtModelScaleDraw::label(double value) const
{
	if (amc)
	{
		// Map extents of scale to number of rows in the table
		const QwtScaleDiv &sd = scaleDiv();
		sd.lowerBound();
		sd.upperBound();
		sd.range();
		float proportion = (value - sd.lowerBound()) / sd.range();
		int numRows = amc->GetModel()->realRowCount();
		int rowToUse = (int) (proportion * (float) numRows);
		int columnIndex = EchoChamber::GetColumnIndex(amc->GetModel(), columnName);
		if (columnIndex==-1)
			return QwtText( QString( "Can't find column" ) );
		QVariant v = amc->GetModel()->data(amc->GetModel()->index(rowToUse, columnIndex));
		return v.toString();
	}
	else
	{
		return QwtScaleDraw::label(value);
	}


}

QwtWindowWatermark::QwtWindowWatermark()
{
	setZ(0.0);
}

void QwtWindowWatermark::SetPixmap(const QPixmap &pixmap)
{
	brush.setTexture(pixmap);
}

int QwtWindowWatermark::rtti() const
{
	return QwtPlotItem::Rtti_PlotUserItem + 123;
}

void QwtWindowWatermark::draw(QPainter *painter,
							  const QwtScaleMap &, const QwtScaleMap &,
							  const QRect &rect) const
{
	QwtPainter::fillRect(painter, rect, brush);
	// QwtPainter::drawText(painter, rect, ...);
}

QwtWindow::QwtWindow(QWidget* parent, Qt::WindowFlags f)
: QMainWindow(parent)
{
	setupUi(this);


}

void QwtWindow::InitWithDb(QStringList queries, QStringList databaseNames, int _initialMode)
{
	newTabIndex=2;
	QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Expanding);
	qwtPlot = new QwtPlot;
	sizePolicy1.setHorizontalStretch(0);
	sizePolicy1.setVerticalStretch(0);
	sizePolicy1.setHeightForWidth(qwtPlot->sizePolicy().hasHeightForWidth());
	qwtPlot->setSizePolicy(sizePolicy1);
	verticalLayout_9->addWidget(qwtPlot);
	QObject::connect(qwtPlot, SIGNAL(legendClicked(QwtPlotItem*)), this, SLOT(onLegendClicked(QwtPlotItem*)));


	qwtPlot->setCanvasBackground("White");
	// If autodelete is true, then children are deleted when the plot is deleted.
	// This causes tabs to crash because tabs have to delete their items in the destructor or else it keeps redrawing when the tab is closed.
	qwtPlot->setAutoDelete(false);
	//	graphTitleLineEdit->setText(_query.lastQuery());

	//mainToolBar->setVisible(false);

	initialMode=_initialMode;

	//	qwtPlot->axisScaleEngine();

	qwtPlot->axisScaleEngine(QwtPlot::xBottom)->setAttributes(QwtScaleEngine::Floating);
	qwtPlot->axisScaleEngine(QwtPlot::yLeft)->setAttributes(QwtScaleEngine::Floating);
	qwtPlot->setAxisAutoScale(QwtPlot::xBottom);
	qwtPlot->setAxisAutoScale(QwtPlot::yLeft);

	/*
	qwtPlot->axisScaleEngine(QwtPlot::xBottom)->setAttribute(QwtScaleEngine::Floating,true);
	qwtPlot->axisScaleEngine(QwtPlot::yLeft)->setAttribute(QwtScaleEngine::Floating,true);
	qwtPlot->axisScaleEngine(QwtPlot::xBottom)->setAttribute(QwtScaleEngine::Floating,true);
	qwtPlot->axisScaleEngine(QwtPlot::yLeft)->setAttribute(QwtScaleEngine::Floating,true);
	qwtPlot->setAxisAutoScale(QwtPlot::xBottom);
	qwtPlot->axisScaleEngine(QwtPlot::xBottom)->setMargins(0,0);
	qwtPlot->axisScaleEngine(QwtPlot::yLeft)->setMargins(0,0);
	qwtPlot->setAxisAutoScale(QwtPlot::yLeft);
	*/

	// void setAxisScaleDraw(int axisId, QwtScaleDraw *);
	// const QwtScaleDraw *axisScaleDraw(int axisId) const;
	// QwtScaleDraw *axisScaleDraw(int axisId);

	yLeftScaleDraw = new QwtModelScaleDraw;
	xBottomScaleDraw = new QwtModelScaleDraw;
	qwtPlot->setAxisScaleDraw(QwtPlot::yLeft, yLeftScaleDraw);
	qwtPlot->setAxisScaleDraw(QwtPlot::xBottom, xBottomScaleDraw);
	leftScaleComboBox->addItem("<Default>");
	bottomScaleComboBox->addItem("<Default>");

	qwtPlot->setAxisMaxMajor(QwtPlot::yLeft,5);
	qwtPlot->setAxisMaxMajor(QwtPlot::xBottom,5);

	for (int i=0; i < databaseNames.size(); i++)
	{
		if (i>0)
			AddTab(queries[i], databaseNames[i]);
		else
			AddQwtWindowTabToNewestTabWidget(queries[i], databaseNames[i]);
	}

	QStringList numericalColumnNames;
	QList<int> numericalColumnIndices;
	QStringList textColumnNames;
	QList<int> textColumnIndices;
	QStringList allColumnNames;
	int timestampColumnIndex;
	// todo
	//for (int i=0; i < databaseNames.size(); i++)
	for (int i=0; i < 1 && databaseNames.size(); i++)
	{
		echoChamber->GetColumnTypes(GetAnimatedModelAtIndex(i)->GetModel(), allColumnNames, numericalColumnNames, numericalColumnIndices, textColumnNames, textColumnIndices, timestampColumnIndex);
		for (int i=0; i < allColumnNames.size(); i++)
		{
			leftScaleComboBox->addItem(allColumnNames[i]);
			bottomScaleComboBox->addItem(allColumnNames[i]);
		}
	}

	QPen penBlue, penRed;
	penBlue.setColor(Qt::blue);
	penRed.setColor(Qt::red);
	UpdateAll();
	//dockWidget->hide();
	actionPoints->setChecked(true);
	dockWidget_2->hide();
	markerBefore.attach(qwtPlot);
	markerBefore.hide();
	markerBefore.setAxis(QwtPlot::xBottom, QwtPlot::yLeft);
	markerBefore.setLineStyle(QwtPlotMarker::VLine);
	markerBefore.setLinePen(penBlue);
	markerAfter.attach(qwtPlot);
	markerAfter.hide();
	markerAfter.setAxis(QwtPlot::xBottom, QwtPlot::yLeft);
	markerAfter.setLineStyle(QwtPlotMarker::VLine);
	markerAfter.setLinePen(penRed);

	/*
	const QwtScaleDiv *scaleDivHorizontal = qwtPlot->axisScaleDiv(QwtPlot::xBottom);
	const QwtScaleDiv *scaleDivVertical = qwtPlot->axisScaleDiv(QwtPlot::yLeft);
	double horizontalLowerBound=scaleDivHorizontal->lowerBound();
	double horizontalUpperBound=scaleDivHorizontal->upperBound();
	double verticalLowerBound=scaleDivVertical->lowerBound();
	double verticalUpperBound=scaleDivVertical->upperBound();
	double horizontalStepSize, verticalStepSize;
	qwtPlot->axisScaleEngine(QwtPlot::xBottom)->autoScale(100,horizontalLowerBound,horizontalUpperBound,horizontalStepSize);
	qwtPlot->axisScaleEngine(QwtPlot::yLeft)->autoScale(100,verticalLowerBound,verticalUpperBound,verticalStepSize);
	*/
	// 

	zoomer = new Zoomer(qwtPlot->canvas());
	//zoomer->setRubberBandPen(QPen(Qt::red, 2, Qt::DotLine));
	//zoomer->setTrackerPen(QPen(Qt::red));

	const QColor c(Qt::darkBlue);
	zoomer->setRubberBandPen(c);
	zoomer->setTrackerPen(c);

#if QT_VERSION < 0x040000
	zoomer->setMousePattern(QwtEventPattern::MouseSelect2,
		Qt::RightButton, Qt::ControlButton);
#else
	zoomer->setMousePattern(QwtEventPattern::MouseSelect2,
		Qt::RightButton, Qt::ControlModifier);
#endif
	zoomer->setMousePattern(QwtEventPattern::MouseSelect3,
		Qt::RightButton);

	QObject::connect(zoomer, SIGNAL(zoomed(QwtDoubleRect)), this, SLOT(onZoomed(QwtDoubleRect)));

	legend=0;
	UpdateLegendVisibility();

	qwtPlot->show();
	Replot();

	resize(512,512);
	show();
	ResizeContents(this,qwtPlot);
	//zoomer->setZoomBase(true);
}
QwtWindow::~QwtWindow()
{
	delete zoomer;
}
void QwtWindow::Replot(void)
{
	if (inReplot==true)
	inReplot=true;
	qwtPlot->replot();
	inReplot=false;
}
QwtWindowTab * QwtWindow::AddQwtWindowTabToNewestTabWidget(QString query, QString database)
{
	int newestWidgetIndex=tabWidget->count()-1;
	tabWidget->widget(newestWidgetIndex)->setAttribute(Qt::WA_DeleteOnClose, true);
	QVBoxLayout *tabLayout = new QVBoxLayout(tabWidget->widget(newestWidgetIndex));
	tabLayout->setSpacing(0);
	tabLayout->setMargin(0);
	QwtWindowTab *sppt = new QwtWindowTab(tabWidget->widget(newestWidgetIndex),0, this, query, database);
	sppt->tabWidget->setCurrentIndex(initialMode);
	sppt->UpdateBasedOnTabWidget();
	sppt->setAttribute(Qt::WA_DeleteOnClose, true);
	tabLayout->addWidget(sppt);
	windowTabs.push_back(sppt);
	if (sppt->colorComboBox->count()>2)
	{
		sppt->colorComboBox->setCurrentIndex(newestWidgetIndex % (sppt->colorComboBox->count()-1));
		sppt->on_colorComboBox_activated(sppt->colorComboBox->currentColor());
	}
	return sppt;
}
void QwtWindow::UpdateAll(void)
{
	UpdateTitle();
	UpdateXAxisTitle();
	UpdateYAxisTitle();
	UpdateGridVisibility();
	UpdateAxesScale();
	UpdateFromXAxisTicksSpinbox();
	UpdateFromYAxisTicksSpinbox();
}
void QwtWindow::UpdateTitle(void)
{
	qwtPlot->setTitle(graphTitleLineEdit->text());
}
void QwtWindow::UpdateXAxisTitle(void)
{
	qwtPlot->setAxisTitle(QwtPlot::xBottom, xAxisLineEdit->text());
}
void QwtWindow::UpdateYAxisTitle(void)
{
	qwtPlot->setAxisTitle(QwtPlot::yLeft, yAxisLineEdit->text());
}
void QwtWindow::on_displayGrid_toggled(bool isSet)
{
	UpdateGridVisibility();
	Replot();
}
void QwtWindow::on_xAxisMinSpinBox_valueChanged(double d)
{
	qwtPlot->setAxisScale(QwtPlot::xBottom,xAxisMinSpinBox->value(),xAxisMaxSpinBox->value());
	if (autoscaleAxes->isChecked()==false)
		Replot();
}
void QwtWindow::on_xAxisMaxSpinBox_valueChanged(double d)
{
	qwtPlot->setAxisScale(QwtPlot::xBottom,xAxisMinSpinBox->value(),xAxisMaxSpinBox->value());
	if (autoscaleAxes->isChecked()==false)
		Replot();
}
void QwtWindow::on_yAxisMinSpinBox_valueChanged(double d)
{
	qwtPlot->setAxisScale(QwtPlot::yLeft,yAxisMinSpinBox->value(),yAxisMaxSpinBox->value());
	if (autoscaleAxes->isChecked()==false)
		Replot();
}
void QwtWindow::on_yAxisMaxSpinBox_valueChanged(double d)
{
	qwtPlot->setAxisScale(QwtPlot::yLeft,yAxisMinSpinBox->value(),yAxisMaxSpinBox->value());
	if (autoscaleAxes->isChecked()==false)
		Replot();
}
void QwtWindow::SetBackground(QString fileName)
{
	watermark.attach(qwtPlot);
	QPixmap pm;
	if (pm.load(fileName))
	{
		watermark.SetPixmap(pm);
		Replot();
		clearBackgroundButton->setEnabled(true);
		backgroundFilename=fileName;
	}
}
void QwtWindow::on_backgroundButton_released()
{
	QString fileName = QFileDialog::getOpenFileName(this,
		tr("Open Image"), QString(), tr("Image Files (*.png *.jpg *.bmp *.dds)"));
	if (fileName.isEmpty()==false)
	{
		SetBackground(fileName);
	}
}
void QwtWindow::on_clearBackgroundButton_released()
{
	backgroundFilename.clear();
	watermark.detach();	
	clearBackgroundButton->setDisabled(true);
	Replot();
}
void QwtWindow::on_autoscaleAxes_toggled(bool isSet)
{
	if (isSet==false)
	{
		// Set axes to current values
		const QwtScaleDiv *scaleDivHorizontal = qwtPlot->axisScaleDiv(QwtPlot::xBottom);
		const QwtScaleDiv *scaleDivVertical = qwtPlot->axisScaleDiv(QwtPlot::yLeft);
		const double horizontalLowerBound=scaleDivHorizontal->lowerBound();
		const double horizontalUpperBound=scaleDivHorizontal->upperBound();
		const double verticalLowerBound=scaleDivVertical->lowerBound();
		const double verticalUpperBound=scaleDivVertical->upperBound();
		xAxisMinSpinBox->setValue(horizontalLowerBound);
		xAxisMinSpinBox->setSingleStep((horizontalUpperBound-horizontalLowerBound)/10.0);
		xAxisMaxSpinBox->setValue(horizontalUpperBound);
		xAxisMaxSpinBox->setSingleStep((horizontalUpperBound-horizontalLowerBound)/10.0);
		yAxisMinSpinBox->setValue(verticalLowerBound);
		yAxisMinSpinBox->setSingleStep((verticalUpperBound-verticalLowerBound)/10.0);
		yAxisMaxSpinBox->setValue(verticalUpperBound);
		yAxisMaxSpinBox->setSingleStep((verticalUpperBound-verticalLowerBound)/10.0);
	}
	else
	{
		qwtPlot->setAxisAutoScale(QwtPlot::xBottom);
		qwtPlot->setAxisAutoScale(QwtPlot::yLeft);
	}
	//zoomer->setZoomBase(true);
	Replot();

	if (isSet==true)
		zoomer->setZoomBase(false);
}
void QwtWindow::UpdateAxesScale(void)
{
	if (autoscaleAxes->checkState()==Qt::Checked)
	{
		qwtPlot->setAxisAutoScale(QwtPlot::yLeft);
		qwtPlot->setAxisAutoScale(QwtPlot::xBottom);
	}
	else
	{
		qwtPlot->setAxisScale(QwtPlot::xBottom,xAxisMinSpinBox->value(),xAxisMaxSpinBox->value());
		qwtPlot->setAxisScale(QwtPlot::yLeft,yAxisMinSpinBox->value(),yAxisMaxSpinBox->value());
	}
	// Crashes
	// zoomer->setZoomBase(true);
}
void QwtWindow::on_graphTitleLineEdit_textChanged()
{
	UpdateTitle();
}
void QwtWindow::on_xAxisLineEdit_textChanged()
{
	UpdateXAxisTitle();
}
void QwtWindow::on_yAxisLineEdit_textChanged()
{
	UpdateYAxisTitle();
}
void QwtWindow::UpdateGridVisibility(void)
{
	if (displayGrid->checkState()==Qt::Checked)
		qpg.attach(qwtPlot);
	else
		qpg.detach();
}
void QwtWindow::on_addPlotPushButton_released()
{
	on_actionAdd_Tab_activated();
}
void QwtWindow::on_actionAdd_Tab_activated()
{
	int i;
	for (i=0; i < tabWidget->count(); i++)
	{
		if (windowTabs[i]->parent()==tabWidget->currentWidget())
		{
			AddTab(windowTabs[i]->GetAMC()->GetQuery(), windowTabs[i]->GetAMC()->GetDBName());
			return;
		}
	}
}
QwtWindowTab * QwtWindow::AddTab(QString query, QString database)
{
	tab = new QWidget();
	tab->setAttribute(Qt::WA_DeleteOnClose, true);
	tab->setObjectName(QString::fromUtf8("tab"));
	QString tabName;
	tabName.sprintf("P%i",newTabIndex++);
	tabWidget->addTab(tab, tabName);
	int windowTabsSize=windowTabs.size();
	QwtWindowTab *qwtWindowTab=AddQwtWindowTabToNewestTabWidget(query, database );
	return qwtWindowTab;
}
void QwtWindow::on_tabWidget_tabCloseRequested(int tabIndex)
{
	if (tabWidget->count()>1)
	{
		// Close happens the next tick!
		// Have to set a timer to replot because of that
		// Can't just replot in the destructor, because the destructor is also called on global window shutdown, which would crash
		bool wasRemoved=false;
		for (int i=0; i < windowTabs.count(); i++)
		{
			if (windowTabs[i]->parent()==tabWidget->widget(tabIndex))
			{
				wasRemoved=true;
				windowTabs.remove(i);
				break;
			}
		}
		assert(wasRemoved);
		tabWidget->widget(tabIndex)->close();
		tabWidget->removeTab(tabIndex);

		QTimer::singleShot(100, this, SLOT(replot()));
	}
}
void QwtWindow::replot(void)
{
	Replot();
}
void QwtWindow::SetAnimationRowIndices(int from, int to, double valueBefore, double valueAfter)
{
	const QwtScaleDiv *scaleDivHorizontal = qwtPlot->axisScaleDiv(QwtPlot::xBottom);
	const QwtScaleDiv *scaleDivVertical = qwtPlot->axisScaleDiv(QwtPlot::yLeft);
	const double horizontalLowerBound=scaleDivHorizontal->lowerBound();
	const double horizontalUpperBound=scaleDivHorizontal->upperBound();
	const double verticalLowerBound=scaleDivVertical->lowerBound();
	const double verticalUpperBound=scaleDivVertical->upperBound();
	if (valueBefore>=scaleDivHorizontal->lowerBound() && valueBefore<=scaleDivHorizontal->upperBound())
	{
		markerBefore.setValue(valueBefore,verticalLowerBound+(verticalUpperBound-verticalLowerBound)/2);
		markerBefore.show();
	}
	if (valueAfter>=scaleDivHorizontal->lowerBound() && valueAfter<=scaleDivHorizontal->upperBound())
	{
		markerAfter.setValue(valueAfter,verticalLowerBound+(verticalUpperBound-verticalLowerBound)/2);
		markerAfter.show();
	}
	Replot();
}
void QwtWindow::ClearAnimationRowIndices(void)
{
	for (int i=0; i < GetAMCCount(); i++)
	{
		GetAnimatedModelAtIndex(i)->ClearAnimationRowIndices();
	}

	for (int i=0; i < windowTabs.count(); i++)
	{
		windowTabs[i]->RebuildOnAnimationTick();
	}
	markerBefore.hide();
	markerAfter.hide();
	Replot();
}
void QwtWindow::on_actionEditSQL_triggered(bool b)
{
// 	QString str = echoChamber->QueryForSQLEdit(GetAnimatedModelAtIndex(0)->GetModel(), dbName);
// 	if (str.isEmpty()==false)
// 	{
// 		tableRowQuery.Preparse(GetAnimatedModelAtIndex(0)->GetModel());
// 		for (int i=0; i < windowTabs.count(); i++)
// 		{
// 			windowTabs[i]->RebuildAll();
// 		}
// 		Replot();
// 	}

	QStringList stringListQueries, stringListDatabases;
	for (int i=0; i < GetAMCCount(); i++)
	{
		stringListQueries.push_back(GetAnimatedModelAtIndex(i)->GetModel()->query().lastQuery());
		stringListDatabases.push_back(GetAnimatedModelAtIndex(i)->GetDBName());
	}
	echoChamber->SpawnSQLQueryTool(stringListQueries, stringListDatabases);
}
void QwtWindow::on_actionEdit_Tick_triggered(bool b)
{
	// todo
	QString str = echoChamber->QueryForTickEdit(GetAnimatedModelAtIndex(0));
}
void QwtWindow::on_leftScaleComboBox_activated(QString string)
{
	UpdateFromLeftScaleComboBox();
}
void QwtWindow::on_bottomScaleComboBox_activated(QString string)
{
	UpdateFromBottomScaleComboBox();
}
void QwtWindow::on_xAxisMajorTicksSpinbox_valueChanged(int newValue)
{
	UpdateFromXAxisTicksSpinbox();
	Replot();
}
void QwtWindow::on_yAxisMajorTicksSpinbox_valueChanged(int newValue)
{
	UpdateFromYAxisTicksSpinbox();
	Replot();
}
void QwtWindow::on_xAxisMinorTicksSpinbox_valueChanged(int newValue)
{
	UpdateFromXAxisTicksSpinbox();
	Replot();
}
void QwtWindow::on_yAxisMinorTicksSpinbox_valueChanged(int newValue)
{
	UpdateFromYAxisTicksSpinbox();
	Replot();
}
void QwtWindow::Save(XMLSaver &xmlSaver)
{
	// this is always first
	SaveType(xmlSaver);
	SaveWindowTitle(xmlSaver,this);
	SaveVersion(xmlSaver);

	QByteArray geo_data = saveGeometry();
	QByteArray layout_data = saveState();
	XML_SAVER_SAVE_ELEMENT(xmlSaver, geo_data);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, layout_data);

	XML_SAVER_SAVE_ELEMENT(xmlSaver, graphTitleLineEdit);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, xAxisLineEdit);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, yAxisLineEdit);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, displayGrid);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, autoscaleAxes);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, xAxisMinSpinBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, xAxisMaxSpinBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, yAxisMinSpinBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, yAxisMaxSpinBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, xAxisMajorTicksSpinbox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, yAxisMajorTicksSpinbox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, xAxisMinorTicksSpinbox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, yAxisMinorTicksSpinbox);
	xmlSaver.SaveElement("BackgroundFilename",backgroundFilename);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, displayLegendCheckBox);


	for (int i=0; i < windowTabs.count(); i++)
	{
		xmlSaver.CDDown(xmlSaver.MakeDirectory("WindowTabs"));
		windowTabs[i]->Save(xmlSaver);
		xmlSaver.CDUp();
	}

	xmlSaver.SaveElement("leftScaleComboBoxText", leftScaleComboBox->currentText());
	xmlSaver.SaveElement("bottomScaleComboBoxText", bottomScaleComboBox->currentText());

}
void QwtWindow::Load(XMLSaver &xmlSaver)
{
	int version;
	LoadVersion(xmlSaver, version);

	int i;
	for (i=0; i < windowTabs.count(); i++)
		delete windowTabs[i];

	int iVal;
	QByteArray geo_data;
	QByteArray layout_data;
	XML_SAVER_LOAD_ELEMENT(xmlSaver, geo_data, false);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, layout_data, false);
	restoreGeometry(geo_data);
	restoreState(layout_data);

	XML_SAVER_LOAD_ELEMENT(xmlSaver, graphTitleLineEdit, "");
	XML_SAVER_LOAD_ELEMENT(xmlSaver, xAxisLineEdit, "");
	XML_SAVER_LOAD_ELEMENT(xmlSaver, yAxisLineEdit, "");
	XML_SAVER_LOAD_ELEMENT(xmlSaver, displayGrid, false);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, autoscaleAxes, false);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, xAxisMinSpinBox, 0);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, xAxisMaxSpinBox, 0);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, yAxisMinSpinBox, 0);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, yAxisMaxSpinBox, 0);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, xAxisMajorTicksSpinbox, 0);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, yAxisMajorTicksSpinbox, 0);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, xAxisMinorTicksSpinbox, 0);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, yAxisMinorTicksSpinbox, 0);

	QString bf;
	xmlSaver.LoadElement("BackgroundFilename", bf, "");
	if (bf.isEmpty()==false)
		SetBackground(bf);

	XML_SAVER_LOAD_ELEMENT(xmlSaver, displayLegendCheckBox, false);
	UpdateLegendVisibility();

	tabWidget->clear();

	if (xmlSaver.CDDown("WindowTabs"))
	{
		do
		{
			QwtWindowTab * tab = AddTab(QString(), QString());
			tab->Load(xmlSaver);
		}  while (xmlSaver.CDNext("WindowTabs"));
		xmlSaver.CDUp();
	}
	actionPoints->setChecked(dockWidget->isVisible());
	actionBackground->setChecked(dockWidget_2->isVisible());

	QString leftScaleComboBoxText;
	xmlSaver.LoadElement("leftScaleComboBoxText", leftScaleComboBoxText, "");
	QString bottomScaleComboBoxText;
	xmlSaver.LoadElement("bottomScaleComboBoxText", bottomScaleComboBoxText, "");

	if (echoChamber->GetColumnIndex(GetAnimatedModelAtIndex(0)->GetModel(), leftScaleComboBoxText)!=-1)
	{
		EchoChamber::SetCurrentText(leftScaleComboBox,leftScaleComboBoxText);
		UpdateFromLeftScaleComboBox();
	}
	if (echoChamber->GetColumnIndex(GetAnimatedModelAtIndex(0)->GetModel(), bottomScaleComboBoxText)!=-1)
	{
		EchoChamber::SetCurrentText(bottomScaleComboBox,bottomScaleComboBoxText);
		UpdateFromBottomScaleComboBox();
	}


	//zoomer->setZoomBase(true);
	zoomer->setZoomBase(true);
}
void QwtWindow::UpdateFromLeftScaleComboBox( void )
{
	QString comboBoxValue = leftScaleComboBox->currentText();
	int index = echoChamber->GetColumnIndex(GetAnimatedModelAtIndex(0)->GetModel(), comboBoxValue);
	if (index!=-1)
		yLeftScaleDraw->UseModel(GetAnimatedModelAtIndex(0), comboBoxValue);
	else
		yLeftScaleDraw->UseModel(0,QString::null);

	// All this is necessary to force it to replot the axes, which there is no direct way to do
	QwtScaleTransformation *t = new QwtScaleTransformation(QwtScaleTransformation::Log10);
	QwtScaleTransformation *original = yLeftScaleDraw->scaleMap().transformation()->copy();
	yLeftScaleDraw->scaleMap().setTransformation(t);
	Replot();
	yLeftScaleDraw->scaleMap().setTransformation(original);
	Replot();
}

void QwtWindow::UpdateFromBottomScaleComboBox( void )
{
	QString comboBoxValue = bottomScaleComboBox->currentText();
	int index = echoChamber->GetColumnIndex(GetAnimatedModelAtIndex(0)->GetModel(), comboBoxValue);
	if (index!=-1)
		xBottomScaleDraw->UseModel(GetAnimatedModelAtIndex(0), comboBoxValue);
	else
		xBottomScaleDraw->UseModel(0,QString::null);

	// All this is necessary to force it to replot the axes, which there is no direct way to do
	QwtScaleTransformation *t = new QwtScaleTransformation(QwtScaleTransformation::Log10);
	QwtScaleTransformation *original = xBottomScaleDraw->scaleMap().transformation()->copy();
	xBottomScaleDraw->scaleMap().setTransformation(t);
	Replot();
	xBottomScaleDraw->scaleMap().setTransformation(original);
	Replot();
}
void QwtWindow::UpdateFromXAxisTicksSpinbox(void)
{
	qwtPlot->setAxisMaxMajor(QwtPlot::xBottom, xAxisMajorTicksSpinbox->value());
	qwtPlot->setAxisMaxMinor(QwtPlot::xBottom, xAxisMinorTicksSpinbox->value());
}
void QwtWindow::UpdateFromYAxisTicksSpinbox(void)
{
	qwtPlot->setAxisMaxMajor(QwtPlot::yLeft, yAxisMajorTicksSpinbox->value());
	qwtPlot->setAxisMaxMinor(QwtPlot::yLeft, yAxisMinorTicksSpinbox->value());
}
void QwtWindow::RenameTab(QString newTabName, QwtWindowTab *tab)
{
	int i;
	for (i=0; i < tabWidget->count(); i++)
	{
		if (tabWidget->widget(i)==tab->parent())
		{
			if (newTabName.isEmpty()==false)
			{
				tabWidget->setTabText(i, newTabName);
			}
			else
			{
				QString tabName;
				tabName.sprintf("P%i",i+1);
				tabWidget->setTabText(i, tabName);
			}
			return;
		}
	}
}
QString QwtWindow::GetTabName(QwtWindowTab *tab)
{
	int i;
	for (i=0; i < tabWidget->count(); i++)
	{
		if (tabWidget->widget(i)==tab->parent())
		{
			return tabWidget->tabText(i);
		}
	}
	return QString::null;
}
bool QwtWindow::HasDefaultTabName(QwtWindowTab *tab)
{
	int i;
	for (i=0; i < tabWidget->count(); i++)
	{
		if (tabWidget->widget(i)==tab->parent())
		{
			QString curName = tabWidget->tabText(i);
			return curName.length()==2 && curName[0]=='P' && curName[1].isDigit();
		}
	}
	return true;
}
void QwtWindow::on_displayLegendCheckBox_toggled(bool b)
{
	UpdateLegendVisibility();
}
void QwtWindow::UpdateLegendVisibility()
{
	if (displayLegendCheckBox->isChecked() && legend==0)
	{
		legend = new QwtLegend;
		legend->setItemMode(QwtLegend::ClickableItem);
		legend->setDisplayPolicy(QwtLegend::AutoIdentifier, QwtLegendItem::ShowLine | QwtLegendItem::ShowSymbol | QwtLegendItem::ShowText);
		legend->setFrameStyle(QFrame::Box|QFrame::Sunken);
		qwtPlot->insertLegend(legend, QwtPlot::BottomLegend);
		ResizeContents(this,qwtPlot);
	}
	else if (displayLegendCheckBox->isChecked()==false && legend!=0)
	{
		delete legend;
		legend=0;
		ResizeContents(this,qwtPlot);
	}
}
void QwtWindow::onLegendClicked(QwtPlotItem *pi)
{
	int i,j;
	for (i=0; i < windowTabs.size(); i++)
	{
		if (windowTabs[i]->HasPlotItem(pi))
		{
			tabWidget->setCurrentWidget((QWidget*)(windowTabs[i]->parent()));
			return;
		}
	}
}
void QwtWindow::onZoomed(const QwtDoubleRect &rect)
{
	autoscaleAxes->setChecked(false);
	on_autoscaleAxes_toggled(false);
}
QString QwtWindow::GetDBName(void) const {return windowTabs[0]->GetAMC()->GetDBName();}
AnimatedModel* QwtWindow::GetAnimatedModelAtIndex(int index) {return windowTabs[index]->GetAMC();}
int QwtWindow::GetAMCCount(void) const {return windowTabs.size();}