#ifndef QWTWINDOW_H
#define QWTWINDOW_H

#include <QWidget>
#include "QSqlQueryModelFetchAll.h"
#include <QSqlQuery>
#include "ui_QwtWindow.h"
#include <qwt_plot_grid.h>
#include <qwt_plot_curve.h>
#include <qwt_plot_marker.h>
#include "WindowAdjustments.h"
#include "AnimatedModelContainer.h"
#include "ISaveableWindow.h"
#include "qwt_scale_draw.h"
#include "qwt_plot.h"
#include "scrollzoomer.h"

using namespace Ui;

class QwtWindowTab;

class Zoomer: public ScrollZoomer
{
public:
	Zoomer(QwtPlotCanvas *canvas);
	virtual ~Zoomer() {}
	 virtual void rescale();
};

class QwtWindowWatermark : public QwtPlotItem
{
public:
	QwtWindowWatermark();

	void SetPixmap(const QPixmap &pixmap);

	virtual int rtti() const;

	virtual void draw(QPainter *painter,
		const QwtScaleMap &, const QwtScaleMap &,
		const QRect &rect) const;

protected:
	QBrush brush;
};

class QwtModelScaleDraw : public QwtScaleDraw
{
public:
	QwtModelScaleDraw() {amc=0;}
	virtual ~QwtModelScaleDraw() {}
	virtual QwtText label(double value) const;
	void UseModel(AnimatedModel* _amc, QString _columnName) {amc=_amc; columnName=_columnName;}
protected:
	QString columnName;
	AnimatedModel* amc;
};

class QwtWindow : public QMainWindow, public QwtWindowClass, public AnimatedModelList, public ISaveableWindow
{
	Q_OBJECT

public:
	// initialMode: 0=line, 1=histogram, 2=bubble
	QwtWindow(QWidget* parent, Qt::WindowFlags f);
	void InitWithDb(QStringList queries, QStringList databaseNames, int _initialMode);
	~QwtWindow();

	virtual void Save(XMLSaver &xmlSaver);
	virtual void Load(XMLSaver &xmlSaver);
	virtual WindowTypeId GetWindowTypeId(void) const {return WTI_QWT_WINDOW;}
	virtual int GetVersionNumber(void) const {return 0;}
	void Replot(void);
	void RenameTab(QString newTabName, QwtWindowTab *tab);
	QString GetTabName(QwtWindowTab *tab);
	bool HasDefaultTabName(QwtWindowTab *tab);
	QwtPlot *qwtPlot;

	QString GetDBName(void) const;
	virtual AnimatedModel* GetAnimatedModelAtIndex(int index);
	virtual int GetAMCCount(void) const;
	virtual void SetAnimationRowIndices(int from, int to, double valueBefore, double valueAfter);

	//QSqlQueryModelFetchAll & GetModel() {return model;}

	public slots:
		void on_graphTitleLineEdit_textChanged();
		void on_xAxisLineEdit_textChanged();
		void on_yAxisLineEdit_textChanged();
		void on_displayGrid_toggled(bool isSet);
		void on_autoscaleAxes_toggled(bool isSet);
		void on_xAxisMinSpinBox_valueChanged(double d);
		void on_xAxisMaxSpinBox_valueChanged(double d);
		void on_yAxisMinSpinBox_valueChanged(double d);
		void on_yAxisMaxSpinBox_valueChanged(double d);
		void on_backgroundButton_released();
		void on_clearBackgroundButton_released();
		void on_actionAdd_Tab_activated();
		void on_tabWidget_tabCloseRequested(int tabIndex);
		void replot(void);
		void on_addPlotPushButton_released();
		void on_actionEditSQL_triggered(bool b);
		void on_actionEdit_Tick_triggered(bool b);
		void on_leftScaleComboBox_activated(QString string);
		void on_bottomScaleComboBox_activated(QString string);
		void on_xAxisMajorTicksSpinbox_valueChanged(int newValue);
		void on_yAxisMajorTicksSpinbox_valueChanged(int newValue);
		void on_xAxisMinorTicksSpinbox_valueChanged(int newValue);
		void on_yAxisMinorTicksSpinbox_valueChanged(int newValue);
		void on_displayLegendCheckBox_toggled(bool b);
		void onLegendClicked(QwtPlotItem *pi);
		void onZoomed(const QwtDoubleRect &rect);

		void on_actionPoints_toggled(bool) {ResizeContents(this,qwtPlot);}
		void on_actionBackground_toggled(bool) {ResizeContents(this,qwtPlot);}

protected:

	void UpdateFromLeftScaleComboBox( void );
	void UpdateFromBottomScaleComboBox( void );
	void UpdateFromXAxisTicksSpinbox(void);
	void UpdateFromYAxisTicksSpinbox(void);
	void UpdateLegendVisibility();

	void UpdateAll(void);
	void UpdateTitle(void);
	void UpdateXAxisTitle(void);
	void UpdateYAxisTitle(void);
	void UpdateGridVisibility(void);
	void UpdateAxesScale(void);
	QwtWindowTab * AddQwtWindowTabToNewestTabWidget(QString query, QString database);
	virtual void ClearAnimationRowIndices(void);
	void SetBackground(QString fileName);
	QwtWindowTab * AddTab(QString query, QString database);

	QVector<QwtWindowTab*> windowTabs;
	//QSqlQueryModelFetchAll model;
	QwtPlotGrid qpg;
	QwtWindowWatermark watermark;
	int initialMode;
	int newTabIndex;
	QwtPlotMarker markerBefore, markerAfter;
	QString backgroundFilename;
	bool inReplot;

	QwtModelScaleDraw *yLeftScaleDraw, *xBottomScaleDraw;
	QwtLegend *legend;
	Zoomer *zoomer;
};

#endif // QWTWINDOW_H
