#ifndef QWTWINDOWTAB_H
#define QWTWINDOWTAB_H

#include <QWidget>
#include <QVector>
#include "ui_QwtWindowTab.h"
#include "qwt_plot_curve.h"
#include "qwt_legend_item.h"
#include "AnimatedModelContainer.h"

using namespace Ui;

class QwtPlotCurve;
class HistogramItem;
class QwtSymbol;
class QPen;
class QwtPlot;
class QwtWindow;
class QwtPlotItem;
class QwtSymbolCurveLegendItem;
class QwtSymbolCurve;

class QwtSymbolCurveLegendItem : public QwtLegendItem
{
public:
	QwtSymbolCurveLegendItem(const QwtSymbolCurve *_symbolCurve) : symbolCurve(_symbolCurve) {}
	virtual void drawIdentifier(
		QPainter *painter, const QRect &rect) const;

protected:
	const QwtSymbolCurve *symbolCurve;
};


class QwtSymbolCurve : public QwtPlotCurve
{
public:
	virtual QWidget *legendItem() const;
	void setData2(const QwtArray<double> &xData, const QwtArray<double> &yData);
	void setData3(const QwtArray<double> &xData, const QwtArray<double> &yData, const QwtArray<double> &sizeData);
	bool IsUsingSymbolPixelSizes(void) const;
	virtual void drawSymbols(QPainter *painter, const QwtSymbol &symbol,
		const QwtScaleMap &xMap, const QwtScaleMap &yMap,
		int from, int to) const;

protected:
	QwtArray<double> symbolPixelSizes;
};


class QwtWindowTab : public QWidget, public QwtWindowTabClass, public AnimatedModel
{
	Q_OBJECT

public:
	QwtWindowTab(QWidget* parent, Qt::WindowFlags f, QwtWindow *_qwtWindow, QString query, QString database);
	virtual ~QwtWindowTab();
	void DeleteAll();

	void RebuildAll(void);
	void RebuildOnAnimationTick(void);
	void UpdateBasedOnTabWidget(void);

	void Save(XMLSaver &xmlSaver);
	void Load(XMLSaver &xmlSaver);
	bool HasPlotItem(QwtPlotItem *pi) const;
	AnimatedModel *GetAMC(void) {return this;}

public slots:
	void on_xColumnComboBox_activated(QString string);
	void on_yColumnComboBox_activated(QString string);
	void on_colorComboBox_activated(QColor color);
	void on_timeColumnComboBox_activated(QString string);
	void on_tabWidget_currentChanged(int idx); // Line, bar, symbol
	void on_curveStyleComboBox_activated(int index);
	void on_penStyleComboBox_activated(QString string);
	void on_linePenWidthSpinBox_valueChanged(int newValue);
	void on_symbolPenWidthSpinBox_valueChanged(int newValue);
	void on_baselineDoubleSpinBox_valueChanged(double newValue);
	void on_symbolStyleComboBox_activated(int index);
	void on_symbolStyleComboBox_2_activated(int index);
	void on_symbolSizeDoubleSpinBox_valueChanged(double newValue);
	void on_histogramIntervalDoubleSpinBox_valueChanged(double newValue);
	void on_divisionCountSpinBox_valueChanged(int newValue);
	void on_histogramAlignmentComboBox_activated(int index);
	void on_lineAlignmentComboBox_activated(int index);
	void on_symbolMagnitudeComboBox_activated(int index);
	void on_symbolSizeAlgorithmComboBox_activated(int index);
	void on_maxSymbolPixelsSpinBox_valueChanged(int newValue);
	void on_baselineSymbolDoubleSpinBox_valueChanged(double newValue);
	void on_absoluteBaselineCheckbox_toggled(bool isSet);
	void on_windowTabLineEdit_editingFinished();

protected:
	void UpdateDataSource(bool useDivisionCount);
	void BuildPlotCurveFromSettings(void);
	//void BuildSymbolFromSettings(void);
	//void DetachCurveList(void);
	void UpdateCurveTitle(void);
	void PostDBInit(QString query, QString database);
	virtual void SetAnimationRowIndices(int from, int to, double valueBefore, double valueAfter);

	// Only one of these is visible at a time
	QwtSymbolCurve *plotCurve;
	HistogramItem *histogramItem;

	QwtSymbol *symbol;

	QPen *pen;
	QwtPlot *qwtPlot;
	QwtWindow *qwtWindow;
	bool inUpdateDataSource;

//	QList<QwtPlotCurve*> plotList;
//	QList<QwtSymbol> symbolList;
};

#endif // QWTWINDOWTAB_H
