#include "SQLQueryTool.h"
#include "SQLQueryToolHelp.h"
#include "EchoChamber.h"
#include "PieWindow.h"
#include "QwtWindow.h"
#include "ScatterPlotWindow.h"
#include "TableWindow.h"
#include "ImageViewer.h"
#include "SQLQueryToolOutputTab.h"
#include <assert.h>

#define SELECT_TABLES_QUERY "SELECT name FROM sqlite_master WHERE type='table' ORDER BY name;"

extern EchoChamber *echoChamber;

SQLQueryTool::SQLQueryTool(QWidget *parent, Qt::WindowFlags flags)
	: QMainWindow(parent,flags)
{
	setupUi(this);
	sqlEntryTabWidget->clear();
//	QStringList stringList;
//	stringList.append("Query");
//	stringList.append("Result");
//	tableWidget->setVerticalHeaderLabels(stringList);
}

SQLQueryTool::~SQLQueryTool()
{
	QSqlDatabase::removeDatabase(windowTitle());
	DeleteTreeWidget();
	int index;
// 	for (index=0; index < models.count(); index++)
// 		delete models[index];
}
QueryInputTab* SQLQueryTool::CurrentSelectedInputTab(void) const
{
	if (sqlEntryTabWidget->count()>0)
		return (QueryInputTab*) sqlEntryTabWidget->widget(sqlEntryTabWidget->count()-1);
	return 0;
}
QueryInputTab* SQLQueryTool::SelectTabWithDB(QString dbName)
{
	QVector<QueryInputTab*> inputTabs;
	GetInputTabs(inputTabs);
	for (int databaseIndex=0; databaseIndex < inputTabs.count(); databaseIndex++)
	{
		if (inputTabs[databaseIndex]->dbName==dbName)
		{
			sqlEntryTabWidget->setCurrentIndex(databaseIndex);
			return inputTabs[databaseIndex];
		}
	}
	return 0;
}
QueryInputTab* SQLQueryTool::GetTabWithDB(QString dbName)
{
	QVector<QueryInputTab*> inputTabs;
	GetInputTabs(inputTabs);
	for (int databaseIndex=0; databaseIndex < inputTabs.count(); databaseIndex++)
	{
		if (inputTabs[databaseIndex]->dbName==dbName)
			return inputTabs[databaseIndex];
	}
	return 0;
}
QueryOutputTab* SQLQueryTool::GetOutputTabWithQueryAndDb(QString query, QString dbName)
{
	QVector<QueryOutputTab*> outputTabs;
	GetOutputTabs(outputTabs);
	for (int i=0; i < outputTabs.size(); i++)
	{
		if (outputTabs[i]->dbName==dbName && outputTabs[i]->query==query)
			return outputTabs[i];
	}
	return 0;
}
void SQLQueryTool::GetInputTabs(QVector<QueryInputTab*> &inputTabs )
{
	inputTabs.clear();
	for (int databaseIndex=0; databaseIndex < sqlEntryTabWidget->count(); databaseIndex++)
	{
		inputTabs.push_back((QueryInputTab*) sqlEntryTabWidget->widget(databaseIndex));
	}
}
void SQLQueryTool::GetOutputTabs(QVector<QueryOutputTab*> &outputTabs )
{
	outputTabs.clear();
	for (int databaseIndex=0; databaseIndex < tabWidget->count(); databaseIndex++)
	{
		outputTabs.push_back((QueryOutputTab*) tabWidget->widget(databaseIndex));
	}
}
void SQLQueryTool::GetInputQueriesAndDatabases(QStringList &queries, QStringList &databases, bool returnDatabasesWithNoQueries)
{
	databases.clear();
	queries.clear();
	QVector<QueryInputTab*> inputTabs;
	GetInputTabs(inputTabs);
	for (int tabIndex=0; tabIndex < inputTabs.count(); tabIndex++)
	{
		QStringList subqueries;
		QStringList subdatabases;
		SplitQuery(inputTabs[tabIndex]->plainTextEdit.toPlainText(), subqueries, inputTabs[tabIndex]->dbName, subdatabases);
		if (subqueries.size()>0)
		{
			for (int j=0; j < subqueries.size(); j++)
			{
				databases.push_back(inputTabs[tabIndex]->dbName);
				queries.push_back(subqueries[j]);
			}
		}
		else if (returnDatabasesWithNoQueries)
		{
			databases.push_back(inputTabs[tabIndex]->dbName);
			queries.push_back(QString());
		}
	}
}
void SQLQueryTool::GetOutputQueriesAndDatabases(QStringList &queries, QStringList &databases)
{
	databases.clear();
	queries.clear();

	QVector<QueryOutputTab*> outputTabs;
	GetOutputTabs( outputTabs );
	// Last tab is 'messages'
	for (int i=0; i < outputTabs.size()-1; i++)
	{
		databases.push_back(outputTabs[i]->dbName);
		queries.push_back(outputTabs[i]->query);
	}
	
}
void SQLQueryTool::AddDatabase(QString databaseName)
{
	if (databaseName.isEmpty())
		return;
	for (int databaseIndex=0; databaseIndex < sqlEntryTabWidget->count(); databaseIndex++)
	{
		if ( ((QueryInputTab*) sqlEntryTabWidget->widget(databaseIndex))->dbName==databaseName)
			return;
	}

	QueryInputTab *tab = new QueryInputTab();
	tab->setAttribute(Qt::WA_DeleteOnClose, true);
	tab->setObjectName(QString::fromUtf8("tab"));
	tab->dbName=databaseName;
	QString tabName = databaseName;
	int chopIndex;
	chopIndex = tabName.lastIndexOf("/");
	if (chopIndex!=-1)
		tabName.remove(0, chopIndex+1);
	chopIndex = tabName.lastIndexOf("\\");
	if (chopIndex!=-1)
		tabName.remove(0, chopIndex+1);
	sqlEntryTabWidget->addTab(tab, tabName);
	// TODO - this doesn't work
	tab->setToolTip(databaseName);
	QVBoxLayout *tabLayout = new QVBoxLayout(tab);
	tabLayout->setSpacing(0);
	tabLayout->setMargin(0);
	tabLayout->addWidget(&tab->plainTextEdit);

	UpdateTreeView();
	treeWidget->expandToDepth(0);
}
void SQLQueryTool::SetLoadedDatabases(QStringList databaseNames)
{
	QVector<QueryInputTab*> inputTabs;
	GetInputTabs(inputTabs);

	// Remove what is in inputTabs and not in databaseNames
	int inputTabIndex=0;
	while (inputTabIndex < inputTabs.size())
	{
		bool hasDatabaseName=false;
		for (int databaseNamesIndex=0; databaseNamesIndex < databaseNames.size(); databaseNamesIndex++)
		{
			if (inputTabs[inputTabIndex]->dbName==databaseNames[databaseNamesIndex])
			{
				hasDatabaseName=true;
				break;
			}
		}

		if (hasDatabaseName==false)
		{
			inputTabs.remove(inputTabIndex);
		}
		else
		{
			inputTabIndex++;
		}
	}

	// Add what is in databaseNames and not in inputTabs
	for (int databaseNamesIndex=0; databaseNamesIndex < databaseNames.size(); databaseNamesIndex++)
	{
		bool hasInputTab=false;
		for (inputTabIndex=0; inputTabIndex < inputTabs.size(); inputTabIndex++)
		{
			if (inputTabs[inputTabIndex]->dbName==databaseNames[databaseNamesIndex])
			{
				hasInputTab=true;
				break;
			}
		}
		if (hasInputTab==false)
			AddDatabase(databaseNames[databaseNamesIndex]);
	}
}
bool SQLQueryTool::UnloadDB(const QString& dbName)
{
	for (int databaseIndex=0; databaseIndex < sqlEntryTabWidget->count(); databaseIndex++)
	{
		if (((QueryInputTab*) sqlEntryTabWidget->widget(databaseIndex))->dbName==dbName)
		{
			sqlEntryTabWidget->removeTab(databaseIndex);

			int i=0;
			while (i < tabWidget->count()-1)
			{
				QueryOutputTab* tab = (QueryOutputTab*) tabWidget->widget(i);
				if (tab->dbName==dbName)
				{
					tabWidget->removeTab(i);
				}
				else
					i++;
			}

			UpdateTreeView();
			treeWidget->expandToDepth(0);
			return true;
		}
	}
	return false;
}
void SQLQueryTool::SelectTree(int treeIndex)
{
	QTreeWidgetItem *item = treeWidget->topLevelItem(treeIndex);
	item->setSelected(true);
	item->setExpanded(true);
}
void SQLQueryTool::SplitQuery(QString queryIn, QStringList &queriesOut, QString dbName, QStringList &databasesOut)
{
	int index=0;
	QString section;
	do
	{
		section = queryIn.section(';',index,index,QString::SectionSkipEmpty | QString::SectionIncludeTrailingSep);
		if (section.isEmpty()==false)
		{
			queriesOut.push_back(section);
			databasesOut.push_back(dbName);
			index++;
		}
	} while (section.isEmpty()==false);
}
void SQLQueryTool::ExecuteQuery(const QString& query, bool addToLastQueryLineEdit)
{
	if (CurrentSelectedInputTab()==0)
		return;

	QStringList queries;
	QStringList databases;
	SplitQuery(query, queries, CurrentSelectedInputTab()->dbName, databases);

	if (queries.count()==0)
		return;

	ExecuteQueries(queries, databases, addToLastQueryLineEdit);
}
void SQLQueryTool::ExecuteQueries(QStringList queriesUnfiltered, QStringList databaseNamesUnfiltered, bool addToLastQueryLineEdit)
{
	assert(queriesUnfiltered.size()==databaseNamesUnfiltered.size());
	bool isEmptyTree = treeWidget->topLevelItemCount()==0;
	SetLoadedDatabases(databaseNamesUnfiltered);
// 	if (isEmptyTree)
// 		UpdateTreeView();

	QStringList queriesFiltered, databaseNamesFiltered;
	for (int i=0; i < queriesUnfiltered.size(); i++)
	{
		if (queriesUnfiltered[i].isEmpty()==false)
		{
			queriesFiltered.push_back(queriesUnfiltered[i]);
			databaseNamesFiltered.push_back(databaseNamesUnfiltered[i]);
		}
	}

	if (queriesFiltered.size()==0)
		return;

	if (queriesFiltered==lastExecutedQueries && databaseNamesFiltered==lastExecutedDatabaseNames)
		return;

	QueryInputTab* inputTab; 
	for (int dbIdx=0; dbIdx < databaseNamesFiltered.size(); dbIdx++)
	{
		inputTab = SQLQueryTool::GetTabWithDB(databaseNamesFiltered[dbIdx]);
		if (inputTab)
			inputTab->plainTextEdit.clear();
	}
	for (int dbIdx=0; dbIdx < databaseNamesFiltered.size(); dbIdx++)
	{
		inputTab = SQLQueryTool::GetTabWithDB(databaseNamesFiltered[dbIdx]);
		if (inputTab)
		{
			int chopIndex = queriesFiltered[dbIdx].lastIndexOf("\r");
			if (chopIndex!=-1)
				queriesFiltered[dbIdx].remove(0, chopIndex+1);
			chopIndex = queriesFiltered[dbIdx].lastIndexOf("\n");
			if (chopIndex!=-1)
				queriesFiltered[dbIdx].remove(0, chopIndex+1);
			inputTab->plainTextEdit.appendPlainText(queriesFiltered[dbIdx]);
			if (queriesFiltered[dbIdx].endsWith(";", Qt::CaseInsensitive)==false)
				inputTab->plainTextEdit.appendPlainText(";");
		}
	}

	int index=0;
	QVector<QSqlQueryModelFetchAll*> newModels;
	for (index=0; index < queriesFiltered.count(); index++)
	{
		QSqlQueryModelFetchAll *newModel = new QSqlQueryModelFetchAll();
		newModel->setQuery(queriesFiltered[index],QSqlDatabase::database(databaseNamesFiltered[index]));
		QSqlError error = newModel->lastError();
		if (error.isValid()==true)
		{
			for (int index2=0; index2 < newModels.count(); index2++)
				delete newModels[index2];

			QString msg;
			if (queriesFiltered.count()>1)
				msg.sprintf("Query %i: ", index+1);
			msg+=error.databaseText();
			listWidget->addItem(msg);
			listWidget->setCurrentRow(listWidget->count()-1);
			tabWidget->setCurrentIndex(tabWidget->count()-1);
			delete newModel;
			return;
		}

		newModels.push_back(newModel);
	}


	if (addToLastQueryLineEdit)
	{
		for (index=0; index < queriesFiltered.count(); index++)
			echoChamber->AddToLastQueryLineEdit(queriesFiltered[index]);
	}

// 	for (index=0; index < models.count(); index++)
// 		delete models[index];
// 	models.clear();
// 	
	// Reduce to only error tab
	while (tabWidget->count()>1)
		tabWidget->removeTab(tabWidget->count()-2);	

	for (index=0; index < newModels.count(); index++)
	{
		QSqlQueryModelFetchAll *newModel = newModels[index];
	//	models.push_back(newModel);

		QueryOutputTab *tab = new QueryOutputTab();
		tab->setAttribute(Qt::WA_DeleteOnClose, true);
		tab->setObjectName(QString::fromUtf8("tab"));
		tab->dbName=databaseNamesFiltered[index];
		tab->query=newModel->query().lastQuery();

		QString tabName;
		tabName.sprintf("Result %i",index+1);
		tabWidget->insertTab(index, tab, tabName);
		QVBoxLayout *tabLayout = new QVBoxLayout(tab);
		tabLayout->setSpacing(0);
		tabLayout->setMargin(0);
		tabLayout->setContentsMargins(0,0,0,0);
		SQLQueryToolOutputTab *tableView = new SQLQueryToolOutputTab(tab, 0, tab->dbName, tab->query, newModel);
		tabLayout->addWidget(tableView);
		tableView->setAttribute(Qt::WA_DeleteOnClose, true);

		QObject::connect(tableView, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(on_tableView_doubleClicked(QModelIndex)));

#ifndef QT_NO_WHATSTHIS
		tableView->setWhatsThis(QApplication::translate("SQLQueryToolClass", "Result of the currently executed SQL query. Read-only.", 0, QApplication::UnicodeUTF8));
#endif // QT_NO_WHATSTHIS

		tabWidget->setCurrentIndex(0);
	}

	//echoChamber->RegenerateTimelineConstraints();

	lastExecutedQueries=queriesFiltered;
	lastExecutedDatabaseNames=databaseNamesFiltered;
}
void SQLQueryTool::on_updateTreeView_released()
{
	UpdateTreeView();
}
void SQLQueryTool::on_actionHow_Do_I_activated()
{
	SQLQueryToolHelp *sqlQueryToolHelp = new SQLQueryToolHelp(QMessageBox::Information, "SQL Query Tool Help", "Not Done Yet", QMessageBox::Ok, this);
	sqlQueryToolHelp->setAttribute(Qt::WA_DeleteOnClose, true);
	sqlQueryToolHelp->show();
}
void SQLQueryTool::AppendQuery(const QString &query)
{
	QueryInputTab* tab = CurrentSelectedInputTab();
	if (tab)
		tab->plainTextEdit.appendPlainText(query);
}
void SQLQueryTool::on_actionQuery_activated(void)
{
	ExecuteQuery(true);
}
void SQLQueryTool::on_actionNew_activated()
{
	QStringList databases;
	QStringList queries;
	GetOutputQueriesAndDatabases(queries, databases);


	ExecuteQuery(false);
	echoChamber->SpawnSQLQueryTool(queries, databases);
}
void SQLQueryTool::ShowNoQueryMessageBox(void)
{
	QMessageBox::information( this, "No query to display",
		"Type an SQL Query, or click the tree to the left",
		QMessageBox::Ok );
}
// QSqlQueryModelFetchAll* SQLQueryTool::GetActiveModel(void)
// {
// 	if (models.size()==0)
// 	{
// 		ShowNoQueryMessageBox();
// 		return 0;
// 	}
// 	if (models.size()==1)
// 	{
// 		return models[0];
// 	}
// 	// Choose model based on selected tab
// 	if (tabWidget->currentIndex()==tabWidget->count()-1)
// 	{
// 		// Don't know which one to choose!
// 		QMessageBox::warning( this, "Select a result tab",
// 			"Multiple SQL query result tabs are active, but the messages tab is selected. Choose the result tab you want to use.",
// 			QMessageBox::Ok );
// 		return 0;
// 	}
// 	return models[tabWidget->currentIndex()];
// }
void SQLQueryTool::on_actionTable_activated()
{
// 	QSqlQueryModelFetchAll* model = GetActiveModel();
// 	if (model==0)
// 		return;

	TableWindow *tableWindow;
	QStringList databases;
	QStringList queries;
	ExecuteQuery(false);
	GetOutputQueriesAndDatabases(queries, databases);
	if (queries.size()>0 && queries[0].isEmpty()==false)
	{
		tableWindow = echoChamber->SpawnTableWindow(queries, databases );
		echoChamber->SetWindowName(tableWindow, "Table", tableWindow->GetDBName());
	}
	else
		QMessageBox::warning( this, "No queries to display",
		 			"Type an SQL query and press execute, or click on the tree view to generate a query automatically.",
		 			QMessageBox::Ok );
}
void SQLQueryTool::on_actionLine_Graph_activated()
{
// 	QSqlQueryModelFetchAll* model = GetActiveModel();
// 	if (model==0)
// 		return;

	QwtWindow *w;
	QStringList databases;
	QStringList queries;
	ExecuteQuery(false);
	GetOutputQueriesAndDatabases(queries, databases);
	if (queries.size()>0 && queries[0].isEmpty()==false)
	{
		w = echoChamber->SpawnQwtWindow(queries, databases,0);
		echoChamber->SetWindowName(w, "Line Graph", w->GetDBName());
	}
	else
		QMessageBox::warning( this, "No queries to display",
		"Type an SQL query and press execute, or click on the tree view to generate a query automatically.",
		QMessageBox::Ok );
}
void SQLQueryTool::on_actionHistogram_activated()
{
// 	QSqlQueryModelFetchAll* model = GetActiveModel();
// 	if (model==0)
// 		return;

	QwtWindow *w;
	QStringList databases;
	QStringList queries;
	ExecuteQuery(false);
	GetOutputQueriesAndDatabases(queries, databases);
	if (queries.size()>0 && queries[0].isEmpty()==false)
	{
		w = echoChamber->SpawnQwtWindow(queries, databases,1);
		echoChamber->SetWindowName(w, "Histogram", w->GetDBName());
	}
	else
		QMessageBox::warning( this, "No queries to display",
		"Type an SQL query and press execute, or click on the tree view to generate a query automatically.",
		QMessageBox::Ok );
}
void SQLQueryTool::on_actionPie_Chart_activated()
{
// 	QSqlQueryModelFetchAll* model = GetActiveModel();
// 	if (model==0)
// 		return;

	PieWindow *w;
	QStringList databases;
	QStringList queries;
	ExecuteQuery(false);
	GetOutputQueriesAndDatabases(queries, databases);
	if (queries.size()>0 && queries[0].isEmpty()==false)
	{
		w = echoChamber->SpawnPieWindow(queries, databases);
		echoChamber->SetWindowName(w, "Pie Chart", w->GetDBName());
	}
	else
		QMessageBox::warning( this, "No queries to display",
		"Type an SQL query and press execute, or click on the tree view to generate a query automatically.",
		QMessageBox::Ok );
}
void SQLQueryTool::on_actionHeat_Map_activated()
{
// 	QSqlQueryModelFetchAll* model = GetActiveModel();
// 	if (model==0)
// 		return;

	ScatterPlotWindow *w;
	QStringList databases;
	QStringList queries;
	ExecuteQuery(false);
	GetOutputQueriesAndDatabases(queries, databases);
	if (queries.size()>0 && queries[0].isEmpty()==false)
	{
		w = echoChamber->SpawnScatterPlotWindow(queries, databases, true);
		echoChamber->SetWindowName(w, "Heat Map", w->GetDBName());
	}
	else
		QMessageBox::warning( this, "No queries to display",
		"Type an SQL query and press execute, or click on the tree view to generate a query automatically.",
		QMessageBox::Ok );
}
void SQLQueryTool::on_actionBubble_Chart_activated()
{
// 	QSqlQueryModelFetchAll* model = GetActiveModel();
// 	if (model==0)
// 		return;

	QwtWindow *w;
	QStringList databases;
	QStringList queries;
	ExecuteQuery(false);
	GetOutputQueriesAndDatabases(queries, databases);
	if (queries.size()>0 && queries[0].isEmpty()==false)
	{
		w = echoChamber->SpawnQwtWindow(queries, databases, 2);
		echoChamber->SetWindowName(w, "Symbol Chart", w->GetDBName());
	}
	else
	{
		QMessageBox::warning( this, "No queries to display",
		"Type an SQL query and press execute, or click on the tree view to generate a query automatically.",
		QMessageBox::Ok );
	}
}
void SQLQueryTool::on_actionScatter_Plot_activated()
{
// 	QSqlQueryModelFetchAll* model = GetActiveModel();
// 	if (model==0)
// 		return;

	ScatterPlotWindow *w;
	QStringList databases;
	QStringList queries;
	ExecuteQuery(false);
	GetOutputQueriesAndDatabases(queries, databases);
	if (queries.size()>0 && queries[0].isEmpty()==false)
	{
		w = echoChamber->SpawnScatterPlotWindow(queries, databases, false);
		echoChamber->SetWindowName(w, "Scatter Plot", w->GetDBName());
	}
	else
		QMessageBox::warning( this, "No queries to display",
		"Type an SQL query and press execute, or click on the tree view to generate a query automatically.",
		QMessageBox::Ok );
}
void SQLQueryTool::on_actionWizard_activated()
{
	QMessageBox::information(this, "TODO", "Not done yet.");
}
void SQLQueryTool::ExecuteQuery(bool addToLastQueryLineEdit)
{
	QStringList databases;
	QStringList queries;
	GetInputQueriesAndDatabases(queries, databases, false);
	ExecuteQueries(queries, databases, addToLastQueryLineEdit);
}
void SQLQueryTool::SetQuery(QString query)
{
	QueryInputTab* inputTab = CurrentSelectedInputTab();
	if (inputTab)
		inputTab->plainTextEdit.setPlainText(query);
}
void SetParentSelectedRecursive(QTreeWidgetItem* item, bool isSelected)
{
	if (item->parent())
	{
		item->parent()->setSelected(isSelected);
		SetParentSelectedRecursive(item->parent(),isSelected);
	}
}
void SetChildrenSelectedRecursive(QTreeWidgetItem* item, bool isSelected)
{
	for (int i=0; i < item->childCount(); i++)
	{
		item->child(i)->setSelected(isSelected);
		SetChildrenSelectedRecursive(item->child(i),isSelected);
	}
}
void DeselectAll(QTreeWidget *treeWidget)
{
	for (int i=0; i < treeWidget->topLevelItemCount(); i++)
	{
		SetChildrenSelectedRecursive(treeWidget->topLevelItem(i),false);
		treeWidget->topLevelItem(i)->setSelected(false);
	}
}
bool IsSelectedOrChildrenSelected(QTreeWidgetItem* item)
{
	if (item->isSelected())
		return true;
	for (int i=0; i < item->childCount(); i++)
	{
		if (IsSelectedOrChildrenSelected(item->child(i)))
			return true;
	}
	return false;
}
void SQLQueryTool::GenerateQueryFromTreeWidget(QTreeWidgetItem* topLevelItem)
{
	int tablesIndex, columnsIndex;
	QString usedTables;
	QString selectedFields;
	QTreeWidgetItem* table;
	bool anyChildrenSelected;
	int numTablesSelected=0;
	QString query = "SELECT ";
	for (tablesIndex=0; tablesIndex<topLevelItem->childCount(); tablesIndex++)
	{
		if (IsSelectedOrChildrenSelected(topLevelItem->child(tablesIndex)))
			numTablesSelected++;
	}
	for (tablesIndex=0; tablesIndex<topLevelItem->childCount(); tablesIndex++)
	{
		table = topLevelItem->child(tablesIndex);
		if (IsSelectedOrChildrenSelected(table))
		{
			if (usedTables.isEmpty()==false)
				usedTables+=", ";
			usedTables+=table->text(0);

			anyChildrenSelected=false;
			for (columnsIndex=0; columnsIndex<table->childCount(); columnsIndex++)
			{
				if (table->child(columnsIndex)->isSelected())
				{
					if (selectedFields.isEmpty()==false)
						selectedFields+=", ";
					if (numTablesSelected>1)
					{
						selectedFields+=table->text(0);
						selectedFields+=".";
					}
					selectedFields+=table->child(columnsIndex)->text(0);

					anyChildrenSelected=true;
				}
			}

			if (anyChildrenSelected==false)
			{
				if (selectedFields.isEmpty()==false)
					selectedFields+=", ";
				if (numTablesSelected>1)
				{
					selectedFields+=table->text(0);
					selectedFields+=".*";
				}
				else
					selectedFields+="*";
			}

			// If child is selected, parent does not need to be
			//		if (anyChildrenSelected)
			//			table->setSelected(false);
		}
	}

	if (selectedFields.isEmpty())
		return;
	query+=selectedFields;
	query+=" FROM ";
	query+=usedTables;
	if (limitRowsToSpinBox->value()>0)
	{
		query+=" LIMIT ";
		QString valueToNum;
		valueToNum.sprintf("%i", limitRowsToSpinBox->value());
		query+=valueToNum;
	}
	query+=";";

	// ExecuteQuery(query, true);
	// Switch selected input tab to correct database
	QueryInputTab *queryInputTab = SelectTabWithDB(topLevelItem->text(0));
	if (queryInputTab)
	{
		// Set query
		echoChamber->AddToLastQueryLineEdit(query);
		queryInputTab->plainTextEdit.setPlainText(query);
		ExecuteQuery(false);
	}
}
void SQLQueryTool::on_treeWidget_itemClicked(QTreeWidgetItem* widgetItem,int idx)
{
	QTreeWidgetItem* topLevelItem=widgetItem;
	int tablesIndex;
	if (widgetItem->parent()==0)
	{
		for (tablesIndex=0; tablesIndex<treeWidget->topLevelItemCount(); tablesIndex++)
		{
			if (treeWidget->topLevelItem(tablesIndex)!=topLevelItem)
			{
				// Deselect itself, and all children
				SetChildrenSelectedRecursive(treeWidget->topLevelItem(tablesIndex),false);
				treeWidget->topLevelItem(tablesIndex)->setSelected(false);
			}
			else
			{
				// Deselect all children
				SetChildrenSelectedRecursive(treeWidget->topLevelItem(tablesIndex),false);
			}

		}

		QStringList stringList;
		stringList.push_back(widgetItem->text(0));
		SetLoadedDatabases(stringList);
		QStringList stringList2;
		stringList2.push_back(SELECT_TABLES_QUERY);
		ExecuteQueries(stringList2, stringList, true);
		return;
	}

	// Clicking a widget deselects its children and parent
	if (widgetItem->isSelected()==true)
	{
		SetChildrenSelectedRecursive(widgetItem,false);
		SetParentSelectedRecursive(widgetItem,false);
	}

	while (topLevelItem->parent())
	{
		topLevelItem=topLevelItem->parent();
	}
	// Unselect all but topLevelItem (for multiple databases)
	if (topLevelItem)
	{
		for (tablesIndex=0; tablesIndex<treeWidget->topLevelItemCount(); tablesIndex++)
		{
			if (treeWidget->topLevelItem(tablesIndex)!=topLevelItem)
			{
				SetChildrenSelectedRecursive(treeWidget->topLevelItem(tablesIndex),false);
			}
		}
	}
	GenerateQueryFromTreeWidget(topLevelItem);
}
void SQLQueryTool::on_tableView_doubleClicked(QModelIndex modelIndex)
{
	QImage qImage;
	if (EchoChamber::GetImageFromModel(modelIndex, &qImage))
	{
		ImageViewer *imageViewer = echoChamber->SpawnImageViewer();
		echoChamber->SetWindowName(imageViewer, "Image Viewer", QString());
		imageViewer->SetToImage(qImage);
	}
}
void SQLQueryTool::on_limitRowsToSpinBox_editingFinished()
//void SQLQueryTool::on_limitRowsToSpinBox_valueChanged(int v)
{
	if (treeWidget->selectedItems().size()==0)
		return;
	QTreeWidgetItem* topLevelItem=treeWidget->selectedItems()[0];
	while (topLevelItem->parent())
		topLevelItem=topLevelItem->parent();
	GenerateQueryFromTreeWidget(topLevelItem);
}
void SQLQueryTool::UpdateTreeView(void)
{
	DeleteTreeWidget();

	QTreeWidgetItem *treeWidgetItem, *tableWidgetItem, *databaseWidgetItem;
	QIcon tableIcon;
	tableIcon.addFile(QString::fromUtf8("Resources/sql_table.png"), QSize(), QIcon::Normal, QIcon::Off);
	QIcon columnImageIcon;
	columnImageIcon.addFile(QString::fromUtf8("Resources/sql_column_image.png"), QSize(), QIcon::Normal, QIcon::Off);
	QIcon columnTextIcon;
	columnTextIcon.addFile(QString::fromUtf8("Resources/sql_column_text.png"), QSize(), QIcon::Normal, QIcon::Off);
	QIcon columnNumberIcon;
	columnNumberIcon.addFile(QString::fromUtf8("Resources/sql_column_number.png"), QSize(), QIcon::Normal, QIcon::Off);
	QIcon columnNumberUndefined;
	columnNumberUndefined.addFile(QString::fromUtf8("Resources/sql_column_undefined.png"), QSize(), QIcon::Normal, QIcon::Off);
	QIcon databaseIcon;
	databaseIcon.addFile(QString::fromUtf8("Resources/sql_database.png"), QSize(), QIcon::Normal, QIcon::Off);

	const QStringList &databaseNames = echoChamber->GetDatabases();
	unsigned int databaseNameIndex;
	for (databaseNameIndex=0; databaseNameIndex<databaseNames.size(); databaseNameIndex++)
	{
		QSqlDatabase treeViewDb = QSqlDatabase::addDatabase(DATABASE_TYPE, "TEMP");
		treeViewDb.setDatabaseName(databaseNames[databaseNameIndex]);
		treeViewDb.open();
		databaseWidgetItem = new QTreeWidgetItem((QTreeWidget*)0, QStringList(databaseNames[databaseNameIndex]));
		databaseWidgetItem->setIcon(0,databaseIcon);
		treeWidget->addTopLevelItem(databaseWidgetItem);
		databaseWidgetItem->setToolTip(0,databaseNames[databaseNameIndex]);

		QSqlQuery query(treeViewDb);
		query.exec(SELECT_TABLES_QUERY);
		while (query.next())
		{
			QString name = query.value(0).toString();
			tableWidgetItem = new QTreeWidgetItem((QTreeWidget*)0, QStringList(name));
			tableWidgetItem->setIcon(0,tableIcon);
			databaseWidgetItem->addChild(tableWidgetItem);

			QString queryStr = "PRAGMA table_info(";
			queryStr+=name;
			queryStr+=");";
			QSqlQuery query2(treeViewDb);
			query2.exec(queryStr);
			while (query2.next())
			{
				QString columnName = query2.value(1).toString();
				QString columnType = query2.value(2).toString();
				treeWidgetItem = new QTreeWidgetItem((QTreeWidget*)0, QStringList(columnName));
				if (columnType.isEmpty())
					treeWidgetItem->setIcon(0,columnNumberUndefined);
				if (columnType.compare("BLOB",Qt::CaseInsensitive)==0 || columnType.compare("BYTEA",Qt::CaseInsensitive)==0)
					treeWidgetItem->setIcon(0,columnImageIcon);
				else if (columnType.compare("smallint",Qt::CaseInsensitive)==0 ||
					columnType.compare("integer",Qt::CaseInsensitive)==0 ||
					columnType.compare("bigint",Qt::CaseInsensitive)==0 ||
					columnType.compare("decimal",Qt::CaseInsensitive)==0 ||
					columnType.compare("numeric",Qt::CaseInsensitive)==0 ||
					columnType.compare("real",Qt::CaseInsensitive)==0 ||
					columnType.compare("double precision",Qt::CaseInsensitive)==0 ||
					columnType.compare("serial",Qt::CaseInsensitive)==0 ||
					columnType.compare("bigserial",Qt::CaseInsensitive)==0)
					treeWidgetItem->setIcon(0,columnNumberIcon);
				else
					treeWidgetItem->setIcon(0,columnTextIcon);

				tableWidgetItem->addChild(treeWidgetItem);
			}
		}
		QSqlDatabase::removeDatabase("TEMP");
	}	
}
void SQLQueryTool::DeleteTreeWidget(void)
{
	QTreeWidgetItem *item = treeWidget->invisibleRootItem();
	if (item)
	{
		for(int i=item->childCount()-1; i>=0; i--) {
			DeleteTreeWidgetItems(item->child(i));
		}
	}
	treeWidget->clear();
}

void SQLQueryTool::DeleteTreeWidgetItems(QTreeWidgetItem *item)
{
	if (!item) return;
	for(int i=item->childCount()-1; i>=0; i--)
	{
		DeleteTreeWidgetItems(item->child(i));
	}
	delete item;
}
void SQLQueryTool::Save(XMLSaver &xmlSaver)
{
	// this is always first
	SaveType(xmlSaver);
	SaveWindowTitle(xmlSaver,this);
	SaveVersion(xmlSaver);


	QStringList databases;
	QStringList queries;
	GetInputQueriesAndDatabases(queries, databases, true);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, databases);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, queries);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, limitRowsToSpinBox);
	
	QTreeWidgetItem *parent, *child;
	QList<int> indices;
	QList<QTreeWidgetItem*> selectedItems = treeWidget->selectedItems();
	for (int i=0; i < selectedItems.size(); i++)
	{
		xmlSaver.CDDown(xmlSaver.MakeDirectory("SelectedTreeItems"));

		indices.clear();
		child = selectedItems[i];
		parent = child->parent();
		while (parent)
		{
			indices.push_front(parent->indexOfChild(child));
			child=parent;
			parent=child->parent();
		}
		indices.push_front(treeWidget->indexOfTopLevelItem(child));

		xmlSaver.SaveElement("SelectedItem", selectedItems[i]->text(0));
		xmlSaver.SaveElement("Indices", indices);
		
		xmlSaver.CDUp();
	}

	// Screws things up
//	QByteArray geo_data = saveGeometry();
//	QByteArray layout_data = saveState();
//	XML_SAVER_SAVE_ELEMENT(xmlSaver, geo_data);
//	XML_SAVER_SAVE_ELEMENT(xmlSaver, layout_data);
}
void SQLQueryTool::Load(XMLSaver &xmlSaver)
{
	int version;
	LoadVersion(xmlSaver, version);

	QStringList databases;
	QStringList queries;
	XML_SAVER_LOAD_ELEMENT(xmlSaver, databases, false);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, queries, false);
	ExecuteQueries(queries, databases, false);
	
	DeselectAll(treeWidget);

	XML_SAVER_LOAD_ELEMENT(xmlSaver, limitRowsToSpinBox, "1000");

	QList<int> indices;
	int numSelectedItems;
	int index;
	QString itemName;

	if (xmlSaver.CDDown("SelectedTreeItems"))
	{
		do
		{
			xmlSaver.LoadElement("SelectedItem", itemName, "");
			indices.clear();
			xmlSaver.LoadElement("Indices", indices, false);

			if (indices.length() && treeWidget->topLevelItemCount()>indices[0])
			{
				QTreeWidgetItem *treeWidgetItem = treeWidget->topLevelItem(indices[0]);
				for (int j=1; j < indices.length(); j++)
				{
					treeWidgetItem->setExpanded(true);
					if (treeWidgetItem->childCount()<=indices[j])
					{
						treeWidgetItem=0;
						break;
					}
					treeWidgetItem=treeWidgetItem->child(indices[j]);
				}
				if (treeWidgetItem && treeWidgetItem->text(0)==itemName)
					treeWidgetItem->setSelected(true);
			}

		} while (xmlSaver.CDNext("SelectedTreeItems"));
		xmlSaver.CDUp();
	}



	// Screws things up
//	QByteArray geo_data;
//	QByteArray layout_data;
//	XML_SAVER_LOAD_ELEMENT(xmlSaver, geo_data, false);
//	XML_SAVER_LOAD_ELEMENT(xmlSaver, layout_data, false);
//	restoreGeometry(geo_data);
//	restoreState(layout_data);
}
/*
void SQLQueryTool::on_tableView_pressed(QModelIndex modelIdx)
{
	on_tableViewHeader_sectionClicked(modelIdx.column());
}
void SQLQueryTool::on_tableViewHeader_sectionClicked(int idx)
{
//	plainTextEdit->insert(model.record().fieldName(idx));
	QString query = "SELECT (";
	bool isFirstField=true;
	for (int i=0; i < tableView->horizontalHeader()->count(); i++)
	{
		if (1) // if (tableView->horizontalHeader()->isSectionSelected(i))
		{
			if (isFirstField==false)
				query+=", ";
			query+=model->record().fieldName(i);
			isFirstField=false;
		}
	}
	query += ") FROM ";
	query += model->tableName();	
	plainTextEdit->setPlainText(query);
	plainTextEdit->setFocus();
	ExecuteQuery(false);
}
*/
// http://doc.trolltech.com/4.5/qtsql.html

/*
void SQLQueryTool::changeEvent ( QEvent * event )
{
_OutputDebugString("changeEvent\n");
QWidget::changeEvent(event);
}
void SQLQueryTool::focusInEvent(QFocusEvent *e)
{
_OutputDebugString("focusInEvent\n");
QWidget::focusInEvent(e);
emit gotFocus(this);
}
void SQLQueryTool::enterEvent(QEvent *e)
{
_OutputDebugString("enterEvent\n");
QWidget::enterEvent(e);
emit gotFocus(this);
}
void SQLQueryTool::mousePressEvent(QMouseEvent *e)
{
_OutputDebugString("mousePressEvent\n");
QWidget::mousePressEvent(e);
}
*/