#ifndef SQLQUERYTOOL_H
#define SQLQUERYTOOL_H

#include <QMainWindow>
#include "ui_SQLQueryTool.h"
#include <QtSql>
#include <QTableView>
#include <QPlainTextEdit>
#include "ByteArrayAsImageItemDelegate.h"
#include "QSqlQueryModelFetchAll.h"
#include "ISaveableWindow.h"

using namespace Ui;
class EchoChamber;

class QueryInputTab : public QWidget
{
public:
	QString dbName;
	QPlainTextEdit plainTextEdit;
};

class QueryOutputTab : public QWidget
{
public:
	QString dbName;
	QString query;
};

class SQLQueryTool : public QMainWindow, public SQLQueryToolClass, public ISaveableWindow
{
	Q_OBJECT

public:
	SQLQueryTool(QWidget *parent = 0, Qt::WindowFlags flags = 0);
	~SQLQueryTool();
	void SetQuery(QString query);
	void ExecuteQuery(const QString& query, bool addToLastQueryLineEdit);
	void ExecuteQueries(QStringList queries, QStringList databaseNames, bool addToLastQueryLineEdit);
	void AppendQuery(const QString &query);
	void ExecuteQuery(bool addToLastQueryLineEdit);
	void AddDatabase(QString databaseName);
	void SetLoadedDatabases(QStringList databaseNames);
	bool UnloadDB(const QString& dbName);
	void SelectTree(int treeIndex);
	void UpdateTreeView(void);

	virtual void Save(XMLSaver &xmlSaver);
	virtual void Load(XMLSaver &xmlSaver);
	virtual WindowTypeId GetWindowTypeId(void) const {return WTI_SQL_QUERY_TOOL;}
	virtual int GetVersionNumber(void) const {return 0;}

public slots:
	void on_actionHow_Do_I_activated();
	void on_actionQuery_activated();
	void on_actionNew_activated();

	void on_actionTable_activated();
	void on_actionLine_Graph_activated();
	void on_actionHistogram_activated();
	void on_actionPie_Chart_activated();
	void on_actionHeat_Map_activated();
	void on_actionBubble_Chart_activated();
	void on_actionScatter_Plot_activated();
	void on_actionWizard_activated();

	void on_updateTreeView_released();
	void on_treeWidget_itemClicked(QTreeWidgetItem*,int);
	void on_tableView_doubleClicked(QModelIndex modelIndex);
	// valueChanged gives poor performance for big tables, disrupts typing
	// void on_limitRowsToSpinBox_valueChanged(int v);
	void on_limitRowsToSpinBox_editingFinished();

protected:

private:
//	QSqlDatabase db;
	// QSqlQueryModelFetchAll *model;
	//QVector<QSqlQueryModelFetchAll*> models;
	QueryInputTab* CurrentSelectedInputTab(void) const;
	QueryInputTab* SelectTabWithDB(QString dbName);
	QueryInputTab* GetTabWithDB(QString dbName);
	QueryOutputTab* GetOutputTabWithQueryAndDb(QString query, QString dbName);
	void GetInputTabs(QVector<QueryInputTab*> &inputTabs );
	void GetOutputTabs(QVector<QueryOutputTab*> &outputTabs );
	void GetOutputQueriesAndDatabases(QStringList &queries, QStringList &databases);
	void GetInputQueriesAndDatabases(QStringList &queries, QStringList &databases, bool returnDatabasesWithNoQueries);
	QStringList lastExecutedQueries, lastExecutedDatabaseNames;

	QSqlTableModel *m;
	void DeleteTreeWidgetItems(QTreeWidgetItem *item);
	void DeleteTreeWidget(void);
	void ShowNoQueryMessageBox(void);
	void GenerateQueryFromTreeWidget(QTreeWidgetItem* topLevelItem);
	void SplitQuery(QString queryIn, QStringList &queriesOut, QString dbName, QStringList &databasesOut);

//	QSqlQueryModelFetchAll* GetActiveModel(void);
};

#endif // SQLQUERYTOOL_H
