#include "ScatterPlotPointTab.h"
#include "ScatterPlotWindow.h"
#include "EchoChamber.h"
#include <assert.h>
#include <QFileDialog>
#include <QMessageBox>
#include <QtOpenGL/QGLFramebufferObject>

extern EchoChamber *echoChamber;


ScatterPlotPointTab::ScatterPlotPointTab(QWidget* parent, Qt::WindowFlags f, ScatterPlotWindow *_scatterPlotWindow, QString query, QString database)
	: QWidget(parent, f),
	scatterPlotWindow(_scatterPlotWindow),
	pointTextureId( 0 ),
	heatMapColorTexture1( 0 ),
	heatMapColorTexture2( 0 ),
	calculateBoundingBox( true )
{
	assert( scatterPlotWindow ); // Ensure scatterPlotWindow is valid.

	setupUi(this);

	echoChamber->AddDefaultColors(minColorComboBox);
	echoChamber->AddDefaultColors(maxColorComboBox);
	minColorComboBox->setCurrentColor(Qt::black);
	maxColorComboBox->setCurrentColor(Qt::white);
	echoChamber->AddDefaultColors(fixedRGBColorComboBox);

	if (tabWidget->currentIndex()==SPPTCM_COLOR_BY_POSITION || tabWidget->currentIndex()==SPPTCM_USE_CONSISTENT_FIXED_COLOR || tabWidget->currentIndex()==SPPTCM_USE_PER_POINT_COLOR)
		rateOfDiffusionSpinBox->setEnabled(false);
	else
		rateOfDiffusionSpinBox->setEnabled(true);
	

	if (query.size()>0)
	{
		PostDBInit(query, database);
	}
}

ScatterPlotPointTab::~ScatterPlotPointTab()
{
	freeOpenGLResources();
	//DeleteAll();
}
void ScatterPlotPointTab::PostDBInit(QString query, QString database)
{
	echoChamber->InitAMCWithDB(this, query, database );
	bool defaultWasSet;
	echoChamber->SetupIntensityComboBox(GetAMC()->GetModel(), intensityByColumnComboBox);
	echoChamber->SetupRGBComboBox(GetAMC()->GetModel(), rgbByColumnComboBox, &defaultWasSet);
	echoChamber->SetupXYZTimeComboBoxes(GetAMC()->GetModel(), xColumnComboBox, yColumnComboBox, zColumnComboBox, 0);
	xColumnComboBox->addItem("<Zero>");
	yColumnComboBox->addItem("<Zero>");
	zColumnComboBox->addItem("<Zero>");


}
void ScatterPlotPointTab::freeOpenGLResources()
{
	scatterPlotWindow->getGLWidget()->freeTexture( pointTextureId );
	scatterPlotWindow->getGLWidget()->freeTexture( heatMapColorTexture1 );
	scatterPlotWindow->getGLWidget()->freeTexture( heatMapColorTexture2 );
}

void ScatterPlotPointTab::DeleteAll(void)
{

}

float GetNumberDefaultZero(const QSqlQueryModelFetchAll *model, int row, int column)
{
	if (column==-1)
		return 0.0f;

	QVariant variant = model->data(model->index(row, column));
	if (variant.isNull()==false)
		return variant.toDouble();
	else
		return 0;
}
void ScatterPlotPointTab::RebuildAll(void)
{
	echoChamber->SetupXYZTimeComboBoxes(GetAMC()->GetModel(), xColumnComboBox, yColumnComboBox, zColumnComboBox, 0);
	xColumnComboBox->addItem("<Zero>");
	yColumnComboBox->addItem("<Zero>");
	zColumnComboBox->addItem("<Zero>");

	echoChamber->SetupIntensityComboBox(GetAMC()->GetModel(), intensityByColumnComboBox);
	bool defaultWasSet;
	echoChamber->SetupRGBComboBox(GetAMC()->GetModel(), rgbByColumnComboBox, &defaultWasSet);

	calculateBoundingBox=true;
}
void ScatterPlotPointTab::CalculateInputThisFrame(void)
{
	float fixedPointRadius=pointRadiusSpinBox->value();
	float fixedIntensity=intensityFixedSpinBox->value();
	float fixedDiffusion=rateOfDiffusionSpinBox->value();
	QColor minColor=minColorComboBox->currentColor();
	QColor maxColor=maxColorComboBox->currentColor();
	QColor fixedColor=fixedRGBColorComboBox->currentColor();
	
	QSqlQueryModelFetchAll *model = GetAMC()->GetModel();
	int xColumn = EchoChamber::GetColumnIndex(model, xColumnComboBox->currentText());
	int yColumn = EchoChamber::GetColumnIndex(model, yColumnComboBox->currentText());
	int zColumn = EchoChamber::GetColumnIndex(model, zColumnComboBox->currentText());
	int rgbColumn; 
	int intensityColumn;

	if (tabWidget->currentIndex()==0)
	{
		assert(tabWidget->tabText(tabWidget->currentIndex()).toLower()=="pos");
		inputThisFrame.colorMode=SPPTCM_COLOR_BY_POSITION;
		inputThisFrame.pointRadius=pointRadiusSpinBox->value();
		inputThisFrame.pointPositionalMinColor=minColorComboBox->currentColor();
		inputThisFrame.pointPositionalMaxColor=maxColorComboBox->currentColor();
		if (inputThisFrame.pointPositionalMinColor.red() < inputThisFrame.pointPositionalMaxColor.red())
			inputThisFrame.pointPositionalMinColor.setRed(inputThisFrame.pointPositionalMaxColor.red());
		if (inputThisFrame.pointPositionalMinColor.green() < inputThisFrame.pointPositionalMaxColor.green())
			inputThisFrame.pointPositionalMinColor.setGreen(inputThisFrame.pointPositionalMaxColor.green());
		if (inputThisFrame.pointPositionalMinColor.blue() < inputThisFrame.pointPositionalMaxColor.blue())
			inputThisFrame.pointPositionalMinColor.setBlue(inputThisFrame.pointPositionalMaxColor.blue());
	}
	else if (tabWidget->currentIndex()==1)
	{
		assert(tabWidget->tabText(tabWidget->currentIndex()).toLower()=="rgb");
		inputThisFrame.colorMode=SPPTCM_USE_CONSISTENT_FIXED_COLOR;
	}
	else if (tabWidget->currentIndex()==2)
	{
		assert(tabWidget->tabText(tabWidget->currentIndex()).toLower()=="rgb col");
		inputThisFrame.colorMode=SPPTCM_USE_PER_POINT_COLOR;
		rgbColumn=EchoChamber::GetColumnIndex(model, rgbByColumnComboBox->currentText());
	}
	else if (tabWidget->currentIndex()==3)
	{
		assert(tabWidget->tabText(tabWidget->currentIndex()).toLower()=="intensity");
		inputThisFrame.colorMode=SPPTCM_USE_CONSISTENT_HEAT_MAP_INTENSITY;
	}
	else
	{
		assert(tabWidget->tabText(tabWidget->currentIndex()).toLower()=="intensity col");
		inputThisFrame.colorMode=SPPTCM_USE_PER_POINT_HEAT_MAP_INTENSITY;
		intensityColumn=EchoChamber::GetColumnIndex(model, intensityByColumnComboBox->currentText());
	}

	inputThisFrame.numPoints=0;
	inputThisFrame.pointPositionalMinColor=minColor;
	inputThisFrame.pointPositionalMaxColor=maxColor;
	inputThisFrame.pointRadius=fixedPointRadius;
	inputThisFrame.heatMapDiffusionRate=fixedDiffusion;
	inputThisFrame.pointTextureId=pointTextureId;
	inputThisFrame.heatMapColorTexture1=heatMapColorTexture1;
	inputThisFrame.heatMapColorTexture2=heatMapColorTexture2;
//	ColoredPoint coloredPoint;
	QColor color;
	
	int rowStart, rowEnd;
	int tempRowStart;
	GetAMC()->GetAnimationRowIndices(&rowStart, &rowEnd, MAX_POINTS-1);

	tempRowStart=rowStart;
	double elapsedDiffusionTime;
	bool useDiffusion=false;
	float diffusionAlpha;
	double timeAtRow, timeAtStartRow, timeAtEndRow;
	int timeStampColumnIndex;
	if (tempRowStart > 0 && 
		(tabWidget->currentIndex()==SPPTCM_USE_CONSISTENT_HEAT_MAP_INTENSITY || tabWidget->currentIndex()==SPPTCM_USE_PER_POINT_HEAT_MAP_INTENSITY))
	{
		// Go back in milliseconds
		if (rateOfDiffusionSpinBox->value()>0.0)
		{
			timeStampColumnIndex = EchoChamber::GetColumnIndex(GetAMC()->GetModel(), echoChamber->timestampColumnComboBox->currentText());
			if (timeStampColumnIndex!=1)
			{
				GetAMC()->GetDoubleAtIndex(tempRowStart--,timeStampColumnIndex,&timeAtEndRow);
				while (tempRowStart>0)
				{
					GetAMC()->GetDoubleAtIndex(tempRowStart,timeStampColumnIndex,&timeAtRow);
					if (timeAtEndRow - timeAtRow > rateOfDiffusionSpinBox->value())
					{
						GetAMC()->GetDoubleAtIndex(++tempRowStart,timeStampColumnIndex,&timeAtRow);
						break;
					}
					tempRowStart--;
				}

				useDiffusion=timeAtEndRow > timeAtRow;
				if (useDiffusion)
				{
					rowStart=tempRowStart;
					timeAtStartRow=timeAtRow;
				}
			}
		}
	}

	for (int row = rowStart; row < rowEnd; ++row)
	{
		inputThisFrame.points[inputThisFrame.numPoints].point.setX(GetNumberDefaultZero(model, row, xColumn));
		inputThisFrame.points[inputThisFrame.numPoints].point.setY(GetNumberDefaultZero(model, row, yColumn));
		inputThisFrame.points[inputThisFrame.numPoints].point.setZ(GetNumberDefaultZero(model, row, zColumn));

		if (useDiffusion)
		{
			GetAMC()->GetDoubleAtIndex(row,timeStampColumnIndex,&timeAtRow);
			if (timeAtRow >= timeAtEndRow)
				diffusionAlpha=1.0;
			else
				diffusionAlpha = .1f + .9f * ((timeAtRow-timeAtStartRow) / (timeAtEndRow-timeAtStartRow));
		}
		else
			diffusionAlpha=1.0;

		switch (inputThisFrame.colorMode)
		{
		case SPPTCM_COLOR_BY_POSITION:
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[0]=0;
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[1]=0;
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[2]=0;
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[3]=(unsigned char) fixedColor.alpha();
			break;
		case SPPTCM_USE_CONSISTENT_FIXED_COLOR:
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[0]=(unsigned char) fixedColor.red();
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[1]=(unsigned char) fixedColor.green();
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[2]=(unsigned char) fixedColor.blue();
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[3]=(unsigned char) fixedColor.alpha();
			break;
		case SPPTCM_USE_PER_POINT_COLOR:
			{
				QVariant variant = model->data(model->index(row, rgbColumn));
				if (variant.isNull()==false)
					color = QColor(variant.toString());
				else
					continue; // If no color, what can we do?
				if (color.isValid()==false)
					continue; // If no color, what can we do?

				inputThisFrame.points[inputThisFrame.numPoints].perPointColor[0]=(unsigned char) color.red();
				inputThisFrame.points[inputThisFrame.numPoints].perPointColor[1]=(unsigned char) color.green();
				inputThisFrame.points[inputThisFrame.numPoints].perPointColor[2]=(unsigned char) color.blue();
				inputThisFrame.points[inputThisFrame.numPoints].perPointColor[3]=(unsigned char) color.alpha();
			}
			break;
		case SPPTCM_USE_PER_POINT_HEAT_MAP_INTENSITY:
			inputThisFrame.points[inputThisFrame.numPoints].perPointIntensity=GetNumberDefaultZero(model, row, intensityColumn);
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[0]=255.0*inputThisFrame.points[inputThisFrame.numPoints].perPointIntensity;
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[1]=255.0*inputThisFrame.points[inputThisFrame.numPoints].perPointIntensity;
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[2]=255.0*inputThisFrame.points[inputThisFrame.numPoints].perPointIntensity;
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[3]=255.0*diffusionAlpha;

			break;
		case SPPTCM_USE_CONSISTENT_HEAT_MAP_INTENSITY:
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[0]=255.0*fixedIntensity;
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[1]=255.0*fixedIntensity;
			inputThisFrame.points[inputThisFrame.numPoints].perPointColor[2]=255.0*fixedIntensity;
			//inputThisFrame.points[inputThisFrame.numPoints].perPointColor[3]=255.0*diffusionAlpha;
                        inputThisFrame.points[inputThisFrame.numPoints].perPointColor[3]= calculateColorPortion( diffusionAlpha );

			break;
		}

	//	inputThisFrame.points.push_back(coloredPoint);
		inputThisFrame.numPoints++;
	}

	if (calculateBoundingBox)
	{
		boundingBox.reset();
		if (GetAMC()->UseAnimationRowIndices())
		{
			// Animation row indices narrow to subset, have to calculate bounding box among all points
			QPoint3F point;
			for (int row = 0; row < model->realRowCount(); ++row)
			{
				point.setX(GetNumberDefaultZero(model, row, xColumn));
				point.setY(GetNumberDefaultZero(model, row, yColumn));
				point.setZ(GetNumberDefaultZero(model, row, zColumn));
			}
			boundingBox.add(point);
		}
		else
		{
			for (int i = 0; i < inputThisFrame.numPoints; ++i)
			{
				boundingBox.add(inputThisFrame.points[i].point);
			}
		}
		calculateBoundingBox=false;
	}
}
void ScatterPlotPointTab::SetPointTexture(QString fileName)
{
	QImage pointTexture;
	if (pointTexture.load(fileName))
	{
		// Create a texture on the associated OpenGL window from the selected image and update the button icon + state.
		if (scatterPlotWindow->getGLWidget()->createTexture( pointTexture, pointTextureId ))
		{
			QPixmap pixMap = QPixmap::fromImage(pointTexture);
			setTextureButton->setIcon(QIcon(pixMap));
			clearTextureButton->setEnabled(true);
			currentPointTextureFilename=fileName;
		}
		else
			DisplayTextureLoadError(fileName);
	}
	else
		DisplayTextureLoadError(fileName);
}
void ScatterPlotPointTab::ChangeHeatMapTexture1(QString fileName)
{
	QImage pointTexture;
	if (pointTexture.load(fileName))
	{
		// Create a texture on the associated OpenGL window from the selected image and update the button icon + state.
		if (scatterPlotWindow->getGLWidget()->createTexture( pointTexture, heatMapColorTexture1 ))
		{
			QPixmap pixMap = QPixmap::fromImage(pointTexture);
			changeTexture1PushButton->setIcon(QIcon(pixMap));
			heatMapColorTexture1Filename=fileName;
		}
		else
			DisplayTextureLoadError(fileName);
	}
	else
		DisplayTextureLoadError(fileName);
}
void ScatterPlotPointTab::ChangeHeatMapTexture2(QString fileName)
{
	QImage pointTexture;
	if (pointTexture.load(fileName))
	{
		// Create a texture on the associated OpenGL window from the selected image and update the button icon + state.
		if (scatterPlotWindow->getGLWidget()->createTexture( pointTexture, heatMapColorTexture2 ))
		{
			QPixmap pixMap = QPixmap::fromImage(pointTexture);
			changeTexture2PushButton->setIcon(QIcon(pixMap));
			heatMapColorTexture2Filename=fileName;
		}
		else
			DisplayTextureLoadError(fileName);
	}
	else
		DisplayTextureLoadError(fileName);
}
void ScatterPlotPointTab::DisplayTextureLoadError(QString fileName)
{
	if (fileName.isEmpty()==false)
	{
		QMessageBox::warning(this, tr("Texture load failed"), QString("Failed to load ") + fileName + QString("."));
	}
}
void ScatterPlotPointTab::on_setTextureButton_released()
{
	QString fileName = QFileDialog::getOpenFileName(this,
		tr("Open Image"), QString("ScatterPlot"), tr("Image Files (*.png *.jpg *.bmp *.dds)"));
	if (fileName.isEmpty()==false)
	{
		SetPointTexture(fileName);
	}
}
void ScatterPlotPointTab::on_changeTexture1PushButton_released()
{
	QString fileName = QFileDialog::getOpenFileName(this,
		tr("Open Image"), QString("ScatterPlot/HeatMap"), tr("Image Files (*.png *.jpg *.bmp *.dds)"));
	if (fileName.isEmpty()==false)
		ChangeHeatMapTexture1(fileName);
}
void ScatterPlotPointTab::on_changeTexture2PushButton_released()
{
	QString fileName = QFileDialog::getOpenFileName(this,
		tr("Open Image"), QString("ScatterPlot/HeatMap"), tr("Image Files (*.png *.jpg *.bmp *.dds)"));
	if (fileName.isEmpty()==false)
		ChangeHeatMapTexture2(fileName);
}
void ScatterPlotPointTab::on_tabWidget_currentChanged(int idx)
{
	if (tabWidget->currentIndex()==SPPTCM_COLOR_BY_POSITION || tabWidget->currentIndex()==SPPTCM_USE_CONSISTENT_FIXED_COLOR || tabWidget->currentIndex()==SPPTCM_USE_PER_POINT_COLOR)
		rateOfDiffusionSpinBox->setEnabled(false);
	else
		rateOfDiffusionSpinBox->setEnabled(true);
}
void ScatterPlotPointTab::on_clearTextureButton_released()
{
	ClearPointTexture();
}
void ScatterPlotPointTab::ClearPointTexture(void)
{
	if (pointTextureId && scatterPlotWindow && scatterPlotWindow->getGLWidget())
	{
		scatterPlotWindow->getGLWidget()->freeTexture( pointTextureId );
		if (!pointTextureId)
		{
			clearTextureButton->setEnabled(false);
			setTextureButton->setIcon(QIcon());
			currentPointTextureFilename.clear();
		}
	}
}
void ScatterPlotPointTab::on_xColumnComboBox_currentIndexChanged(int newIndex)
{
	calculateBoundingBox=true;
}
void ScatterPlotPointTab::on_yColumnComboBox_currentIndexChanged(int newIndex)
{
	calculateBoundingBox=true;
}
void ScatterPlotPointTab::on_zColumnComboBox_currentIndexChanged(int newIndex)
{
	calculateBoundingBox=true;
}
void ScatterPlotPointTab::Save(XMLSaver &xmlSaver)
{
	AMC_Save(xmlSaver);
	xmlSaver.SaveElement("DBName", GetDBName());
	xmlSaver.SaveElement("Query", GetQuery());

	QByteArray geo_data;
	geo_data = saveGeometry();
	XML_SAVER_SAVE_ELEMENT(xmlSaver, geo_data);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, xColumnComboBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, yColumnComboBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, zColumnComboBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, pointRadiusSpinBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, rateOfDiffusionSpinBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, currentPointTextureFilename);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, heatMapColorTexture1Filename);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, heatMapColorTexture2Filename);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, tabWidget);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, maxColorComboBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, minColorComboBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, fixedRGBColorComboBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, rgbByColumnComboBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, intensityFixedSpinBox);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, intensityByColumnComboBox);
}
void ScatterPlotPointTab::Load(XMLSaver &xmlSaver)
{
	AMC_Load(xmlSaver);
	QString database, query;
	xmlSaver.LoadElement("DBName", database, "");
	xmlSaver.LoadElement("Query", query, "");
	PostDBInit(query, database);

	int iValue;
	double dValue;
	QByteArray geo_data;
	XML_SAVER_LOAD_ELEMENT(xmlSaver, geo_data, false);
	restoreGeometry(geo_data);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, xColumnComboBox, true);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, yColumnComboBox, true);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, zColumnComboBox, true);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, pointRadiusSpinBox, QString::number(pointRadiusSpinBox->value()));
	XML_SAVER_LOAD_ELEMENT(xmlSaver, rateOfDiffusionSpinBox, QString::number(rateOfDiffusionSpinBox->value()));
	QString currentPointTextureFilename,heatMapColorTexture1Filename,heatMapColorTexture2Filename;
	XML_SAVER_LOAD_ELEMENT(xmlSaver, currentPointTextureFilename, "");
	XML_SAVER_LOAD_ELEMENT(xmlSaver, heatMapColorTexture1Filename, "");
	XML_SAVER_LOAD_ELEMENT(xmlSaver, heatMapColorTexture2Filename, "");
	if (currentPointTextureFilename.isEmpty()==false)
		SetPointTexture(currentPointTextureFilename);
	else
		ClearPointTexture();
	if (heatMapColorTexture1Filename.isEmpty()==false)
		ChangeHeatMapTexture1(heatMapColorTexture1Filename);
	if (heatMapColorTexture2Filename.isEmpty()==false)
		ChangeHeatMapTexture2(heatMapColorTexture2Filename);

	XML_SAVER_LOAD_ELEMENT(xmlSaver, tabWidget, 0);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, maxColorComboBox, true);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, minColorComboBox, true);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, fixedRGBColorComboBox, true);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, rgbByColumnComboBox, true);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, intensityFixedSpinBox, 0);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, intensityByColumnComboBox, true);

	if (tabWidget->currentIndex()==SPPTCM_COLOR_BY_POSITION || tabWidget->currentIndex()==SPPTCM_USE_CONSISTENT_FIXED_COLOR || tabWidget->currentIndex()==SPPTCM_USE_PER_POINT_COLOR)
		rateOfDiffusionSpinBox->setEnabled(false);
	else
		rateOfDiffusionSpinBox->setEnabled(true);
}

