#ifndef SCATTERPLOTWINDOW_H
#define SCATTERPLOTWINDOW_H

// GLEW needs to be 1st include where ever OpenGL is used.
#include <GL/glew.h>

#include <QMainWindow>
#include <QWidget>
#include <QSqlQuery>
#include <QTimer>
#include <QGLWidget>
#include "ui_ScatterPlotWindow.h"
#include "QSqlQueryModelFetchAll.h"
#include <QSqlQuery>
#include "QDataTypes3D.h"
#include "ScatterPlotPointTab.h"
#include "WindowAdjustments.h"
#include "AnimatedModelContainer.h"
#include "ISaveableWindow.h"

using namespace Ui;

class ScatterPlotWidget;
class RenderablePoint;
class QFile;
class QGLFramebufferObject;

// Window that contains the controls
class ScatterPlotWindow : public QMainWindow, public ScatterPlotWindowClass, public AnimatedModelList, public ISaveableWindow
{
	Q_OBJECT

public:
	ScatterPlotWindow(QWidget* parent, Qt::WindowFlags f);
	void InitWithDb(QStringList queries, QStringList databaseNames, bool startInHeatMapMode);
	~ScatterPlotWindow();

	//QSqlQueryModelFetchAll & GetModel() {return model;}
	inline int GetPointSourceCount(void) const {return windowTabs.size();}
	// Index goes from 0 to GetPointSourceCount
	inline void CalculateInput(int index) { windowTabs[index]->CalculateInputThisFrame(); }
	inline ScatterPlotFrameInput * GetInputThisFrame(int index) { return windowTabs[index]->GetInputThisFrame(); }
	inline ScatterPlotPointTabColorMode GetColorMode(int index) { return windowTabs[index]->GetColorMode(); }
	inline const QAxisAlignedBoundingBox & GetBoundingBox(int index) const { return windowTabs[ index ]->GetBoundingBox(); }
//	inline GLuint GetHeatMapSampler(int index) {
//		if (windowTabs[index]->tabWidget->currentIndex()==SPPTCM_USE_CONSISTENT_HEAT_MAP_INTENSITY)
//			return windowTabs[index]->GetConsistentHeatMapTexture();
//		return windowTabs[index]->GetPerPointHeatMapTexture();
//	}
//        inline GLuint GetHeatMapSampler(ScatterPlotPointTab * const tab)
//        {
//            return (tab->tabWidget->currentIndex() == SPPTCM_USE_CONSISTENT_HEAT_MAP_INTENSITY) ? tab->GetConsistentHeatMapTexture() : tab->GetPerPointHeatMapTexture();
//        }
	inline ScatterPlotWidget * getGLWidget() const { return scatterPlotWidget; }
	virtual void Save(XMLSaver &xmlSaver);
	virtual void Load(XMLSaver &xmlSaver);
	virtual WindowTypeId GetWindowTypeId(void) const {return WTI_SCATTER_PLOT_WINDOW;}
	virtual int GetVersionNumber(void) const {return 0;}

// 	
// 	virtual AnimatedModel* GetAnimatedModelAtIndex(int index) {return this;}
// 	virtual int GetAMCCount(void) const {return 1;}

	QString GetDBName(void) const;
	virtual AnimatedModel* GetAnimatedModelAtIndex(int index);
	virtual int GetAMCCount(void) const;

public slots:
		// UI callbacks
	//void on_ScatterPlotPointTab_pointRadiusSpinBox_valueChanged(int val);
	void on_actionAdd_Tab_activated();
	void on_pointTabWidget_tabCloseRequested(int tabIndex);
	void on_addPlotPushButton_released();
	
	void on_actionPoints_toggled(bool) {ResizeContents(this,centralWindowWidget);}
	void on_actionBackground_toggled(bool) {ResizeContents(this,centralWindowWidget);}
	void on_actionPlanes_toggled(bool) {ResizeContents(this,centralWindowWidget);}
	void on_actionCamera_toggled(bool) {ResizeContents(this,centralWindowWidget);}
	void on_actionEdit_SQL_triggered(bool b);
	void on_actionEdit_Tick_triggered(bool b);
	void on_backgroundColorCombobox_activated(QColor color);

protected:
	QList<ScatterPlotPointTab*> windowTabs;

	ScatterPlotWidget * const scatterPlotWidget;
	unsigned int newTabIndex;
	//QSqlQueryModelFetchAll model;
	ScatterPlotPointTab * AddScatterPlotPointTabToNewestTabWidget(QString query, QString database);
	ScatterPlotPointTab * AddTab(QString query, QString database);
	virtual void closeEvent(QCloseEvent * event);
	void freeOpenGL();
};

// Draws the 3d
class ScatterPlotWidget : public QGLWidget
{
	Q_OBJECT

public:
	ScatterPlotWidget(QWidget* parent, Qt::WindowFlags f);
	~ScatterPlotWidget();
	virtual void initializeGL();
	virtual void resizeGL( int w, int h );
	virtual void paintGL();
	virtual void mousePressEvent(QMouseEvent *event);
	virtual void mouseDoubleClickEvent(QMouseEvent * event);
	virtual void mouseMoveEvent(QMouseEvent *event);

	inline ScatterPlotWindow * GetParent(void) const { return scatterPlotWindow; }
	int GetPointTabIndex(void) const;

	virtual void Save(XMLSaver &xmlSaver);
	virtual void Load(XMLSaver &xmlSaver);
	void SetBackgroundColor(QColor color);

public slots:
	// Timer callback
	void animationTimerUpdate();


	void on_backgroundButton_released();
	void on_clearBackgroundButton_released();
	void on_showPlanesCheckBox_toggled(bool isSet);
	void on_ticksPerPlaneSpinbox_valueChanged(int curValue);
	void on_saveAngleButton_released();
	void on_xAxisRadioButton_released();
	void on_yAxisRadioButton_released();
	void on_zAxisRadioButton_released();
	void on_perspectiveRadioButton_released();
	void on_userDefinedRadioButton_released();
	void on_userDefinedCameraHeightSpinbox_valueChanged(double d);
	void on_userDefinedCameraAngleSpinbox_valueChanged(double d);

	void on_ScatterPlotPointTab_pointRadiusSpinBox_valueChanged(int val);
	void on_ScatterPlotPointTab_rateOfDiffusionSpinBox_valueChanged(double val);
	void on_ScatterPlotPointTab_xColumnComboBox_currentIndexChanged(int idx);
	void on_ScatterPlotPointTab_yColumnComboBox_currentIndexChanged(int idx);
	void on_ScatterPlotPointTab_zColumnComboBox_currentIndexChanged(int idx);
	void on_ScatterPlotPointTab_tabWidget_currentChanged(int idx); // Pos, Fixed RGB, RGB by column, intensity, intensity by column
	void on_ScatterPlotPointTab_minColorComboBox_activated(QColor color);
	void on_ScatterPlotPointTab_maxColorComboBox_activated(QColor color);
	void on_ScatterPlotPointTab_fixedRGBColorComboBox_activated(QColor color);
	void on_ScatterPlotPointTab_rgbByColumnComboBox_currentIndexChanged(int idx);
	void on_ScatterPlotPointTab_intensityFixedSpinBox_valueChanged(double val);

protected:
	

	QTimer animationTimer;
	QRectF backgroundDims;

	GLuint backgroundTexId;
	GLuint tickId;

	GLuint tickColor;
	bool tickLabelsSupported;

	GLfloat cameraDefaultDistance, cameraDefaultHeight, cameraDefaultAngle;
	GLfloat cameraCurrentDistance, cameraCurrentHeight, cameraCurrentAngle;
	GLfloat cameraPreDragHeight, cameraPreDragAngle;
	int cameraLogDistance, cameraPreDragLogDistance;

	size_t pointCount;

	float * pointArray;
	size_t pointCapacity;

	unsigned int * pointColorArray;
	size_t pointColorCapacity;

	QPoint lastPos;

	ScatterPlotWindow * const scatterPlotWindow;
	// This is so if I change the spinbox in C++, it doesn't select the radio button as if the user had done so.
	bool spinBoxChangedInCode;

	GLuint shaderProgram;
	GLuint shaderVertexObject;
	GLuint shaderFragmentObject;
	int intensitySamplerLocation;
	int heatMapSamplerLocation;
        QGLFramebufferObject * m_framebufferobject;         // Temporary framebuffer used for compositing heat map onto the renderer.

	QString backgroundTextureFilename;
	void LoadBackground(QString fileName);
	void ClearBackground(void);

        GLuint bindShader(GLenum shaderType, const char * pszShaderPath);
	bool prepareShaders();
        void setPointRenderStates(GLfloat pointSize, GLuint textureId);

private:
    /** Set the camera's perspective.
     * param fovY Field-of-view in degrees along the vertical plane of the camera lens.
     * param aspectRatio Aspect ratio of the rendered pixels (square pixels should use w/h).
     * param nearClip Near clipping plane.
     * param farClip Far clipping plane.
     */
    void setPerspective(float fovY, float aspectRatio, float nearClip, float farClip);


    void setCameraOrbit(GLfloat cameraAimX, GLfloat cameraAimY, GLfloat cameraAimZ, GLfloat cameraDistance, GLfloat cameraHeight, GLfloat cameraAngle);
    
    /** Renders the axis ticks. */
    void renderTicks(const QAxisAlignedBoundingBox & tickVolume);

    /** Creates a display list containing the renderable ticks (if tick updates are infrequent, this is the prefered/faster method.) */
    void createTicks(const QAxisAlignedBoundingBox & tickVolume);

    /** Renders the labels for the X,Y,Z axis as well as individual ticks. (Req. OpenGL 2.0+)  */
    void renderLabels(const QAxisAlignedBoundingBox & tickVolume);

    /** Updates the list of point colors based the min/max color values and on their location relative to their bounding box. */
    //void updatePointColors();
    void updateColorsByPosition(ScatterPlotFrameInput & frameInput, const QAxisAlignedBoundingBox & tickVolume);
    void updateColorsByPosition(RenderablePoint * const renderablePointBatch, unsigned int numberOfPoints, const QAxisAlignedBoundingBox & tickVolume);

    /** Returns the distance that tick labels should be from their respective tick locations.  */
    inline float getCurrentTickLabelOffset() const
    {
        return 0.1f;                    // This should be changed to a proper distance attenuated value.
    }

	void calculateInitialDistance(float minCameraDistance);

	float logToDistance(int intLog);

    /** Stress tests the point and color allocator and renderers.
     * param count The number of points (randomly generated) to generate inside the tick volume and throw at the allocator(s) and renderer.
     */
    //void stressTest(size_t count);

public:
    /** Sets the points to plot on the grid.
     * param pointArraySrc Array of 3D points represented as sets if 3 floats (x, y, z).
     * param pointCountSrc Number of 3D points in the array.  The 'pointArraySrc' is expected to have pointCountSrc * 3 number of floats.
     * param copy Determines whether or not this class should make a local copy of the points (slower) or if the passed in array can be used directly.
     * return Success or failure setting the points to be plotted.setTickVolume
     */
    bool setPlotPoints(float * const pointArraySrc, size_t pointCountSrc, bool copy);

	/** Safely creates a texture on the OpenGL context associated with this GLWidget from the passed in QImage.  The existing textureId (if any) is freed.
	 * param qImage QImage to create a texture from.
	 * param textureId The textureId variable that the texture will be created on.
	 * return Success or failure creating the texture.
	 */
	bool createTexture(const QImage & qImage, GLuint & textureId);

	/** Safely deletes an texture from this OpenGL context.
	 * param textureId The texture ID to free.  If successful, textureId will be set to 0.
	 */
	void freeTexture(GLuint & textureId);

	bool createFrameBuffer(QGLFramebufferObject **fbo);
	void freeFrameBuffer(QGLFramebufferObject **fbo);

};

#endif // SCATTERPLOTWINDOW_H
