#include "SettingsScreen.h"
#include "EchoChamber.h"
#include <QTextBlock>
#include "PostfixTableRowQuery.h"

extern EchoChamber *echoChamber;

void SettingsScreen::SetQPlainTextEditFromQStringList(QPlainTextEdit *plainText, const QStringList &stringList)
{
	plainText->clear();
	int i;
	for (i=0; i < stringList.size(); i++)
		plainText->appendPlainText(stringList[i]);
}

void SettingsScreen::SetQStringListFromQPlainTextEdit(QPlainTextEdit *plainText, QStringList &stringList)
{
	stringList.clear();
	QTextDocument *t = plainText->document();
	int numBlockCounts = t->blockCount();
	int i;
	for (i=0; i < numBlockCounts; i++)
	{
		const QTextBlock &block = t->findBlockByNumber(i);
		if (block.text().isEmpty()==false)
			stringList.push_back(block.text());
	}
}

SettingsScreen::SettingsScreen(QWidget* parent, Qt::WindowFlags f, const EchoChamberSettings& existingSettings)
	: QDialog(parent,f)
{
	setupUi(this);
	newSettings=existingSettings;

	SetQPlainTextEditFromQStringList(defaultXColumnsTextEdit, newSettings.stringListSettings[EchoChamberSettings::SLS_X]);
	SetQPlainTextEditFromQStringList(defaultYColumnsTextEdit, newSettings.stringListSettings[EchoChamberSettings::SLS_Y]);
	SetQPlainTextEditFromQStringList(defaultZColumnsTextEdit, newSettings.stringListSettings[EchoChamberSettings::SLS_Z]);
	SetQPlainTextEditFromQStringList(defaultTimeColumnsTextEdit, newSettings.stringListSettings[EchoChamberSettings::SLS_TIME]);
	SetQPlainTextEditFromQStringList(defaultSymbolMagnitudeColumnsTextEdit, newSettings.stringListSettings[EchoChamberSettings::SLS_MAGNITUDE]);
	SetQPlainTextEditFromQStringList(defaultColorColumnPlainTextEdit, newSettings.stringListSettings[EchoChamberSettings::SLS_COLOR]);
	defaultFunctionCallQueryLineEdit->setText(newSettings.functionCallQuery);
	defaultPacketLogQueryLineEdit->setText(newSettings.packetLogQuery);
	defaultNetworkStatisticsQueryLineEdit->setText(newSettings.networkStatisticsQuery);
	postfixTickDefinitionLineEdit->setText(newSettings.postfixTickDefinition);
	showTipsOnStartupCheckbox->setChecked(echoChamber->showTipsOnStartup);

	UpdatePostfixStatusButton();
}

SettingsScreen::~SettingsScreen()
{

}
void SettingsScreen::on_buttonBox_accepted()
{
	SetQStringListFromQPlainTextEdit(defaultXColumnsTextEdit, newSettings.stringListSettings[EchoChamberSettings::SLS_X]);
	SetQStringListFromQPlainTextEdit(defaultYColumnsTextEdit, newSettings.stringListSettings[EchoChamberSettings::SLS_Y]);
	SetQStringListFromQPlainTextEdit(defaultZColumnsTextEdit, newSettings.stringListSettings[EchoChamberSettings::SLS_Z]);
	SetQStringListFromQPlainTextEdit(defaultTimeColumnsTextEdit, newSettings.stringListSettings[EchoChamberSettings::SLS_TIME]);
	SetQStringListFromQPlainTextEdit(defaultSymbolMagnitudeColumnsTextEdit, newSettings.stringListSettings[EchoChamberSettings::SLS_MAGNITUDE]);
	SetQStringListFromQPlainTextEdit(defaultColorColumnPlainTextEdit, newSettings.stringListSettings[EchoChamberSettings::SLS_COLOR]);

	newSettings.functionCallQuery=defaultFunctionCallQueryLineEdit->text();
	newSettings.packetLogQuery=defaultPacketLogQueryLineEdit->text();
	newSettings.networkStatisticsQuery=defaultNetworkStatisticsQueryLineEdit->text();
	echoChamber->showTipsOnStartup=showTipsOnStartupCheckbox->isChecked();
	QList<PostfixTableRowQuery::QueryToken> tokenList;
	if (PostfixTableRowQuery::EvaluateQuery(postfixTickDefinitionLineEdit->text(),tokenList))
		newSettings.postfixTickDefinition=postfixTickDefinitionLineEdit->text();

	echoChamber->UpdateSettings(newSettings);
	close();
}
void SettingsScreen::on_buttonBox_rejected()
{

}
void SettingsScreen::on_postfixTickDefinitionLineEdit_textEdited(QString newString)
{
	UpdatePostfixStatusButton();
}
void SettingsScreen::UpdatePostfixStatusButton(void)
{
	QList<PostfixTableRowQuery::QueryToken> tokenList;
	if (PostfixTableRowQuery::EvaluateQuery(postfixTickDefinitionLineEdit->text(),tokenList))
		postfixTickDefinitionOKToolButton->setIcon(QIcon("Resources/ok.png"));
	else
		postfixTickDefinitionOKToolButton->setIcon(QIcon("Resources/notok.png"));
}
