#include "TableWindow.h"
#include "EchoChamber.h"
#include "PostfixTableRowQuery.h"
#include "QTableWidget"
#include <QBuffer>
#include <QSqlRecord>
#include <QSqlError>
#include <QImageReader>
#include "ByteArrayAsImageItemDelegate.h"
#include "ImageViewer.h"

extern EchoChamber* echoChamber;

TableGLWidget::TableGLWidget(QWidget* parent, Qt::WindowFlags f) : QGLWidget(parent,0,f) , tableWindow((TableWindow*)parent)
{
}
TableGLWidget::~TableGLWidget()
{
	glDeleteTextures(1,&texId2);
}

void TableGLWidget::initializeGL()
{
	glEnable(GL_TEXTURE_2D);						// Enable Texture Mapping ( NEW )
	glShadeModel(GL_SMOOTH);						// Enable Smooth Shading
	glClearColor(1.0f, 1.0f, 1.0f, 0.0f);					// White Background
	glClearDepth(1.0f);							// Depth Buffer Setup
	glEnable(GL_DEPTH_TEST);						// Enables Depth Testing
	glDepthFunc(GL_LEQUAL);							// The Type Of Depth Testing To Do
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST);
	glGenTextures(1, &texId2);
	glBindTexture(GL_TEXTURE_2D, texId2);
}
void TableGLWidget::resizeGL( int w, int h )
{
	if (h==0)										// Prevent A Divide By Zero By
	{
		h=1;										// Making Height Equal One
	}

	width=w; height=h;
	glViewport(0,0,w,h);						// Reset The Current Viewport
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix
	glOrtho(0.0f,width,height,0.0f,-1.0f,1.0f);				// Create Ortho 640x480 View (0,0 At Top Left)
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
}


void TableGLWidget::paintGL()
{
	int rowNumber, rowEnd;
	tableWindow->GetAnimatedModelAtIndex(0)->GetAnimationRowIndices(&rowNumber, &rowEnd, 1);

	int byteArrayColumn, otherColumn;
	QModelIndex indexByteArray;
	QModelIndex indexString;
	QModelIndex index;
	QVariant variant;
	QByteArray ba;
	QImageReader qir;
	QImage qImage;
	QString string;
	QBuffer buffer;

	unsigned int byteArrayIndex=-1;
	unsigned int stringIndex=-1;
	int columnCount = tableWindow->GetAnimatedModelAtIndex(0)->GetModel()->columnCount();
	for (int columnIndex=0; columnIndex < columnCount; columnIndex++)
	{
		QModelIndex index = tableWindow->GetAnimatedModelAtIndex(0)->GetModel()->index(0,columnIndex);	
		QBuffer buffer;
		const QVariant &v = index.data();
		if (v.type()==QVariant::ByteArray)
			byteArrayIndex=columnIndex;
		else if ( tableWindow->GetAnimatedModelAtIndex(0)->GetModel()->record().fieldName(columnIndex)!=echoChamber->GetTimelineColumn())
			stringIndex=columnIndex;
	}

	if (byteArrayIndex!=-1)
	{
		QModelIndex index1 = tableWindow->GetAnimatedModelAtIndex(0)->GetModel()->index(rowNumber,byteArrayIndex);	
		const QVariant &v1 = index1.data();
		ba = v1.toByteArray();;
		buffer.setBuffer(&ba);
		buffer.open(QIODevice::ReadOnly);
		qir.setDevice(&buffer);
		if (qir.read(&qImage)==false)
		{
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			glColor3f(0,0,0);
			renderText(1, 0, "Image read failed", QFont());
			return;
		}
	}

	if (stringIndex!=-1)
	{
		QModelIndex index2 = tableWindow->GetAnimatedModelAtIndex(0)->GetModel()->index(rowNumber,stringIndex);	
		const QVariant &v2 = index2.data();
		string = tableWindow->GetAnimatedModelAtIndex(0)->GetModel()->record().fieldName(stringIndex);
		string += ": ";
		string += v2.toString();
	}

	QImage texture = QGLWidget::convertToGLFormat(qImage);

	glBindTexture(GL_TEXTURE_2D, texId2);
	GLenum texture_format = (QGLFormat::openGLVersionFlags() & QGLFormat::OpenGL_Version_1_2)
		? GL_BGRA : GL_RGBA;

	// Why do I need this for orthographic, but not perspective?
	texture_format=GL_RGBA;

	if (texId2==1)
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, texture.width(), texture.height(), 0, texture_format, GL_UNSIGNED_BYTE, texture.bits());
	else // Seems that 2nd function is faster to update a texture, rather than create a new one
		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, texture.width(), texture.height(), texture_format, GL_UNSIGNED_BYTE, texture.bits());


	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

	int fontHeight,imageOffset;
	QFont font = QApplication::font(this);
	QFontInfo fi(font);
	if (string.isEmpty()==false)
	{
		fontHeight=fi.pixelSize();
		imageOffset=fontHeight+2;
	}
	else
	{
		fontHeight=0;
		imageOffset=0;
	}

	// ----------------

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);			// Clear Screen And Depth Buffer
	//glClear(GL_COLOR_BUFFER_BIT);


	glColor3f(1,1,1);
	glLoadIdentity();							// Reset The Current Matrix
	//glTranslatef(0.0f,0.0f,-5.0f);						// Move Into The Screen 5 Units
	//glTranslatef(0.0f,0.0f,-4.0f);						// Move Into The Screen 5 Units
	glBindTexture(GL_TEXTURE_2D, texId2);				// Select Our Texture
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 1.0f); glVertex3f( 0,  0,  -1.0f);	// Top Left Of The Texture and Quad
	glTexCoord2f(1.0f, 1.0f); glVertex3f( width,  0,  -1.0f);	// Top Right Of The Texture and Quad
	glTexCoord2f(1.0f, 0.0f); glVertex3f( width, height-imageOffset,  -1.0f);	// Bottom Right Of The Texture and Quad
	glTexCoord2f(0.0f, 0.0f); glVertex3f( 0, height-imageOffset,  -1.0f);	// Bottom Left Of The Texture and Quad
	glEnd();



	if (string.isEmpty()==false)
	{
		glColor3f(0,0,0);
		renderText(1, height-2, string, QFont());
	}
}

TableWindow::TableWindow(QWidget* parent, Qt::WindowFlags f)
: QMainWindow(parent, f)
{
	setupUi(this);

}

void TableWindow::InitWithDb(QStringList queries, QStringList databaseNames)
{
	for (int i=0; i < databaseNames.size(); i++)
		echoChamber->InitAMCWithDB(GetAnimatedModelAtIndex(i), queries[i], databaseNames[i]);

	mainToolBar->setEnabled(false);
	mainToolBar->setVisible(false);
	mainLayout->setSpacing(0);
	mainLayout->setMargin(0);

	RegenerateLayout();

	show();
}
TableWindow::~TableWindow()
{

}
void TableWindow::on_postfixTickDefinitionLineEdit_textEdited(QString newString)
{
	UpdatePostfixStatusButton();
}
void TableWindow::UpdatePostfixStatusButton(void)
{
	QList<PostfixTableRowQuery::QueryToken> tokenList;
	if (PostfixTableRowQuery::EvaluateQuery(postfixTickDefinitionLineEdit->text(),tokenList))
	{
		postfixTickDefinitionOKToolButton->setIcon(QIcon("Resources/ok.png"));
		GetAnimatedModelAtIndex(0)->SetTickDefinition(postfixTickDefinitionLineEdit->text());
	}
	else
		postfixTickDefinitionOKToolButton->setIcon(QIcon("Resources/notok.png"));
}
void TableWindow::RegenerateLayout()
{
	int columnCount = GetAnimatedModelAtIndex(0)->GetModel()->columnCount();
	unsigned int byteArrayCount=0;
	unsigned int otherType=0;
	for (int columnIndex=0; columnIndex < columnCount; columnIndex++)
	{
		QModelIndex modelIndex =GetAnimatedModelAtIndex(0)->GetModel()->index(0,columnIndex);	
		QImage qImage;
		if (EchoChamber::GetImageFromModel(modelIndex, &qImage))
		{
			byteArrayCount++;
		}
		else
		{
			QString fn1 = GetAnimatedModelAtIndex(0)->GetModel()->record().fieldName(columnIndex);
			QString fn2 = echoChamber->GetTimelineColumn();
			if (fn1!=fn2)
				otherType++;
		}
	}

	for (int i=0; i < mainLayout->count(); i++)
	{
		mainLayout->itemAt(i)->widget()->close();
	}

	//QVBoxLayout *tabLayout = new QVBoxLayout(this);
	// If exactly 1 image, and 0 or 1 text fields, use video window
	if (byteArrayCount==1 && otherType<=1)
	{
		tableViewWidget=0;
		tableGLWidget = new TableGLWidget(this,0);
		tableGLWidget->setAttribute(Qt::WA_DeleteOnClose, true);
		tableGLWidget->setUpdatesEnabled(true);
		mainLayout->addWidget(tableGLWidget);
	}
	else
	{
		tableGLWidget=0;
		tableViewWidget = new QTableView(this);
		tableViewWidget->setObjectName(QString::fromUtf8("tableView"));
		tableViewWidget->setModel(GetAnimatedModelAtIndex(0)->GetModel());
		tableViewWidget->setAttribute(Qt::WA_DeleteOnClose, true);
		tableViewWidget->setSelectionMode(QAbstractItemView::MultiSelection);
		tableViewWidget->setSelectionBehavior(QAbstractItemView::SelectRows);
		//QObject::connect(tableWidget, SIGNAL(clicked(QModelIndex)), this, SLOT(on_tableView_clicked(QModelIndex)));
		QObject::connect(tableViewWidget->horizontalHeader(), SIGNAL(sectionClicked(int)), this, SLOT(on_tableViewHeader_sectionClicked(int)));
		QObject::connect(tableViewWidget, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(on_tableView_doubleClicked(QModelIndex)));

		//int columnCount = GetAnimatedModelAtIndex(0)->GetModel()->columnCount();
		//tableWidget->setColumnCount(columnCount);
		/*
		for (int columnIndex=0; columnIndex < columnCount; columnIndex++)
		{
			QTableWidgetItem *item = new QTableWidgetItem(GetAnimatedModelAtIndex(0)->GetModel()->record().fieldName(columnIndex));
			tableWidget->setHorizontalHeaderItem(columnIndex, item);

			QTableWidgetItem *item2 = new QTableWidgetItem("");
			tableWidget->setItem(0, columnIndex, item2);
		}
		*/

		ByteArrayAsImageItemDelegate *sqlqid = new ByteArrayAsImageItemDelegate(this);
		tableViewWidget->setItemDelegate(sqlqid);
		tableViewWidget->resizeColumnsToContents();

		mainLayout->addWidget(tableViewWidget);
	}

	sqlQueryLineEdit->setText(GetAnimatedModelAtIndex(0)->GetModel()->query().lastQuery());
	postfixTickDefinitionLineEdit->setText(GetAnimatedModelAtIndex(0)->tableRowQuery.GetTickDefinition());
	UpdatePostfixStatusButton();

	// "SELECT "
	sqlQueryLineEdit->setCursorPosition(7);
}
void TableWindow::on_sqlQueryLineEdit_textEdited(QString string)
{
	
}
void TableWindow::on_sqlQueryLineEdit_returnPressed()
{
	QSqlQueryModel testModel;
	testModel.setQuery(sqlQueryLineEdit->text());
	QSqlError error = testModel.lastError();
	if (error.isValid()==false)
	{
		GetAnimatedModelAtIndex(0)->GetModel()->setQuery(sqlQueryLineEdit->text());
		RegenerateLayout();
	}
	else
	{
		sqlQueryLineEdit->setText(GetAnimatedModelAtIndex(0)->GetModel()->query().lastQuery());
	}
}
void TableWindow::on_tableView_clicked(QModelIndex idx)
{
	//sqlQueryLineEdit->insert(GetAnimatedModelAtIndex(0)->GetModel()->record().fieldName(idx.column()));
}
void TableWindow::on_tableView_doubleClicked(QModelIndex modelIndex)
{
	QImage qImage;
	if (EchoChamber::GetImageFromModel(modelIndex, &qImage))
	{
		ImageViewer *imageViewer = echoChamber->SpawnImageViewer();
		echoChamber->SetWindowName(imageViewer, "Image Viewer", QString());
		imageViewer->SetToImage(qImage);
	}
}
void TableWindow::on_tableViewHeader_sectionClicked(int idx)
{
	sqlQueryLineEdit->insert(GetAnimatedModelAtIndex(0)->GetModel()->record().fieldName(idx));
	sqlQueryLineEdit->setFocus();
}
void TableWindow::SetAnimationRowIndices(int from, int to, double valueBefore, double valueAfter)
{
	AnimatedModel::SetAnimationRowIndices(from,to,valueBefore,valueAfter);
	if (tableViewWidget)
	{
		tableViewWidget->clearSelection();
		tableViewWidget->scrollTo(GetAnimatedModelAtIndex(0)->GetModel()->index(from,0), QAbstractItemView::PositionAtTop);
		for (int i=from; i <=to; i++)
			tableViewWidget->selectRow(i);
	}

	if (tableGLWidget)
	{
		tableGLWidget->updateGL();
	}
}
void TableWindow::ClearAnimationRowIndices(void)
{
	AnimatedModel::ClearAnimationRowIndices();
	if (tableViewWidget)
	{
		tableViewWidget->scrollTo(GetAnimatedModelAtIndex(0)->GetModel()->index(0,0), QAbstractItemView::PositionAtTop);
	}

	if (tableGLWidget)
	{
		tableGLWidget->updateGL();
	}
}
void TableWindow::Save(XMLSaver &xmlSaver)
{
	// this is always first
	SaveType(xmlSaver);
	SaveWindowTitle(xmlSaver,this);
	SaveVersion(xmlSaver);


	QStringList databaseNames;
	QStringList queries;
	databaseNames.push_back(GetAnimatedModelAtIndex(0)->GetDBName());
	queries.push_back(GetAnimatedModelAtIndex(0)->GetModel()->query().lastQuery());
	XML_SAVER_SAVE_ELEMENT(xmlSaver, databaseNames);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, queries);

	QByteArray geo_data = saveGeometry();
	QByteArray layout_data = saveState();
	XML_SAVER_SAVE_ELEMENT(xmlSaver, geo_data);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, layout_data);
	XML_SAVER_SAVE_ELEMENT(xmlSaver, postfixTickDefinitionLineEdit);
}
void TableWindow::Load(XMLSaver &xmlSaver)
{
	int version;
	LoadVersion(xmlSaver, version);
	QStringList databaseNames;
	QStringList queries;
	XML_SAVER_LOAD_ELEMENT(xmlSaver, databaseNames, false);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, queries, false);
	for (int i=0; i < databaseNames.size(); i++)
		echoChamber->InitAMCWithDB(GetAnimatedModelAtIndex(i), queries[i], databaseNames[i]);

	QByteArray geo_data;
	QByteArray layout_data;
	XML_SAVER_LOAD_ELEMENT(xmlSaver, geo_data, false);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, layout_data, false);
	restoreGeometry(geo_data);
	restoreState(layout_data);
	XML_SAVER_LOAD_ELEMENT(xmlSaver, postfixTickDefinitionLineEdit, "");
	//RegenerateLayout();
	UpdatePostfixStatusButton();
	sqlQueryLineEdit->setText(GetAnimatedModelAtIndex(0)->GetModel()->query().lastQuery());
}
