#ifndef TABLEWINDOW_H
#define TABLEWINDOW_H

#include <QMainWindow>
#include <QSqlQuery>
#include "QSqlQueryModelFetchAll.h"
#include <QGLWidget>
#include "ui_TableWindow.h"
#include "AnimatedModelContainer.h"
#include "ISaveableWindow.h"

using namespace Ui;

class TableWindow;
class QTableView;

class TableGLWidget : public QGLWidget
{
public:
	TableGLWidget(QWidget* parent, Qt::WindowFlags f);
	~TableGLWidget();
	virtual void initializeGL();
	virtual void resizeGL( int w, int h );
	virtual void paintGL();

protected:
	TableWindow *tableWindow;
	GLuint texId2;
	GLuint width,height;
};

class TableWindow : public QMainWindow, public TableWindowClass, public AnimatedModelList, public ISaveableWindow, public AnimatedModel
{
	Q_OBJECT

public:
	TableWindow(QWidget* parent, Qt::WindowFlags f);
	void InitWithDb(QStringList queries, QStringList databaseNames);
	virtual ~TableWindow();
	virtual void Save(XMLSaver &xmlSaver);
	virtual void Load(XMLSaver &xmlSaver);
	virtual WindowTypeId GetWindowTypeId(void) const {return WTI_TABLE_WINDOW;}
	virtual int GetVersionNumber(void) const {return 0;}
	void SetAnimationRowIndices(int from, int to, double valueBefore, double valueAfter);

	
	virtual AnimatedModel* GetAnimatedModelAtIndex(int index) {return this;}
	virtual int GetAMCCount(void) const {return 1;}

	public slots:
		void on_postfixTickDefinitionLineEdit_textEdited(QString newString);
		void on_sqlQueryLineEdit_textEdited(QString string);
		void on_sqlQueryLineEdit_returnPressed();
		void on_tableView_clicked(QModelIndex idx);
		void on_tableViewHeader_sectionClicked(int idx);
		void on_tableView_doubleClicked(QModelIndex modelIndex);

protected:
	void UpdatePostfixStatusButton(void);
	void RegenerateLayout();
	//QSqlQueryModelFetchAll model;

	void ClearAnimationRowIndices(void);

	QTableView *tableViewWidget;
	TableGLWidget *tableGLWidget;
};

#endif // TABLEWINDOW_H
