#include "XMLSaver.h"
#include <QFile>
#include <QListWidget>
#include <QLineEdit>
#include "qtcolorcombobox.h"
#include "EchoChamber.h"
#include <QComboBox>
#include <QCheckBox>
#include <QSpinBox>
#include <QDoubleSpinBox>
#include <QRadioButton>
#include <QMessageBox>

extern EchoChamber *echoChamber;

XMLSaver::XMLSaver(QString rootName) : document("EchoChamber1")
{
	document.setNodeValue(rootName);
	root = document.createElement(rootName);
	document.appendChild(root);
}
XMLSaver::~XMLSaver()
{
	loadFile.close();
}
QDomElement XMLSaver::GetElement(QString elementName)
{
	return root.firstChildElement(elementName);
}
QDomElement XMLSaver::MakeDirectory(QString elementName)
{
	QDomElement tag = document.createElement(elementName);
	root.appendChild(tag);
	return tag;
}
void XMLSaver::CDUp(void)
{
	if (root.parentNode().toElement().isNull()==false)
		root=root.parentNode().toElement();
}
bool XMLSaver::CDNext(QString elementName)
{
	if (root.nextSiblingElement(elementName).isNull()==false)
	{
		root=root.nextSiblingElement(elementName);
		return true;
	}
	return false;
}
bool XMLSaver::CDDown(QString elementName)
{
	if (root.firstChildElement(elementName).isNull()==false)
	{
		root=root.firstChildElement(elementName);
		return true;
	}
	return false;
}
bool XMLSaver::CDDown(QDomElement element)
{
	root=element;
	return true;
}
bool XMLSaver::LoadFile(QString filename)
{
	loadFile.setFileName(filename);
	if (!loadFile.open(QFile::ReadOnly)) {
		return false;
	}
	QString errorMsg;
	int errorLine;
	int errorColumn;
	bool contentSet = document.setContent(&loadFile, &errorMsg, &errorLine, &errorColumn);
	if (contentSet==false && errorMsg.isEmpty()==false)
	{
		loadFile.close();

		QString message;
		message="XML Parse error on file " + filename;
		message+=". ";
		message+=errorMsg;
		message+=". ";
		QString lineAndCol;
		lineAndCol.sprintf("errorLine=%i errorColumn=%i.", errorLine, errorColumn);
		message+=lineAndCol;
		QMessageBox::warning(echoChamber, "XML Parse failure", message );
		return false;
	}

	root=document.firstChildElement();

	return true;
}
bool XMLSaver::SaveFile(QString filename)
{
	QFile file(filename);
	if (!file.open(QFile::WriteOnly)) {
		return false;
	}
	file.write(document.toByteArray());
	file.close();
	return true;
}
void XMLSaver::LoadElement(QString elementName, bool &value, bool defaultValue)
{
	QDomElement element = GetElement(elementName);
	value = element.attribute("value", defaultValue==true ? "1" : "0").toInt();
}
void XMLSaver::LoadElement(QString elementName, QCheckBox *value, bool defaultValue)
{
	QDomElement element = GetElement(elementName);
	value->setChecked(element.attribute("value", defaultValue==true ? "1" : "0").toInt()==1 ? true : false);
}
void XMLSaver::LoadElement(QString elementName, int &value, QString defaultValue)
{
	QDomElement element = GetElement(elementName);
	value = element.attribute("value", defaultValue).toInt();
}
void XMLSaver::LoadElement(QString elementName, float &value, QString defaultValue)
{
	QDomElement element = GetElement(elementName);
	value = element.attribute("value", defaultValue).toFloat();
}
void XMLSaver::LoadElement(QString elementName, QStringList &stringList, bool defaultToCurrentValues)
{
	QDomElement element = GetElement(elementName);
	QDomElement listItem = element.firstChildElement();
	if (listItem.isNull()==false)
	{
		stringList.clear();
		while (listItem.isNull()==false)
		{
			QString attribute = listItem.attribute("value");
			stringList.push_back(attribute);
			listItem = listItem.nextSiblingElement();
		}
	}
	else
	{
		if (defaultToCurrentValues==false)
			stringList.clear();
	}
}
void XMLSaver::LoadElement(QString elementName, QList<int> &stringList, bool defaultToCurrentValues)
{
	QDomElement element = GetElement(elementName);
	QDomElement listItem = element.firstChildElement();
	if (listItem.isNull()==false)
	{
		stringList.clear();
		while (listItem.isNull()==false)
		{
			int attribute = listItem.attribute("value", "0").toInt();
			stringList.push_back(attribute);
			listItem = listItem.nextSiblingElement();
		}
	}
	else
	{
		if (defaultToCurrentValues==false)
			stringList.clear();
	}
}
void XMLSaver::LoadElement(QString elementName, QListWidget *stringList, bool defaultToCurrentValues)
{
	QDomElement element = GetElement(elementName);
	QDomElement listItem = element.firstChildElement();
	if (listItem.isNull()==false)
	{
		stringList->clear();
		while (listItem.isNull()==false)
		{
			QString attribute = listItem.attribute("value");
			stringList->insertItem(stringList->count()-1,attribute);
			listItem = listItem.nextSiblingElement();
		}
	}
	else
	{
		if (defaultToCurrentValues==false)
			stringList->clear();
	}
}
void XMLSaver::LoadElement(QString elementName, QtColorComboBox *stringList, bool defaultToCurrentValues)
{
	int i=0;
	QDomElement element = GetElement(elementName);
	int currentIndex = element.attribute("currentIndex", "0").toInt();
	QDomElement listItem = element.firstChildElement();
	if (listItem.isNull()==false)
	{
		stringList->clear();
		stringList->setColorDialogEnabled(false);

		while (listItem.isNull()==false)
		{
			QString color = listItem.attribute("color");
			QString itemText = listItem.attribute("itemText");
			stringList->insertColor(i++,QColor(color),itemText);

			listItem = listItem.nextSiblingElement();
		}

		stringList->setCurrentIndex(currentIndex);
		stringList->setColorDialogEnabled(true);
	}
	else
	{
		if (defaultToCurrentValues==false)
		{
			stringList->clear();
			echoChamber->AddDefaultColors(stringList);
		}
	}
}
void XMLSaver::LoadElement(QString elementName, QString &value, QString defaultValue)
{
	QDomElement element = GetElement(elementName);
	value = element.attribute("value", defaultValue);
}
void XMLSaver::LoadElement(QString elementName, QComboBox *value, bool unused)
{
	QDomElement element = GetElement(elementName);
	QString selectedItem = element.attribute("value", "");
	if (selectedItem.isEmpty()==false)
	{
		int index = value->findText(selectedItem);
		if (index!=-1)
			value->setCurrentIndex(index);
	}
}
void XMLSaver::LoadElement(QString elementName, QLineEdit *value, QString defaultValue)
{
	QDomElement element = GetElement(elementName);
	value->setText(element.attribute("value", defaultValue));
}
void XMLSaver::LoadElement(QString elementName, QByteArray &value, bool defaultToCurrentValues)
{
	QDomElement element = GetElement(elementName);
	if (element.isNull()==false)
	{
		value = QByteArray::fromBase64(element.attribute("value").toAscii());
	}
	else
	{
		if (defaultToCurrentValues==false)
			value.clear();
	}
}
void XMLSaver::LoadElement(QString elementName, QSpinBox *value, QString defaultValue)
{
	QDomElement element = GetElement(elementName);
	value->setValue(element.attribute("value", defaultValue ).toInt());
}
void XMLSaver::LoadElement(QString elementName, QDoubleSpinBox *value, QString defaultValue)
{
	QDomElement element = GetElement(elementName);
	value->setValue(element.attribute("value", defaultValue ).toDouble());
}
void XMLSaver::LoadElement(QString elementName, QTabWidget *value, QString defaultValue)
{
	QDomElement element = GetElement(elementName);
	value->setCurrentIndex(element.attribute("value", defaultValue ).toInt());
}
void XMLSaver::LoadElement(QString elementName, QRadioButton *value, bool defaultValue)
{
	QDomElement element = GetElement(elementName);
	value->setChecked(element.attribute("value", defaultValue==true ? "1" : "0" ).toInt());
}
void XMLSaver::SaveElement(QString elementName, QStringList stringList)
{
	QDomElement element1, element2;
	element1 = document.createElement(elementName);
	root.appendChild(element1);
	element1.setAttribute("type", "QStringList");
	for (int i=0; i < stringList.size(); i++)
	{
		element2 = document.createElement("listItem");
		element1.appendChild(element2);
		element2.setAttribute("value", stringList[i]);
	}
}
void XMLSaver::SaveElement(QString elementName, QList<int> &stringList)
{
	QDomElement element1, element2;
	element1 = document.createElement(elementName);
	element1.setAttribute("type", "QList<int>");
	root.appendChild(element1);
	for (int i=0; i < stringList.size(); i++)
	{
		element2 = document.createElement("listItem");
		element1.appendChild(element2);
		element2.setAttribute("value", stringList[i]);
	}
}
void XMLSaver::SaveElement(QString elementName, QListWidget *stringList)
{
	QDomElement element1, element2;
	element1 = document.createElement(elementName);
	root.appendChild(element1);
	element1.setAttribute("type", "QStringList");
	for (int i=0; i < stringList->count(); i++)
	{
		element2 = document.createElement("listItem");
		element1.appendChild(element2);
		element2.setAttribute("value", stringList->item(i)->text());
	}
}
void XMLSaver::SaveElement(QString elementName, QtColorComboBox *stringList)
{
	QDomElement element1, element2;
	element1 = document.createElement(elementName);
	element1.setAttribute("currentIndex", stringList->currentIndex());
	element1.setAttribute("type", "QtColorComboBox");
	root.appendChild(element1);
	for (int i=0; i < stringList->count(); i++)
	{
		element2 = document.createElement("listItem");
		element1.appendChild(element2);
		element2.setAttribute("color", stringList->color(i).name());
		element2.setAttribute("itemText", stringList->itemText(i));
	}
}
void XMLSaver::SaveElement(QString elementName, bool value)
{
	QDomElement tag = document.createElement(elementName);
	root.appendChild(tag);
	tag.setAttribute("value", (int) value);
	tag.setAttribute("type", "bool");
}
void XMLSaver::SaveElement(QString elementName, QCheckBox *value)
{
	QDomElement tag = document.createElement(elementName);
	root.appendChild(tag);
	tag.setAttribute("value", value->isChecked());
	tag.setAttribute("type", "QCheckBox");
}
void XMLSaver::SaveElement(QString elementName, int value)
{
	QDomElement tag = document.createElement(elementName);
	root.appendChild(tag);
	tag.setAttribute("value", value);
	tag.setAttribute("type", "int");
}
void XMLSaver::SaveElement(QString elementName, float value)
{
	QDomElement tag = document.createElement(elementName);
	root.appendChild(tag);
	tag.setAttribute("value", value);
	tag.setAttribute("type", "float");
}
void XMLSaver::SaveElement(QString elementName, QString value)
{
	QDomElement tag = document.createElement(elementName);
	root.appendChild(tag);
	tag.setAttribute("value", value);
	tag.setAttribute("type", "QString");
}
void XMLSaver::SaveElement(QString elementName, QComboBox *value)
{
	QDomElement tag = document.createElement(elementName);
	root.appendChild(tag);
	tag.setAttribute("value", value->currentText());
	tag.setAttribute("type", "QComboBox");
}
void XMLSaver::SaveElement(QString elementName, QLineEdit *value)
{
	QDomElement tag = document.createElement(elementName);
	root.appendChild(tag);
	tag.setAttribute("value", value->text());
	tag.setAttribute("type", "QLineEdit");
}
void XMLSaver::SaveElement(QString elementName, QByteArray value)
{
	QDomElement tag = document.createElement(elementName);
	QByteArray base64Encoded = value.toBase64();
	QString stringEncoded = QString::fromAscii(base64Encoded.constData(),base64Encoded.length());
	root.appendChild(tag);
	tag.setAttribute("value", stringEncoded);
	tag.setAttribute("type", "QByteArray");
}
void XMLSaver::SaveElement(QString elementName, QSpinBox *value)
{
	QDomElement tag = document.createElement(elementName);
	root.appendChild(tag);
	tag.setAttribute("value", value->value());
	tag.setAttribute("type", "QSpinBox");
}
void XMLSaver::SaveElement(QString elementName, QDoubleSpinBox *value)
{
	QDomElement tag = document.createElement(elementName);
	root.appendChild(tag);
	tag.setAttribute("value", value->value());
	tag.setAttribute("type", "QDoubleSpinBox");
}
void XMLSaver::SaveElement(QString elementName, QTabWidget *value)
{
	QDomElement tag = document.createElement(elementName);
	root.appendChild(tag);
	tag.setAttribute("value", value->currentIndex());
	tag.setAttribute("type", "QTabWidget");
}
void XMLSaver::SaveElement(QString elementName, QRadioButton *value)
{
	QDomElement tag = document.createElement(elementName);
	root.appendChild(tag);
	tag.setAttribute("value", value->isChecked());
	tag.setAttribute("type", "QRadioButton");
}
